package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImUserModel;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;
import java.util.Set;

public interface ClassGroupDao extends BaseDAO<Integer, ClassGroup> {


    /**
     * 查询老师乐团课课程
     *
     * @param teacherId
     * @return
     */
    List<ClassGroup> findClassGroup4Teacher(@Param("teacherId") Integer teacherId);

    /**
     * 获取所有合奏班
     *
     * @return
     */
    List<ClassGroup> findAllMixClassGroup(@Param("musicGroupId") String musicGroupId);

    /**
     * @return java.util.List<com.ym.mec.biz.dal.entity.ClassGroup>
     * @Author: Joburgess
     * @Date: 2019/10/8
     * @params [musicGroupId]
     * @describe 获取所有班级
     */
    List<ClassGroup> findAllClassGroupByMusicGroup(@Param("musicGroupId") String musicGroupId);

    /**
     * 根据合奏班ID查询该合奏班下的子班级
     *
     * @param mixClassGroupId
     * @return
     */
    List<ClassGroup> findMixClassChildClassGroup(@Param("mixClassGroupId") Integer mixClassGroupId);

    /**
     * 根据乐团编号查询单技班级信息
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findAllNormalClassGroupByMusicGroupId(@Param("musicGroupId") String musicGroupId);

    /**
     * 查询合奏班包含子班的名字信息
     *
     * @param classGroupId
     * @return
     */
    ClassGroup findMixClassChildClassGroupNames(@Param("classGroupId") int classGroupId);

    /**
     * 跟班级ids查询班级信息
     *
     * @param ids
     * @return
     */
    List<ClassGroup> findClassGroupByIds(@Param("ids") String ids);

    /**
     * 根据教师编号获取教师所在乐团，带了哪些班级
     *
     * @param params
     * @return
     */
    List<TeacherMusicClassInfoDto> queryGroupCourses(Map<String, Object> params);

    /**
     * COUNT教师所在乐团带了哪些班级
     *
     * @param params
     * @return
     */
    int countGroupCourses(Map<String, Object> params);


    /**
     * 获取乐团所有班级列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroupTeachersDto> findClassGroupByMusicGroupId(@Param("musicGroupId") String musicGroupId, @Param("type") String type, @Param("classGroupId") Integer classGroupId);

    /**
     * 获取学员所在群组信息
     *
     * @param userId
     * @param search
     * @return
     */
    List<ImGroupModel> queryUserGroups(@Param("userId") Integer userId, @Param("search") String search);

    /**
     * 获取vip课的学生名称列表
     *
     * @param vipIds
     * @return
     */
    List<Map<String, String>> findVipStuNames(String vipIds);

    /**
     * 获取班级下的学员列表
     *
     * @param classGroupId
     * @return
     */
    List<ImUserModel> findGroupUsers(Integer classGroupId);

    /**
     * count学员购买列表
     *
     * @param params
     * @return
     */
    int countStudentPayLog(Map<String, Object> params);

    /**
     * 分页查询学员购买列表
     *
     * @param params
     * @return
     */
    List<StudentPayInfoDto> queryStudentPayLog(Map<String, Object> params);

    /**
     * @return int
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [classGroupId]
     * @describe 统计班级人数
     */
    int countClassStudent(Long classGroupId);

    /**
     * 班级增加学生人数
     *
     * @param classGroup
     * @return
     */
    int addStudentNum(@Param("classGroup") ClassGroup classGroup);

    /**
     * 获取当前班级老师id列表
     *
     * @param id
     * @return
     */
    Set<Integer> queryTeacherIds(String id);

    /**
     * 获取当前班级老师数量
     *
     * @param classGroupId
     * @return
     */
    Integer countTeacherNum(String classGroupId);

    /**
     * 获取老师所在班级群组列表
     *
     * @param userId
     * @param search
     * @return
     */
    List<ImGroupModel> queryTeacherGroups(@Param("userId") Integer userId, @Param("search") String search);

    /**
     * 获取未分配合奏的单技班列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findNoClassSubjects(String musicGroupId);

    /**
     * @return com.ym.mec.biz.dal.entity.ClassGroup
     * @Author: Joburgess
     * @Date: 2019/10/16
     * @params [vipGroupId]
     * @describe 根据小课获取关联的班级信息
     */
    ClassGroup findByVipGroup(Long vipGroupId);

    /**
     * @return com.ym.mec.biz.dal.entity.ClassGroup
     * @Author: Joburgess
     * @Date: 2019/10/16
     * @params [classGroupId]
     * @describe 锁定班级信息
     */
    ClassGroup lockClassGroup(Long classGroupId);

    /**
     * 获取当前班级列表老师数量
     *
     * @param classGroupIds
     * @return
     */
    List<Map<Integer, Integer>> countTeacherNums(@Param("classGroupIds") String classGroupIds);

    /**
     * @return java.util.List<com.ym.mec.biz.dal.dto.TeacherClassGroupDto>
     * @Author: Joburgess
     * @Date: 2019/10/17
     * @params [teacherId]
     * @describe 获取老师乐团班级
     */
    List<TeacherClassGroupDto> findTeacherMusicClassGroup(@Param("teacherId") Long teacherId);

    /**
     * 根据老师编号列表获取小课数量
     *
     * @param teacherIds
     * @return
     */
    List<Map<Integer, Integer>> countTeacherVipNum(String teacherIds);

    /**
     * @return java.util.List<com.ym.mec.biz.dal.dto.TeacherClassGroupDto>
     * @Author: Joburgess
     * @Date: 2019/10/17
     * @params [teacherId]
     * @describe 获取老师vip课班级
     */
    List<TeacherClassGroupDto> findTeacherVipClassGroup(@Param("teacherId") Long teacherId);

    /**
     * 根据合奏班编号列表，获取单技班名称列表
     *
     * @param classGroupIds
     * @return
     */
    List<Map<Integer, String>> queryNamesByClassId(@Param("classGroupIds") String classGroupIds);

    /**
     * @param classGroupId: 班级编号
     * @return java.util.List<java.lang.Integer>
     * @describe 查询班级下所有学生的部门编号列表
     * @author Joburgess
     * @date 2019/10/21
     */
    List<Integer> findStudentOrganIdsByClassGroup(Long classGroupId);

    /**
     * 乐团详情--班级详情列表
     *
     * @param params
     * @return
     */
    List<MusicGroupClassGroupDto> queryMusicGroupClassGroup(Map<String, Object> params);

    /**
     * COUNT乐团详情--班级详情列表
     *
     * @param params
     * @return
     */
    int countMusicGroupClassGroup(Map<String, Object> params);

    /**
     * 根据班级编号列表，获取老师名字
     *
     * @param classGroupIds
     * @return
     */
    List<Map<Integer, String>> countStudentNum(@Param("classGroupIds") String classGroupIds, @Param("teacherRole") String teacherRole);

    /**
     * 根据班级编号列表，获取班级在读人数
     *
     * @param classGroupIds
     * @return
     */
    List<Map<Integer, Integer>> countStudyNum(@Param("classGroupIds") String classGroupIds);

    /**
     * 根据班级编号列表，获取班级退班人数
     *
     * @param classGroupIds
     * @return
     */
    List<Map<Integer, Integer>> countQuitNum(@Param("classGroupIds") String classGroupIds);

    /**
     * 获取乐团详情---课表详情列表
     *
     * @param params
     * @return
     */
    List<MusicGroupCourseScheduleDto> queryMusicGroupCourseSchedule(Map<String, Object> params);

    /**
     * COUNT乐团详情---课表详情列表
     *
     * @param params
     * @return
     */
    int countMusicGroupCourseSchedule(Map<String, Object> params);

    /**
     * 根据班级编号列表获取班级名称
     *
     * @param classGroupIds
     * @return
     */
    List<Map<Integer, String>> findNameById(String classGroupIds);

    /**
     * 获取学员班级列表
     *
     * @param musicGroupId
     * @param teacherId
     * @return
     */
    List<ConditionDto> queryStudentClassGroup(@Param("musicGroupId") String musicGroupId, @Param("teacherId") Integer teacherId);

    /**
     * 批量新增班级上的课次数
     *
     * @param classGroupList
     * @return
     */
    int batchIncreaseClassTimes(List<Integer> classGroupList);

    /**
     * 获取学生端，vip课程列表
     *
     * @param musicGroupIds
     * @return
     */
    List<CourseListDto> queryStudentVipCourses(@Param("musicGroupIds") String musicGroupIds,@Param("userId") Integer userId);

    /**
     * 获取学生端,学员乐团课列表详情
     *
     * @param musicGroupIds
     * @return
     */
    List<CourseListDto> queryStudentMusicGroupCourses(@Param("musicGroupIds") String musicGroupIds,@Param("userId") Integer userId);

    /**
     * 查询对应类型所有班级列表
     *
     * @param musicGroupId
     * @param type
     * @return
     */
    List<ClassGroup> findClassGroupByMusicGroupIdAndType(@Param("musicGroupId") String musicGroupId, @Param("type") ClassGroupTypeEnum type);

    /**
     * 根据合奏班id获取子班及老师
     *
     * @param mixClassGroupId
     * @return
     */
    List<ClassGroupTeachersDto> findClassGroupByMixClassGroupId(@Param("mixClassGroupId") Integer mixClassGroupId);

    /**
     * @param classGroups: 班级编号列表
     * @return java.util.List<java.lang.String>
     * @describe 根据班级编号列表获取班级名称列表
     * @author Joburgess
     * @date 2019/10/31
     */
    List<String> findClassGroupNamesByClassGroups(@Param("classGroups") List<Integer> classGroups);

    /**
     * 获取当前班级相关的老师列表
     *
     * @param groupId
     * @return
     */
    List<ImUserModel> findGroupTeacher(Integer groupId);

    /**
     * @param courseScheduleIds: 课程编号列表
     * @return com.ym.mec.biz.dal.entity.ClassGroup
     * @describe 根据课程获取班级信息
     * @author Joburgess
     * @date 2019/11/4
     */
    List<ClassGroup> findByCourseSchedules(@Param("courseScheduleIds") List<Long> courseScheduleIds);

    /**
     * @param courseScheduleId: 课程编号
     * @return com.ym.mec.biz.dal.entity.ClassGroup
     * @describe 根据课程获取班级信息
     * @author Joburgess
     * @date 2019/11/4
     */
    ClassGroup findByCourseSchedule(Integer courseScheduleId);

    /**
     * @param maxCourseScheduleId: 合奏班编号
     * @return java.util.List<java.lang.Integer>
     * @describe 获取合奏班下所有子班级
     * @author Joburgess
     * @date 2019/11/5
     */
    List<ClassGroup> findClassGroupsByMixClassGroup(Integer maxCourseScheduleId);

    /**
     * 根据乐团跟声部获取提高班列表
     *
     * @param musicGroupId
     * @param subjectId
     * @return
     */
    List<HighClassGroupDto> findClassGroupByMusicGroupIdAndSubjectId(@Param("musicGroupId") String musicGroupId, @Param("subjectId") Integer subjectId);

    int batchUpdateClassGroupCourseTimes(@Param("classGroupCourseTimes") Map<Integer,Integer> classGroupCourseTimes);
}
