package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.ClassGroupTeachersDto;
import com.ym.mec.biz.dal.entity.ClassGroupTeacherMapper;
import com.ym.mec.biz.dal.enums.TeachTypeEnum;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface ClassGroupTeacherMapperDao extends BaseDAO<Long, ClassGroupTeacherMapper> {


    /**
     * 批量插入班级老师
     *
     * @param classGroupTeacherMapperList
     * @return
     */
    int classGroupTeachersInsert(@Param("classGroupTeacherMapperList") List<ClassGroupTeacherMapper> classGroupTeacherMapperList);

    /**
     * 查询班级老师
     *
     * @param classGroupIds
     * @return
     */
    List<ClassGroupTeacherMapper> findClassGroupTeachers(@Param("classGroupIds") String classGroupIds);

    /**
     * 批量删除班级的老师关联关系
     *
     * @param classGroupTeacherMapperList
     * @return
     */
    int delMusicGroupTeacherMapper(@Param("classGroupTeacherMapperList") List<ClassGroupTeacherMapper> classGroupTeacherMapperList);

    /**
     * @param classGroupId:班级编号
     * @param teacherRole:
     * @return com.ym.mec.biz.dal.entity.ClassGroupTeacherMapper
     * @Author: Joburgess
     * @Date: 2019/10/18
     * @describe 根据班级编号和教师类型获取老师信息
     */
    ClassGroupTeacherMapper findByClassGroupAndRole(@Param("classGroupId") Long classGroupId,
                                                    @Param("teacherRole") TeachTypeEnum teacherRole);

    /**
     * @param musicGroupId:
     * @return java.util.List<com.ym.mec.biz.dal.entity.ClassGroupTeacherMapper>
     * @describe 获取乐团下的所有主教老师
     * @author Joburgess
     * @date 2019/10/18
     */
    List<ClassGroupTeacherMapper> findByMusicGroup(String musicGroupId);

    /**
     * 批量查询班级的老师关联关系
     *
     * @param classGroupTeacherMapperList
     * @return
     */
    List<ClassGroupTeacherMapper> getClassGroupTeacherMapper(@Param("classGroupTeacherMapperList") List<ClassGroupTeacherMapper> classGroupTeacherMapperList);
}