package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.StudentAttendanceStatisticsResponse;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.CourseScheduleStudentPayment;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface CourseScheduleStudentPaymentDao extends BaseDAO<Long, CourseScheduleStudentPayment> {

    /**
     * @return int
     * @Author: Joburgess
     * @Date: 2019/10/10
     * @params [courseScheduleStudentPayments]
     * @describe 批量插入
     */
    int batchInsert(List<CourseScheduleStudentPayment> courseScheduleStudentPayments);

    /**
     * @return java.util.List<com.ym.mec.biz.dal.entity.CourseScheduleStudentPayment>
     * @Author: Joburgess
     * @Date: 2019/10/15
     * @params [ids]
     * @describe 根据课程编号批量获取学生缴费信息
     */
    List<CourseScheduleStudentPayment> findByCourseScheduleIds(@Param("ids") List<Long> ids);

    /**
     * 删除学生对应的课程
     * @param userId
     * @param courseScheduleList
     * @return
     */
    int deleteStudentCourseSchedule(@Param("userId") Integer userId, @Param("courseScheduleList") List<CourseSchedule> courseScheduleList);

    /**
     * @describe 批量删除课程对应的学生缴费记录
     * @author Joburgess
     * @date 2019/10/31
     * @param courseScheduleIds: 课程编号列表
     * @return int
     */
    int deleteByCourseSchedule(@Param("courseScheduleIds") List<Long> courseScheduleIds);

    /**
     * @describe 获取指定班级中的学生
     * @author Joburgess
     * @date 2019/10/28
     * @param params: 班级编号
     * @return java.util.List<com.ym.mec.biz.dal.dto.StudentAttendanceStatisticsResp>
     */
    List<StudentAttendanceStatisticsResponse> findStudentByClassGroup(Map<String,Object> params);
    int countStudentByClassGroup(Map<String,Object> params);
}
