package com.ym.mec.biz.dal.dao;

import com.ym.mec.auth.api.entity.SysUserRole;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.dto.MusicCardDto;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImUserModel;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Set;

public interface MusicGroupDao extends BaseDAO<String, MusicGroup> {
	
	MusicGroup getLocked(String id);

    /**
     * 获取学员所在乐团详情列表
     * @param userId
     * @return
     */
    List<MusicCardDto> queryUserMusicGroups(Integer userId);

    /**
     * 获取个人中心乐团数据
     * @param userId
     * @return
     */
    MusicCardDto queryPersonalMusicGroups(@Param("userId") Integer userId);

    /**
     * 获取用户有效的乐团数
     * @param userId
     * @return
     */
    Integer countUserMusicGroups(Integer userId);

    /**
     * 根据课程类型获取剩余课时
     * @param userId
     * @param type
     * @return
     */
    Integer countCourseByType(@Param("userId") Integer userId, @Param("type") ClassGroupTypeEnum type);

    /**
     * 根绝学员编号获取相关老师列表
     * @param studentId
     * @return
     */
    List<BasicUserDto> findTeachersByStuId(Integer studentId);

    /**
     * 获取学员所在乐团
     * @param userId
     * @return
     */
    List<ImGroupModel> queryUserGroups(@Param("userId") Integer userId,@Param("search") String search);

    /**
     * 获取乐团总人数
     * @param musicGroupIds
     * @return
     */
    List<Map<String, BigDecimal>> countMusicGroupUserNum(@Param("musicGroupIds") String musicGroupIds);

    /**
     * 获取乐团所有学员基本信息
     * @param musicGroupId
     * @return
     */
    List<ImUserModel> findGroupUsers(String musicGroupId);

    /**
     * 获取教师所在乐团列表
     * @param userId
     * @param search
     * @return
     */
    List<ImGroupModel> queryTeacherGroups(@Param("userId") Integer userId, @Param("search")String search);

    /**
     * 获取当前乐团所有教师列表
     * @param musicGroupId
     * @return
     */
    Set<Integer> queryTeacherIds(String musicGroupId);

    /**
     * 根据乐团编号列表获取乐团在读人数map
     * @param musicGroupIds
     * @return
     */
    List<Map<String, Long>> countPayNum(@Param("musicGroupIds") String musicGroupIds);

    /**
     * 获取乐团列表
     * @param musicGroupIds
     * @return
     */
    List<MusicGroup> queryListByIds(@Param("musicGroupIds") String musicGroupIds);

    /**
     * @describe 根据乐团获取对应的结算方式
     * @author Joburgess
     * @date 2019/11/3
     * @param musicGroupIds: 乐团编号列表
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.String>>
     */
    List<Map<String,String>> findSettlementTypeByMusicGroups(@Param("musicGroupIds") List<String> musicGroupIds);
    
    /**
     * 查询首页的乐团数量
     * @param ogranId
     * @return
     */
    Map<String,Integer> queryOrganMusicGroupNum(@Param("organId") Integer ogranId);
    
    /**
     * 查询本月乐团课程数
     * @param organId
     * @return
     */
    Integer queryCurrentMonthCoursesNum(@Param("organId") Integer organId);

    /**
     * 获取乐团下的教师map
     * @param musicGroupId
     * @return
     */
    List<Map<Integer,String>> queryTeacherMap(String musicGroupId);

    /**
     * 获取乐团下的教师map
     * @param musicGroupId
     * @return
     */
    List<Map<Integer,String>> queryTeacherIdMap(String musicGroupId);

    /**
     * 根据角色编号获取有权限的用户列表
     * @param roleIds
     * @return
     */
    Set<Integer> queryUserIdByRoleId(@Param("roleIds") Set<Integer> roleIds);
}
