package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.entity.MusicGroupStudentFee;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface MusicGroupStudentFeeDao extends BaseDAO<Long, MusicGroupStudentFee> {

	/**
	 * 根据乐团编号和学员编号获取乐团费用信息
	 * @param userId
	 * @return
	 */
	MusicGroupStudentFee findByUser(@Param("userId") Integer userId, @Param("musicGroupId") String musicGroupId);

	/**
	 * @describe 根据学生编号和乐团编号更新旷课次数
	 * @author Joburgess
	 * @date 2019/10/19
	 * @param userId: 用户编号
	 * @param musicGroupId: 乐团编号
	 * @param operate: 0重置,1加1
	 * @return int
	 */
	int updateStudentAbsenteeismTimes(@Param("userId") Integer userId, @Param("musicGroupId") String musicGroupId, @Param("operate") Integer operate);

	/**
	 * 更新乐团下所有学生的下次缴费时间
	 * @param musicGroupId 乐团编号
	 * @param nextPaymentDate 下次缴费时间
	 * @return
	 */
	int updateNextPaymentDate(@Param("musicGroupId") String musicGroupId, @Param("nextPaymentDate") Date nextPaymentDate);

	/**
	 * 根据可提前缴费的天数查询即将续费的列表
	 * @param days 可提前缴费天数
	 * @return
	 */
	List<MusicGroupStudentFee> queryWillRenewList(int days);

	/**
	 * 批量更新对象
	 * @param updateList
	 * @return
	 */
	int batchUpdate(List<MusicGroupStudentFee> updateList);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/17
	 * @params [musicGroupIds]
	 * @return java.util.List<java.util.Map<java.lang.Long,java.lang.Integer>>
	 * @describe 统计乐团下连续旷课超过次数的学生人数
	 */
	List<Map<Long,Integer>> countContinuosAbsenteeismStudentNum(@Param("musicGroupIds") List<Long> musicGroupIds);

	/**
	 * 根据乐团编号列表和学员编号获取学员缴费状态
	 * @param userId
	 * @return
	 */
	List<Map<String,String>> queryUserPaymentStatus(@Param("userId") Integer userId, @Param("musicGroupIds") String musicGroupIds);

	/**
	 * @describe 获取学生对应乐团下的连续旷课次数
	 * @author Joburgess
	 * @date 2019/10/28
	 * @param musicGroupId: 乐团编号
	 * @param userIds: 用户编号列表
	 * @return java.util.List<java.util.Map<java.lang.Integer,java.lang.Integer>>
	 */
	List<Map<Integer,Integer>> findContinuousAbsenteeismTimesByUsersAndMusicGroup(@Param("musicGroupId") String musicGroupId,
																				  @Param("userIds") List<Integer> userIds);

	/**
	 * 获取当前乐团未缴费的学员人数
	 * @param musicGroupId
	 * @return
	 */
    Integer countStudentNoPayNum(String musicGroupId);
}
