package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.StudentAttendanceResponse;
import com.ym.mec.biz.dal.dto.StudentAttendanceStatusCountDto;
import com.ym.mec.biz.dal.dto.StudentPersonalAttendanceDto;
import com.ym.mec.biz.dal.dto.StudentStatusCountUtilEntity;
import com.ym.mec.biz.dal.entity.StudentAttendance;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface StudentAttendanceDao extends BaseDAO<Long, StudentAttendance> {

    /**
     * @Author: Joburgess
     * @Date: 2019/9/11
     * 批量插入学生上课签到情况
     */
    int addStudentAttendances(List<StudentAttendance> studentAttendances);

    /**
     * @describe 根据课程删除对应的点名记录
     * @author Joburgess
     * @date 2019/10/21
     * @param courseScheduleId:
     * @return int
     */
    @Delete("delete from student_attendance where course_schedule_id_=#{courseScheduleId}")
    int deleteStudentAttendancesByCourse(Long courseScheduleId);

    /**
     * 统计学生各种状态人数
     * @param classId
     * @return
     */
    List<StudentStatusCountUtilEntity> countStudentStatus(@Param("classId") Long classId);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/12
     * 获取学生签到所需信息
     */
    StudentAttendance getStudentAttendanceInfo(StudentAttendance studentAttendance);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/29
     * 获取对应学生最后一次的签到记录
     */
    StudentAttendance getStudentLatestAttendanceInfo(StudentAttendance studentAttendance);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/16
     * 获取学生个人上课签到记录
     */
    List<StudentPersonalAttendanceDto> queryStudentPersonalAttendances(Map<String,Object> params);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/16
     * 获取学生个人上课签到记录统计
     */
    int queryStudentPersonalAttendancesCount(Map<String,Object> params);


    /**
     * @Author: Joburgess
     * @Date: 2019/9/18
     * 统计正常签到的学生数量
     */
    @Select("SELECT COUNT(*) FROM student_attendance WHERE class_group_id_=#{classGroupID} AND status_=\"NORMAL\"")
    int countNormalAttendanceStudentNums(Long classGroupID);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/16
     * @params [courseScheduleId]
     * @return com.ym.mec.biz.dal.dto.StudentPersonalAttendanceDto
     * @describe 根据课程统计点名记录
     */
    @Select("SELECT COUNT(*) FROM student_attendance WHERE course_schedule_id_=#{courseScheduleId}")
    int countByCourseSchedule(Long courseScheduleId);

    /**
     * @describe 根据课程获取最后的点名时间
     * @author Joburgess
     * @date 2019/10/24
     * @param courseScheduleId: 课程编号
     * @return java.util.Date
     */
    Date findLatestAttendanceDate(Long courseScheduleId);

    /**
     * @describe 根据班级和学生编号查询签到记录
     * @author Joburgess
     * @date 2019/10/28
     * @param classGroupId: 班级编号
     * @param userIds: 学生编号列表
     * @return java.util.List<com.ym.mec.biz.dal.dto.StudentAttendanceResponse>
     */
    List<StudentAttendanceResponse> findByClassGroupAndUsers(@Param("classGroupId") Long classGroupId,
                                                             @Param("userIds") List<Integer> userIds);

    /**
     * @describe 统计指定班级中学生不同签到状态的次数
     * @author Joburgess
     * @date 2019/10/28
     * @param classGroupId: 班级编号
     * @param userIds: 用户编号
     * @return java.util.List<com.ym.mec.biz.dal.dto.StudentAttendanceStatusCountDto>
     */
    List<StudentAttendanceStatusCountDto> countStudentAttendanceStatus(@Param("classGroupId") Long classGroupId,
                                                                       @Param("userIds") List<Integer> userIds);

    /**
     * @describe 获取课程的当前课次
     * @author Joburgess
     * @date 2019/10/28
     * @param courseScheduleIds: 课程编号列表
     * @return java.util.List<java.util.Map<java.lang.Integer,java.lang.Integer>>
     */
    List<Map<Integer,Integer>> getCurrentClassTimesByCourses(@Param("courseScheduleIds") List<Integer> courseScheduleIds);

    /**
     * @describe 获取班级做后点名的时间
     * @author Joburgess
     * @date 2019/10/28
     * @param classGroupId: 班级编号
     * @return java.util.Date
     */
    Date findLatestSignInDate(Integer classGroupId);

    /**
     * 根据签到状态和课程编号查询签到记录
     * @param userId
     * @param courseScheduleId
     * @return
     */
    StudentAttendance findByStatusAndCourseScheduleId(@Param("userId") Integer userId, @Param("courseScheduleId") int courseScheduleId);


    /**
     * 封装未签到学员列表
     * @param courseScheduleId
     * @return
     */
    List<StudentAttendance> queryNoSignStudentRecord(Integer courseScheduleId);
}
