package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.Mapper;
import com.ym.mec.biz.dal.dto.TeacherAbnormalAttendanceInfoDto;
import com.ym.mec.biz.dal.dto.TeacherPersonalAttendanceDto;
import com.ym.mec.biz.dal.entity.TeacherAttendance;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface TeacherAttendanceDao extends BaseDAO<Long, TeacherAttendance> {

    /**
     * 获取教师个人的签到记录
     * @param params
     * @return
     */
    List<TeacherPersonalAttendanceDto> getTeacherPersonalAttendances(Map<String,Object> params);

    int batchInsert(@Param("teacherAttendances") List<TeacherAttendance> teacherAttendances);

    int batchDeleteByCourseSchedules(@Param("courseScheduleIds") List<Long> courseScheduleIds);

    /**
     * 教师个人签到记录统计
     * @param params
     * @return
     */
    int getTeacherPersonalAttendancesCount(Map<String,Object> params);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/24
     * 统计课次
     */
    int countClassTime(Long classGroupId);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/11
     * @params [teacherId, courseScheduleId]
     * @return com.ym.mec.biz.dal.entity.TeacherAttendance
     * @describe 根据老师编号和课程编号获取老师签到记录
     */
    TeacherAttendance findByTeacherAttendanceInfo(@Param("teacherId") Long teacherId,
                                                  @Param("courseScheduleId") Long courseScheduleId);
	
    /**
     * 查询超过指定分钟数还未签退的用户
     * @param minutes 分钟数
     * @return
     */
    List<Mapper> queryNoSignOutListByOverMinutes(Integer minutes);

    /**
     * @describe 根据日期获取部门下异常签到的课程记录
     * @author Joburgess
     * @date 2019/10/25
     * @param organId: 部门编号
     * @param date: 日期
     * @param signInStatus: 签到状态：0异常，1正常，null未签到
     * @return java.util.List<com.ym.mec.biz.dal.dto.TeacherAbnormalAttendanceInfoDto>
     */
    List<TeacherAbnormalAttendanceInfoDto> findTeacherAbnormalSignInCourseScheduleRecord(@Param("organId") Integer organId,
                                                                                         @Param("date") Date date,
                                                                                         @Param("signInStatus") Integer signInStatus);

    /**
     * @describe 根据月份获取部门下异常签到的日期
     * @author Joburgess
     * @date 2019/10/25
     * @param organId: 部门编号
     * @param date: 日期
     * @return java.util.List<java.util.Date>
     */
    List<Date> findTeacherAbnormalSignInDates(@Param("organId") Integer organId,@Param("date") Date date);

    /**
     * @describe 根据日期获取对应签到状态的课程数量
     * @author Joburgess
     * @date 2019/10/25
     * @param organId: 部门编号
     * @param date: 日期
     * @param signInStatus: 签到状态：0异常，1正常，null未签到
     * @return int
     */
    int countTeacherAbnormalSignInClassTimes(@Param("organId") Integer organId,
                                             @Param("date") Date date,
                                             @Param("signInStatus") Integer signInStatus);

    /**
     * @describe
     * @author Joburgess
     * @date 2019/11/11
     * @param courseScheduleIds: 课程编号列表
     * @return java.util.List<java.util.Map<java.lang.Integer,java.lang.Integer>>
     */
    List<TeacherAttendance> findTeacherIdByCourseSchedule(@Param("courseScheduleIds") List<Long> courseScheduleIds);
}
