package com.ym.mec.biz.dal.dao;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

public interface TeacherDao extends BaseDAO<Integer, Teacher> {

    /**
     * 根据证件号查询老师
     *
     * @param certificateNum 证件号
     * @return Teacher 老师信息
     */
    Teacher findByCertificateNum(@Param("certificateNum") String certificateNum);

    /**
     * 修改用户信息
     *
     * @param user
     */
    void updateUser(SysUser user);

    /**
     * 新增sysUser
     *
     * @param user
     */
    void addSysUser(SysUser user);

    /**
     * 获取和当前教师相关的学员列表
     *
     * @param teacherId
     * @param search
     * @return
     */
    List<BasicUserDto> queryGroupStudents(@Param("teacherId") Integer teacherId, @Param("search") String search);

    /**
     * 根据用户编号列表获取用户名称
     *
     * @param userIds
     * @return
     */
    List<Map<Integer, String>> queryNameByIds(@Param("userIds") String userIds);

    /**
     * 根据用户编号列表获取手机号
     *
     * @param userIds
     * @return
     */
    List<Map<Integer, String>> queryPhoneByIds(@Param("userIds") String userIds);

    /**
     * 根据用户编号获取用户名称
     *
     * @param userId
     * @return
     */
    String queryNameById(@Param("userId") Integer userId);

    /**
     * 查询分部所有老师
     * @param organId
     * @return
     */
    List<Teacher> findTeachers(@Param("organId") Integer organId);

    /**
     * count上课结算列表
     * @param params
     * @return
     */
    int countCloses(Map<String, Object> params);

    /**
     * 获取教师上课结算列表
     * @param params
     * @return
     */
    List<TeacherCloseDto> queryCloses(Map<String, Object> params);

    /**
     * @describe 获取指定乐团下的所有老师
     * @author Joburgess
     * @date 2019/10/19
     * @param musicGroupId: 乐团编号
     * @return java.util.List<com.ym.mec.biz.dal.dto.BasicUserDto>
     */
    List<BasicUserDto> findTeacherByMusicGroup(String musicGroupId);


    /**
     * 获取教师未结算课酬综合
     * @param teacherId
     * @return
     */
    BigDecimal findTeacherNoPay(Integer teacherId);

    /**
     * 获取乐团详情老师列表
     * @param params
     * @return
     */
    List<MusicGroupTeachersDto> queryMusicGroupTeachers(Map<String, Object> params);

    /**
     * Count乐团详情老师列表
     * @param params
     * @return
     */
    int countMusicGroupTeachers(Map<String, Object> params);

    /**
     * 根据教师编号列表，和班级编号列表获取教师签到次数
     * @param teacherIdStr
     * @return
     */
    List<Map<Integer, Integer>> findSignNum(@Param("teacherIdStr") String teacherIdStr,@Param("musicGroupId") String musicGroupId);

    /**
     * 获取教师乐团上课记录（乐团管理--乐团详情）
     * @param params
     * @return
     */
    List<MusicGroupTeacherAttendanceDto> queryTeacherAttendance(Map<String, Object> params);

    /**
     * COUNT教师乐团上课记录（乐团管理--乐团详情）
     * @param params
     * @return
     */
    int countTeacherAttendance(Map<String, Object> params);

    /**
     * 获取乐团学员报名列表
     * @param params
     * @return
     */
    List<musicGroupStudentApplyDto> queryMusicGroupStudentApply(Map<String, Object> params);

    /**
     * COUNT乐团学员报名列表
     * @param params
     * @return
     */
    int countMusicGroupStudentApply(Map<String, Object> params);

    /**
     * 获取老师的群组基本信息
     * @param teacherIds
     * @param search
     * @return
     */
    List<BasicUserDto> queryMusicGroupStudents(@Param("teacherIds") String teacherIds,@Param("search") String search);

    /**
     * @describe 根据声部获取部门下的老师
     * @author Joburgess
     * @date 2019/11/4
     * @param organId: 部门编号
     * @param subjectId: 声部编号
     * @return java.util.List<com.ym.mec.biz.dal.entity.Teacher>
     */
    List<Teacher> findOrganTeacherBySubject(@Param("organId") Integer organId,
                                            @Param("subjectId") Integer subjectId);

    /**
     * 根据分部查询老师数
     * @param organId
     * @return
     */
    Map<String,Integer> queryOrganTeacherNum(@Param("organId") Integer organId);
}