package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.School;
import com.ym.mec.biz.dal.entity.VipGroup;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

public interface VipGroupDao extends BaseDAO<Long, VipGroup> {

	/**
	 * @describe 锁定指定vip课
	 * @author Joburgess
	 * @date 2019/10/21
	 * @param vipGroupId: vip课编号
	 * @return com.ym.mec.biz.dal.entity.VipGroup
	 */
	VipGroup getLockVipGroup(Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/9
	 * @params [organId]
	 * @return java.util.List<com.ym.mec.biz.dal.entity.VipGroup>
	 * @describe 根据部门获取
	 */
	List<VipGroup> findAllByOrgan(Map<String, Object> params);

	int countVipGroupByOrgan(Map<String, Object> params);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/9
	 * @params [params]
	 * @return java.util.List<com.ym.mec.biz.dal.dto.VipGroupStudentDto>
	 * @describe 获取小课学员
	 */
	List<VipGroupStudentDto> findVipGroupStudents(Map<String, Object> params);

	int countVipGroupStudents(Map<String, Object> params);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/9
	 * @params [params]
	 * @return java.util.List<com.ym.mec.biz.dal.dto.VipGroupCourseSchduleRecordDto>
	 * @describe 获取vip课教学记录
	 */
	List<VipGroupCourseSchduleRecordDto> findTeachingRecord(Map<String, Object> params);

	int countTeachingRecord(Map<String, Object> params);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/9
	 * @params [vipGroupId]
	 * @return int
	 * @describe 统计已结算课程数量
	 */
	int countIsSalaryNum(Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/9
	 * @params [vipGroupId]
	 * @return int
	 * @describe 统计被投诉课程数量
	 */
	int countComplaintsNum(Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/9
	 * @params [vipGroupId]
	 * @return com.ym.mec.biz.dal.dto.TeachingRecordBaseInfo
	 * @describe 获取vip课的总课次及已上课次
	 */
	TeachingRecordBaseInfo getClassTimes(Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [userID]
	 * @return java.util.List<com.ym.mec.biz.dal.dto.StudentManageVipGroupDto>
	 * @describe 获取对应学生vip课
	 */
	List<StudentManageVipGroupDto> findStudentVipGroups(Integer userId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [params]
	 * @return java.util.List<com.ym.mec.biz.dal.dto.StudentVipGroupShowListDto>
	 * @describe 学生端获取vip课列表
	 */
	List<StudentVipGroupShowListDto> findVipGroups(Map<String, Object> params);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [params]
	 * @return int
	 * @describe 统计vip数
	 */
	int countVipGorups(Map<String, Object> params);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [vipGroupId]
	 * @return com.ym.mec.biz.dal.dto.VipGroupManageDetailDto
	 * @describe 获取小课详情
	 */
	VipGroupManageDetailDto getVipGroupBaseInfo(Long vipGroupId);

	/**
	 * @describe 获取vip课财务基本信息
	 * @author Joburgess
	 * @date 2019/11/5
	 * @param vipGroupId: vip课编号
	 * @return com.ym.mec.biz.dal.dto.VipGroupSalaryBaseInfo
	 */
	VipGroupSalaryBaseInfo getVipGroupSalaryInfo(@Param("vipGroupId") Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/11
	 * @params [vipGroupId]
	 * @return int
	 * @describe 统计扣费笔数
	 */
	int countVipGroupDeductionNum(Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [params]
	 * @return java.util.List<com.ym.mec.biz.dal.entity.CourseSchedule>
	 * @describe 获取小课考勤记录
	 */
	List<CourseSchedule> findVipGroupAttendances(Map<String, Object> params);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [params]
	 * @return int
	 * @describe 统计小课考勤记录数
	 */
	int countVipGroupAttendance(Map<String, Object> params);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [vipGroupId]
	 * @return com.ym.mec.biz.dal.dto.StudentVipGroupDetailDto
	 * @describe 获取vip课详情
	 */
	StudentVipGroupDetailDto getVipGroupDetail(Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [vipGroupId]
	 * @return java.util.List<java.util.Map<java.lang.String,java.lang.Integer>>
	 * @describe 统计剩余课时数
	 */
	List<Map<String, Integer>> countSurplusClassTimes(Long vipGroupId);

	/**
	 * @describe 统计vip课已上课程数量
	 * @author Joburgess
	 * @date 2019/10/21
	 * @param vipGroupIds: vip课编号列表
	 * @return java.util.List<java.util.Map<java.lang.Long,java.lang.Integer>>
	 */
	List<Map<Long, Integer>> countVipGroupOverCourse(List<Long> vipGroupIds);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/12
	 * @params [vipGroupId]
	 * @return java.math.BigDecimal
	 * @describe 统计学生vip课未上课时费用
	 */
	List<Map<Integer, BigDecimal>> countSurplusCourseFee(Long vipGroupId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/14
	 * @params [teacherId]
	 * @return int
	 * @describe 统计老师开启的vip课数
	 */
	int countTeacherVipGroups(Long teacherId);

	/**
	 * 根据教师编号获取老师vip课数量
	 * @param teacherId
	 * @return
	 */
	int countTeacherVipClass(Integer teacherId);

	/**
	 * 根据教师编号获取老师vip课列表
	 * @param params
	 * @return
	 */
	List<TeacherVipClassInfoDto> getTeacherVipClass(Map<String, Object> params);

	/**
	 * 根据老师查询vip课教学点
	 * @param userId
	 * @return
	 */
	List<School> querySchoolByUserId(Integer userId);

	/**
	 * @describe 根据课程编号获取vip课
	 * @author Joburgess
	 * @date 2019/10/23
	 * @param courseScheduleId: 课程编号
	 * @return com.ym.mec.biz.dal.entity.VipGroup
	 */
	VipGroup findByCourseSchedule(Integer courseScheduleId);

	/**
	 * 查询vip课数目
	 * @param organId
	 * @return
	 */
	Map<String, Integer> queryVipGroupNum(@Param("organId") Integer organId);
	
	/**
	 * 查询当月vip课数量
	 * @param organId
	 * @return
	 */
	Integer queryCurrentMonthCoursesNum(@Param("organId") Integer organId);

	/**
	 * @describe 获取报名已截至但是报名人数还未达到的vip课程
	 * @author Joburgess
	 * @date 2019/11/13
	 * @param :
	 * @return java.util.List<com.ym.mec.biz.dal.entity.VipGroup>
	 */
	List<VipGroup> findNoCreateSuccessVipGroup();

}
