package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.enums.SalarySettlementTypeEnum;
import com.ym.mec.biz.dal.page.CourseScheduleQueryInfo;
import com.ym.mec.biz.dal.page.StudentPayLogQueryInfo;
import com.ym.mec.biz.dal.page.VipClassQueryInfo;
import com.ym.mec.biz.dal.page.queryMusicGroupCourseScheduleQueryInfo;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.BaseService;

import java.util.List;

public interface ClassGroupService extends BaseService<Integer, ClassGroup> {

    /**
     * 查询老师乐团课课程
     *
     * @param teacherId
     * @return
     */
    List<ClassGroup> findClassGroup4Teacher(Integer teacherId);

    /**
     * 合奏班相关班级获取
     *
     * @param mixClassGroupId
     * @return
     */
    List<ClassGroup> findClassGroup(String musicGroupId, Integer mixClassGroupId);

    /**
     * 根据乐团编号查询单技班列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findAllNormalClassGroupByMusicGroupId(String musicGroupId);


    /**
     * 根据乐团编号查询合奏班列表
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findAllMixClassGroupByMusicGroupId(String musicGroupId);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/8
     * @params [musicGroupId]
     * @return java.util.List<com.ym.mec.biz.dal.entity.ClassGroup>
     * @describe 获取乐团下所有班级
     */
    List<ClassGroup> findAllClassGroupByMusicGroup(String musicGroupId);

    /**
     * 添加单技班
     *
     * @param classGroup
     * @return
     */
    ClassGroup addClassGroup(ClassGroup classGroup) throws Exception;

    /**
     * 添加合奏班
     *
     * @param classGroupIds
     * @return
     * @throws Exception
     */
    ClassGroup addMixClassGroup(String musicGroupId, String name, String classGroupIds) throws Exception;

    /**
     * 跟班级ids查询班级信息
     *
     * @param ids
     * @return
     */
    List<ClassGroup> findClassGroupByIds(String ids);


    /**
     * 提高班批量添加
     *
     * @param highClassGroupList
     * @return
     * @throws Exception
     */
    List<HighClassGroupDto> addHighClassGroup(List<HighClassGroupDto> highClassGroupList) throws Exception;

    /**
     * 获取老师乐团课程信息列表
     *
     * @param queryInfo
     * @return
     */
    PageInfo<TeacherMusicClassInfoDto> getTeacherMusicClass(QueryInfo queryInfo);

    /**
     * 获取老师VIP课程信息列表
     *
     * @param queryInfo
     * @return
     */
    PageInfo<TeacherVipClassInfoDto> getTeacherVipClass(VipClassQueryInfo queryInfo);


    /**
     * 获取乐团班级老师课酬
     *
     * @param musicGroupId
     * @param type 班级类型（多种类型）
     * @return
     */
    List<ClassGroupTeachersDto> getClassGroupAndTeachers(String musicGroupId,String type,Integer classGroupId);

    /**
     * 获取乐团班级老师课酬
     *
     * @param musicGroupId
     * @return
     */
    List<ClassGroupTeachersDto> getClassGroupAndTeacherSalary(String musicGroupId, SalarySettlementTypeEnum type,Integer classGroupId) throws Exception;


    /**
     * 获取学生所购买课程列表
     *
     * @param userId
     * @param queryInfo
     * @return
     */
    PageInfo<CourseListDto> queryCoursePage(QueryInfo queryInfo,Integer userId);

    /**
     * 获取学员所在群组信息
     * @param userId
     * @param search
     * @return
     */
    List<ImGroupModel> queryUserGroups(Integer userId,String search);

    /**
     * 获取群组下用户基本信息
     * @param groupId
     * @return
     */
    List<ImUserModel> findGroupUsers(Integer groupId);

    /**
     * 分页查询学生购买记录
     * @param studentPayLogQueryInfo
     * @return
     */
    PageInfo<StudentPayInfoDto> queryStudentPayLog(StudentPayLogQueryInfo studentPayLogQueryInfo);

    /**
     * 班级添加学生
     * @param classGroup
     * @param userId
     * @return
     */
    ClassGroup addStudent(ClassGroup classGroup,Integer userId);

    /**
     * 获取未分配合奏的单技班列表
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> findNoClassSubjects(String musicGroupId);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/16
     * @params [classGroupId, num]
     * @return boolean
     * @describe 跟新班级人数
     */
    boolean updateClassStudentNum(Long classGroupId,Integer num);

    /**
     * 删除单技班级
     * @param classGroupId
     */
    void delSingle(Integer classGroupId);

    /**
     * 删除合奏班级
     * @param classGroupId
     */
    void delMix(Integer classGroupId);

    /**
     * 成团确认，校验参数，并设置团长
     * @param teacherId
     * @param musicGroupId
     */
    void addMusicGroupTeam(Integer teacherId, String musicGroupId,Integer improventClassesNum) throws Exception;

    /**
     * @Author: Joburgess
     * @Date: 2019/10/17
     * @params [type]
     * @return java.util.List<com.ym.mec.biz.dal.dto.TeacherClassGroupDto>
     * @describe 获取老师所在班级列表
     */
    List<TeacherClassGroupDto> findTeacherClassGroups(String type);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/18
     * @params @param classGroupId:班级编号
     * @return com.ym.mec.biz.dal.dto.TeacherClassHeadInfo
     * @describe 获取班级信息
     */
    TeacherClassHeadInfo findTeacherClassGroupInfo(Long classGroupId);

    /**
     * @describe 获取班级对应学生列表
     * @author Joburgess
     * @date 2019/10/18
     * @param queryInfo:
     * @return org.snaker.engine.access.Page
     */
    PageInfo findTeacherClassStudents(CourseScheduleQueryInfo queryInfo);

    /**
     * @describe 根据班级获取点名列表
     * @author Joburgess
     * @date 2019/10/19
     * @param courseScheduleId:
     * @return java.util.List<com.ym.mec.biz.dal.dto.StudentAttendanceViewDto>
     */
    List<StudentAttendanceViewDto> findAttendanceStudentByCourse(Long courseScheduleId);

    /**
     * 班级调整(新增班级)
     * @param classGroup4MixDto
     * @return
     */
    ClassGroup classGroupAdjust(ClassGroup4MixDto classGroup4MixDto) throws Exception;

    /**
     * 班级调整(修改班级)
     * @param classGroup4MixDto
     * @return
     */
    ClassGroup classGroupUpdate(ClassGroup4MixDto classGroup4MixDto) throws Exception;

    /**
     * 班级调整(增加临时班级)
     * @param classGroup4MixDto
     * @return
     */
    ClassGroup classGroupSnap(ClassGroup4MixDto classGroup4MixDto) throws Exception;


    /**
     * 乐团详情--班级详情列表
     * @param queryInfo
     * @return
     */
    PageInfo<MusicGroupClassGroupDto> queryMusicGroupClassGroup(QueryInfo queryInfo);

    /**
     * 乐团详情-----课表详情
     * @param queryInfo
     * @return
     */
    PageInfo<MusicGroupCourseScheduleDto> queryMusicGroupCourseSchedule(queryMusicGroupCourseScheduleQueryInfo queryInfo);
    
    /**
     * 批量新增班级上的课次数
     * @param classGroupList
     * @return
     */
    boolean batchIncreaseClassTimes(List<Integer> classGroupList);

    /**
     * 获取提高班班级
     * @param musicGroupId
     * @return
     */
    List<ClassGroup> highClassGroupList(String musicGroupId);


    List<ClassGroupTeachersDto> classGroupAndTeacher(Integer classGroupId);


    /**
     * 根据乐团跟声部获取提高班列表
     *
     * @param musicGroupId
     * @param subjectId
     * @return
     */
    List<HighClassGroupDto> findClassGroupByMusicGroupIdAndSubjectId(String musicGroupId,Integer subjectId);

    /**
     * @describe 批量更新指定班级人数
     * @author Joburgess
     * @date 2019/11/12
     * @param classGroupIds: 班级编号列表
     * @return int
     */
    int batchUpdateClassCourseTimes(List<Integer> classGroupIds);
}
