package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.CourseAttendanceDetailHeadInfoDto;
import com.ym.mec.biz.dal.dto.CourseScheduleDto;
import com.ym.mec.biz.dal.dto.TeacherAttendanceDto;
import com.ym.mec.biz.dal.entity.ClassGroupTeacherMapper;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.CourseScheduleComplaints;
import com.ym.mec.biz.dal.enums.AuditStatusEnum;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.CourseScheduleQueryInfo;
import com.ym.mec.biz.dal.page.StudentCourseScheduleRecordQueryInfo;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.BaseService;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface CourseScheduleService extends BaseService<Long, CourseSchedule> {

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/10
	 * 根据课程ID获取当前课程的信息
	 */
	TeacherAttendanceDto getCurrentCourseDetail(Long courseID);

	/**
	 * @describe 根据课程编号删除课程计划及对应的学生缴费、教师课酬记录
	 * @author Joburgess
	 * @date 2019/10/31
	 * @param courseScheduleIds:
	 * @return int
	 */
	void deleteCourseSchedules(List<Long> courseScheduleIds);

	/**
	 * @describe 判断该课程是否只能进行正常签到
	 * @author Joburgess
	 * @date 2019/11/5
	 * @param startClassTime: 课程开始时间
	 * @param userId: 教师编号
	 * @param checkSchool: 是否判断教学点
	 * @param schoolId: 学校编号
	 * @return com.ym.mec.biz.dal.enums.YesOrNoEnum
	 */
	YesOrNoEnum enableOnlyNormalAttendance(Date startClassTime,Long userId,boolean checkSchool,Integer schoolId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 批量插入课程计划
	 */
	void batchAddCourseSchedule(List<CourseSchedule> courseSchedules);

	/**
	 * @describe 创建乐团课课程计划
	 * @author Joburgess
	 * @date 2019/10/24
	 * @param courseSchedules: 课程计划列表
	 * @return void
	 */
	void createCourseSchedules(List<CourseSchedule> courseSchedules);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 批量更新课程计划
	 */
	void batchUpdateCourseSchedule(List<CourseSchedule> courseSchedules, Long musicGroupID);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 根据日期获取当日排课信息
	 */
	List<CourseScheduleDto> getCourseSchedulesWithDate(Date classDate);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 根据日期获取当日排课信息
	 */
	Map getStudentCourseSchedulesWithDate(Date classDate);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 根据日期获取当日排课信息
	 */
	List<CourseScheduleDto> getTeacherCourseSchedulesWithDate(Integer userId, Date classDate, String type);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 根据日期获取当日排课信息
	 */
	PageInfo getCourseSchedulesWithDateByEdu(CourseScheduleQueryInfo queryInfo);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 根据月份获取乐团在该月有课的日期
	 */
	List<Date> getCourseScheduleDateByMonth(Long musicGroupID, Date month);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 根据学生获取乐团在该月有课的日期
	 */
	List<Date> getCourseScheduleDateByStudent(Date month, Integer isAttend);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/17
	 * 获取教师在该月有课的日期
	 */
	List<Date> getCourseScheduleDateByTeacher(Date month, String type);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/11
	 * @params [month]
	 * @return java.util.List<java.util.Date>
	 * @describe 获取
	 */
	List<Date> getCourseScheduleDates(Date month);

	/**
	 * @describe 课程调整
	 * @author Joburgess
	 * @date 2019/11/7
	 * @param newCourseSchedules: 课程计划列表
	 * @return void
	 */
	void courseAdjust(List<CourseSchedule> newCourseSchedules);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/23
	 * 课时调整
	 */
	void classStartDateAdjust(List<CourseSchedule> newCourseSchedules);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/24
	 * 课程交换
	 */
	void courseSwap(Long courseScheduleId1, Long courseScheduleId2);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/14
	 * @params [day1, day2]
	 * @return void
	 * @describe 对调两天的课程
	 */
	void courseSwapWithDay(Date day1, Date day2);

	/**
	 * @describe 检测数据库不存在的课程是否存在错误
	 * @author Joburgess
	 * @date 2019/10/31
	 * @param courseSchedules: 课程列表
	 * @return void
	 */
	void checkNewCourseSchedules(List<CourseSchedule> courseSchedules,boolean checkExistCourseSchedule);

	/**
	 * 获取班级未开始的课程计划
	 *
	 * @param classGroupTeacherMapperList
	 * @return
	 */
	List<CourseSchedule> findClassGroupNoStartCourseSchedules(List<ClassGroupTeacherMapper> classGroupTeacherMapperList);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/30
	 * 获取学生上课记录
	 */
	Object findStudentCourseScheduleRecords(StudentCourseScheduleRecordQueryInfo queryInfo);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [courseScheduleComplaints]
	 * @return void
	 * @describe 课程投诉
	 */
	void courseScheduleCommplaint(CourseScheduleComplaints courseScheduleComplaints);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [id] 申诉编号
	 * @params [teacherLiabilityRatio] 老师责任比
	 * @params [studentLiabilityRatio] 学生责任比
	 * @params [status] 审批状态
	 * @return void
	 * @describe 课程投诉审核
	 */
	void courseScheduleCommplaintAudit(Long id, Double teacherLiabilityRatio, Double studentLiabilityRatio, AuditStatusEnum status);

	/**
	 * 更新课程至结束状态
	 * @return
	 */
	boolean updateCourseScheduleToOverStatus();

	/**
	 * @describe 获取老师指定班级的排课
	 * @author Joburgess
	 * @date 2019/10/18
	 * @param queryInfo: 参数
	 * @return com.ym.mec.common.page.PageInfo
	 */
	PageInfo queryTeacherClassCourseSchedule(CourseScheduleQueryInfo queryInfo);

	/**
	 * @describe 获取点名详情界面头部信息
	 * @author Joburgess
	 * @date 2019/10/24
	 * @param courseScheduleId: 课程编号
	 * @return com.ym.mec.biz.dal.dto.CourseAttendanceDetailHeadInfoDto
	 */
	CourseAttendanceDetailHeadInfoDto findCourseAttendanceDetailHeadInfo(Long courseScheduleId);

	/**
	 * @describe 检测指定乐团下所有班级是否已排课
	 * @author Joburgess
	 * @date 2019/10/18
	 * @param musicGroupId: 乐团编号
	 * @return void
	 */
	void checkMusicGroupClassCourse(String musicGroupId);

	/**
	 * @describe 根据课程计划编号获取当前课次
	 * @author Joburgess
	 * @date 2019/10/20
	 * @param courseSchedules:
	 * @return java.util.Map<java.lang.Integer,java.lang.Integer>
	 */
	Map<Integer, Integer> findCourseScheduleCurrentTimes(List<Integer> courseSchedules);

	/**
	 * 查询班级未开始的课程
	 *
	 * @param classGroupId
	 * @return
	 */
	List<CourseSchedule> findNoStartCoursesByClassGroupId(Integer classGroupId);

	/**
	 * 学生有课提醒
	 * @return
	 */
	void studentCoursesScheduleRemind();

	/**
	 * 学生上课提醒
	 */
	void studentSignInRemind();

	/**
	 * 老师课程提醒
	 */
	void teacherCoursesScheduleRemind();

	/**
	 * 老师上课提醒
	 */
	void teacherSignInRemind();

	/**
	 * @describe 获取班级的课程计划
	 * @author Joburgess
	 * @date 2019/11/5
	 * @param classGroupId: 班级编号
	 * @return java.util.List<com.ym.mec.biz.dal.entity.CourseSchedule>
	 */
	List<CourseSchedule> findCourseScheduleByClassGroup(Integer classGroupId);
}
