package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.TeacherAbnormalAttendanceInfoDto;
import com.ym.mec.biz.dal.dto.TeacherSignOutDto;
import com.ym.mec.biz.dal.entity.TeacherAttendance;
import com.ym.mec.biz.dal.enums.SignStatusEnum;
import com.ym.mec.biz.dal.page.TeacherCloseQueryInfo;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.BaseService;

import java.util.Date;
import java.util.List;
import java.util.Map;

public interface TeacherAttendanceService extends BaseService<Long, TeacherAttendance> {

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/10
	 * 添加教师签到记录
	 */
	Map<String, Object> addTeacherAttendanceRecord(TeacherSignOutDto teacherSignOutDto);

	/**
	 * @describe 调价教师签到记录
	 * @author Joburgess
	 * @date 2019/11/4
	 * @param courseScheduleId: 课程编号
	 * @param userId: 用户编号
	 * @param signStatus: 签到or签退
	 * @param updateCourseScheduleStatus: 是否更新课程状态
	 * @return void
	 */
	void addTeacherAttendanceRecord(Integer courseScheduleId,
									Integer userId,
									SignStatusEnum signStatus,
									boolean updateCourseScheduleStatus);

	/**
	 * 获取教师个人的签到记录
	 * @return
	 */
	PageInfo getTeacherPersonalAttendances(TeacherCloseQueryInfo queryInfo);

	/**
	 * 推送未签退消息提醒
	 * @return
	 */
	boolean pushNoSignOutMessage();

	/**
	 * @describe 根据日期获取部门下异常签到的课程记录
	 * @author Joburgess
	 * @date 2019/10/25
	 * @param organId: 部门编号
	 * @param date: 日期
	 * @param signInStatus: 签到状态：0异常，1正常，null未签到
	 * @return java.util.List<com.ym.mec.biz.dal.dto.TeacherAbnormalAttendanceInfoDto>
	 */
	List<TeacherAbnormalAttendanceInfoDto> findTeacherAbnormalSignInCourseScheduleRecord(Integer organId,Date date,Integer signInStatus);

	/**
	 * @describe 根据月份获取部门下异常签到的日期
	 * @author Joburgess
	 * @date 2019/10/25
	 * @param organId: 部门编号
	 * @param date: 日期
	 * @return java.util.List<java.util.Date>
	 */
	List<Date> findTeacherAbnormalSignInDates(Integer organId, Date date);

	/**
	 * @describe 根据日期获取对应签到状态的课程数量
	 * @author Joburgess
	 * @date 2019/10/25
	 * @param organId: 部门编号
	 * @param date: 日期
	 * @return int
	 */
	Map countTeacherAbnormalSignInClassTimes(Integer organId, Date date);
}
