package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.BaseService;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

public interface TeacherService extends BaseService<Integer, Teacher> {

    /**
     * 根据证件号查询老师信息
     * @param certificateNum 证件号
     * @return Teacher 老师信息
     */
    Teacher findByCertificateNum(String certificateNum);

    /**
     * 新增教师
     * @param teacher
     */
    int add(Teacher teacher) throws Exception;

    /**
     * 修改教师信息
     * @param teacher
     */
    void updateTea(Teacher teacher);

    /**
     * 分页查询教师列表
     * @param queryInfo
     * @return
     */
    PageInfo<Teacher> queryPageDetail(TeacherQueryInfo queryInfo);

    /**
     * 获取教师详情
     * @param id
     * @return
     */
    Teacher getDetail(Integer id);

    /**
     * @Author: Joburgess
     * @Date: 2019/9/24
     * 删除教师
     */
    void deleteTeacher(Long userId);

    /**
     * 获取教师所在群组列表
     * @param userId
     * @param search
     * @return
     */
    List<ImGroupModel> queryTeacherGroups(Integer userId,String search);

    /**
     * 获取和当前教师相关的学员列表
     * @param id
     * @param search
     * @return
     */
    List<BasicUserDto> queryGroupStudents(Integer id, String search);

    /**
     * 查询分部所有老师
     * @param organId
     * @return
     */
    List<Teacher> findTeachers(@Param("organId") Integer organId);

    /**
     * @describe 根据科目获取部门下的教师
     * @author Joburgess
     * @date 2019/11/4
     * @param organId: 部门编号
     * @param subjectId: 科目编号
     * @return java.util.List<com.ym.mec.biz.dal.entity.Teacher>
     */
    List<Teacher> findOrganTeacherBySubject(@Param("organId") Integer organId,
                                            @Param("subjectId") Integer subjectId);

    /**
     * 获取教师上课结算列表
     * @param queryInfo
     * @return
     */
    PageInfo<TeacherCloseDto> queryCloses(TeacherCloseQueryInfo queryInfo);

    /**
     * @describe 获取乐团下的老师
     * @author Joburgess
     * @date 2019/10/19
     * @param musicGroupId:
     * @return java.util.List<com.ym.mec.biz.dal.dto.BasicUserDto>
     */
    List<BasicUserDto> queryMusicGroupTeacher(String musicGroupId);


    /**
     * 获取老师未结算课酬总额
     * @param teacherId
     * @return
     */
    BigDecimal findTeacherNoPay(Integer teacherId);

    /**
     * 获取乐团详情老师列表
     * @param queryInfo
     * @return
     */
    PageInfo<MusicGroupTeachersDto> queryMusicGroupTeachers(MusicGroupTeachersQueryInfo queryInfo);

    /**
     * 获取教师上课记录列表（乐团管理--乐团详情）
     * @param queryInfo
     * @return
     */
    PageInfo<MusicGroupTeacherAttendanceDto> queryTeacherAttendance(MusicGroupTeacherAttendanceQueryInfo queryInfo);

    /**
     * 获取乐团学员的报名列表
     * @param queryInfo
     * @return
     */
    PageInfo<musicGroupStudentApplyDto> queryMusicGroupStudentApply(queryMusicGroupStudentQueryInfo queryInfo);
}
