package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.School;
import com.ym.mec.biz.dal.entity.StudentApplyRefunds.StudentApplyRefundsStatus;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.entity.VipGroup;
import com.ym.mec.biz.dal.enums.AuditStatusEnum;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.BaseService;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

public interface VipGroupService extends BaseService<Long, VipGroup> {

    /**
     * @describe 创建vip课
     * @author Joburgess
     * @date 2019/10/30
     * @param vipGroup: VIP课申请信息
     * @return void
     */
    void createVipGroup(VipGroupApplyDto vipGroup);

    /**
     * @describe 更新vip课程信息
     * @author Joburgess
     * @date 2019/10/30
     * @param vipGroupApplyInfo: vip课信息
     * @return void
     */
    void updateVipGroup(VipGroupApplyDto vipGroupApplyInfo);

    /**
     * @describe 新增vip课程计划
     * @author Joburgess
     * @date 2019/11/11
     * @param vipGroupApplyDto:
     * @return void
     */
    void appendVipGroupCourseSchedules(VipGroupApplyDto vipGroupApplyDto);

    /**
     * @describe vip课程审核
     * @author Joburgess
     * @date 2019/10/22
     * @param vipGroupId: vip课编号
     * @param auditStatus: 审核状态
     * @return void
     */
    void vipApplyAudit(Long vipGroupId,AuditStatusEnum auditStatus,String reason);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [queryInfo]
     * @return com.ym.mec.common.page.PageInfo
     * @describe 获取可购买vip课列表
     */
    PageInfo findStudentVipGroupList(StudentVipGroupQueryInfo queryInfo);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/9
     * @params [queryInfo]
     * @return com.ym.mec.common.page.PageInfo
     * @describe 获取vip课列表
     */
    PageInfo findVipGroups(VipGroupQueryInfo queryInfo);

    /**
     * @describe 获取vip课教学记录
     * @author Joburgess
     * @date 2019/10/21
     * @param queryInfo: 查询条件
     * @return java.util.Map<java.lang.String,java.lang.Object>
     */
    Map<String, Object> findVipGroupTeachingRecord(VipGroupTeachingRecordQueryInfo queryInfo);

    /**
     * @describe 获取vip课教学记录基本信息
     * @author Joburgess
     * @date 2019/10/21
     * @param vipGroupId: vip课编号
     * @return com.ym.mec.biz.dal.dto.TeachingRecordBaseInfo
     */
    TeachingRecordBaseInfo findTeachingRecordBaseInfo(Long vipGroupId);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/9
     * @params [queryInfo]
     * @return com.ym.mec.common.page.PageInfo
     * @describe 获取vip课学员
     */
    PageInfo findVipGroupStudents(VipGroupQueryInfo queryInfo);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [vipGroupId]
     * @return com.ym.mec.biz.dal.dto.VipGroupManageDetailDto
     * @describe 获取vip课详情
     */
    VipGroupManageDetailDto getVipGroupDetail(Long vipGroupId);

    /**
     * @describe 计算vip课程购买总价,及老师课酬
     * @author Joburgess
     * @date 2019/10/23
     * @param vipGroup: vip课程
     * @param teacherId: 要计算课酬的老师的编号
     * @param onlineClassesUnitPrice: 线上课程单价
     * @param offlineClassesUnitPrice: 线下课程单价
     * @param computeTotalPrice: 是否计算课程购买总价
     * @return java.util.Map
     */
    <K extends VipGroup> Map countVipGroupPredictFee(K vipGroup,
                                                     Integer teacherId,
                                                     BigDecimal onlineClassesUnitPrice,
                                                     BigDecimal offlineClassesUnitPrice,
                                                     boolean computeTotalPrice,
                                                     boolean useDefaultTeacherSalary);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [queryInfo]
     * @return com.ym.mec.common.page.PageInfo
     * @describe 获取vip课考勤记录
     */
    PageInfo findVipGroupAttendances(VipGroupAttendanceQueryInfo queryInfo);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [vipGroupId]
     * @return com.ym.mec.biz.dal.dto.StudentVipGroupDetailDto
     * @describe 学生端获取vip课详情
     */
    StudentVipGroupDetailDto getVipGroupShowDetail(Long vipGroupId);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [vipGroupCostCountParamsDto]
     * @return com.ym.mec.biz.dal.dto.VipGroupCostCountDto
     * @describe vip课程费用计算
     */
    VipGroupCostCountDto vipGroupCostCount(VipGroupCostCountParamsDto vipGroupCostCountParamsDto);

    /**
     * @describe 更新vip课班级人数及状态
     * @author Joburgess
     * @date 2019/10/20
     * @param vipGroupId: vip课编号
     * @param num: 人数
     * @param updateVipStatus: 是否修改VIP课状态
     * @return void
     */
    void updateVipGroupStudentNumAndStatus(Long vipGroupId,Integer num,boolean updateVipStatus);

    /**
     * @describe 课程购买成功后，创建课程相关信息
     * @author Joburgess
     * @date 2019/11/12
     * @param vipGroupId: vip课编号
     * @return void
     */
    void createVipGroupCourseScheInfo(Long vipGroupId);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [vipGroupBuyParams]
     * @return void
     * @describe 购买vip课程
     */
    Map buyVipGroup(VipGroupBuyParamsDto vipGroupBuyParams) throws Exception;

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [callbackParams]
     * @return void
     * @describe 支付回调接口
     */
    void orderCallback(StudentPaymentOrder order);

    /**
	 * @Author: Joburgess
	 * @Date: 2019/10/3
	 * @params [vipGroupId, studentId]
	 * @return void
	 * @describe 申请退课
	 */
	void applyRefund(Long vipGroupId, Integer studentId);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/3
     * @params [studentApplyRefunds]
     * @return void
     * @describe 退课申请审核
     */
    void applyRefundAudit(Long id,StudentApplyRefundsStatus status,String remark,BigDecimal amount);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/11
     * @params [queryInfo]
     * @return com.ym.mec.common.page.PageInfo
     * @describe vip财务记录
     */
    Map<String, Object> findVipGroupSalarys(VipGroupSalaryQueryInfo queryInfo);

    /**
     * @Author: Joburgess
     * @Date: 2019/10/11
     * @params [courseScheduleId]
     * @return java.util.List<java.lang.String>
     * @describe 获取上课签到学员
     */
    List<String> findVipGroupAttendanceStudents(Long courseScheduleId);

    /**
     * @describe 停止vip课
     * @author Joburgess
     * @date 2019/10/21
     * @param vipGroupId: vip课编号
     * @param stopReason: 取消原因
     * @return void
     */
    void stopGroup(Long vipGroupId,String stopReason);

    /**
     * 发放月度奖励
     * @return
     */
    boolean awardedMonthlyRewards();
    
    /**
     * 根据老师查询vip课教学点
     * @param userId
     * @return
     */
    List<School> querySchoolByUserId(Integer userId);

    /**
     * @describe 更新vip课基本信息
     * @author Joburgess
     * @date 2019/11/11
     * @param vipGroupApplyBaseInfo:
     * @return void
     */
    void updateVipGroupBaseInfo(VipGroupApplyBaseInfoDto vipGroupApplyBaseInfo);

    /**
     * @describe 取消达到报名时间认为达到报名人数的vip课
     * @author Joburgess
     * @date 2019/11/13
     * @return void
     */
    void stopVipGroupWithNoCreate();
    
}
