package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.CourseScheduleQueryInfo;
import com.ym.mec.biz.dal.page.StudentPayLogQueryInfo;
import com.ym.mec.biz.dal.page.VipClassQueryInfo;
import com.ym.mec.biz.dal.page.queryMusicGroupCourseScheduleQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupMember;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.jiari.JiaRiFeignService;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateConvertor;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ClassGroupServiceImpl extends BaseServiceImpl<Integer, ClassGroup> implements ClassGroupService {

    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private SchoolDao schoolDao;
    @Autowired
    private VipGroupDao vipGroupDao;
    @Autowired
    private VipGroupActivityDao vipGroupActivityDao;
    @Autowired
    private TeacherAttendanceDao teacherAttendanceDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private MusicGroupBuildLogDao musicGroupBuildLogDao;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private ClassGroupRelationService classGroupRelationService;
    @Autowired
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;
    @Autowired
    private ClassGroupRelationDao classGroupRelationDao;
    @Autowired
    private ClassGroupTeacherMapperService classGroupTeacherMapperService;
    @Autowired
    private TeacherDefaultMusicGroupSalaryService teacherDefaultMusicGroupSalaryService;
    @Autowired
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private MusicGroupStudentFeeDao musicGroupStudentFeeDao;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private SubjectService subjectService;
    @Autowired
    private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
    @Autowired
    private CourseScheduleTeacherSalaryDao courseScheduleTeacherSalaryDao;
    @Autowired
    private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
    @Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    private ImFeignService imFeignService;
    @Autowired
    private JiaRiFeignService jiaRiFeignService;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private CourseScheduleService courseScheduleService;
    @Autowired
    private CourseScheduleStudentPaymentService courseScheduleStudentPaymentService;
    @Autowired
    private StudentApplyRefundsDao studentApplyRefundsDao;


    @Override
    public BaseDAO<Integer, ClassGroup> getDAO() {
        return classGroupDao;
    }

    @Override
    public List<ClassGroup> findClassGroup4Teacher(Integer teacherId) {
        return classGroupDao.findClassGroup4Teacher(teacherId);
    }

    @Override
    public List<ClassGroup> findClassGroup(String musicGroupId, Integer mixClassGroupId) {
        if (null == mixClassGroupId) {
            return classGroupDao.findAllMixClassGroup(musicGroupId);
        } else {
            return classGroupDao.findMixClassChildClassGroup(mixClassGroupId);
        }
    }

    @Override
    public List<ClassGroup> findAllNormalClassGroupByMusicGroupId(String musicGroupId) {
        return classGroupDao.findAllNormalClassGroupByMusicGroupId(musicGroupId);
    }

    @Override
    public List<ClassGroup> findAllMixClassGroupByMusicGroupId(String musicGroupId) {
        List<ClassGroup> allMixClassGroup = classGroupDao.findAllMixClassGroup(musicGroupId);
        for (ClassGroup mixClassGroup : allMixClassGroup) {
            ClassGroup mixClassChildClassGroup = classGroupDao.findMixClassChildClassGroupNames(mixClassGroup.getId());
            if (mixClassChildClassGroup != null) {
                mixClassGroup.setClassNames(mixClassChildClassGroup.getName());
            }
        }
        return allMixClassGroup;
    }

    @Override
    public List<ClassGroup> findAllClassGroupByMusicGroup(String musicGroupId) {
        return classGroupDao.findAllClassGroupByMusicGroup(musicGroupId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup addClassGroup(ClassGroup classGroup) throws Exception {
        String userIds = classGroup.getUserIds();
        if (userIds == null || userIds.isEmpty()) throw new Exception("班级学生不能为空");

        Set<String> userIdStrSet = new HashSet<>(Arrays.asList(userIds.split(",")));

        Date date = new Date();
        classGroup.setCreateTime(date);
        classGroup.setUpdateTime(date);
        classGroup.setType(ClassGroupTypeEnum.NORMAL);

        classGroup.setExpectStudentNum(userIdStrSet.size());
        classGroup.setStudentNum(userIdStrSet.size());
        insert(classGroup);

        //2、插入班级学生关联关系
        List<ImGroupMember> imGroupMemberList = new ArrayList<>();
        List<ClassGroupStudentMapper> classGroupStudentList = new ArrayList<>();
        for (String userId : userIdStrSet) {
            ClassGroupStudentMapper classGroupStudentMapper = new ClassGroupStudentMapper();
            classGroupStudentMapper.setMusicGroupId(classGroup.getMusicGroupId());
            classGroupStudentMapper.setClassGroupId(classGroup.getId());
            classGroupStudentMapper.setUserId(Integer.parseInt(userId));
            classGroupStudentMapper.setCreateTime(date);
            classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);
            classGroupStudentList.add(classGroupStudentMapper);

            StudentRegistration studentRegistration = new StudentRegistration();
            studentRegistration.setClassGroupId(classGroup.getId());
            studentRegistration.setUserId(Integer.parseInt(userId));
            studentRegistration.setMusicGroupId(classGroup.getMusicGroupId());
            studentRegistrationDao.updateByUserIdAndMusicGroupId(studentRegistration);
            imGroupMemberList.add(new ImGroupMember(userId));
        }
        imGroupMemberList.addAll(musicGroupService.queryMusicGroupTeachers(classGroup.getMusicGroupId()));
        classGroupStudentMapperDao.classGroupStudentsInsert(classGroupStudentList);
        ImGroupMember[] imGroupMembers = imGroupMemberList.toArray(new ImGroupMember[imGroupMemberList.size()]);
        // 创建群组
        imFeignService.groupCreate(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, classGroup.getName()));
        return classGroup;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup addMixClassGroup(String musicGroupId, String name, String classGroupIds) throws Exception {
        Date date;
        date = new Date();

        List<ClassGroup> classGroups = findClassGroupByIds(classGroupIds);
        int studentNum = 0; //学生数
        String subjectIds = "";
        List<Integer> classGroupIdList = new ArrayList<>();
        for (ClassGroup classGroup : classGroups) {
            studentNum += classGroup.getStudentNum();
            subjectIds += subjectIds.isEmpty() ? classGroup.getSubjectIdList() : "," + classGroup.getSubjectIdList();
            classGroupIdList.add(classGroup.getId());
        }

        //1、插入班级
        ClassGroup classGroup = new ClassGroup();
        classGroup.setMusicGroupId(musicGroupId);
        classGroup.setSubjectIdList(subjectIds);
        classGroup.setName(name);
        classGroup.setType(ClassGroupTypeEnum.MIX);
        classGroup.setExpectStudentNum(studentNum);
        classGroup.setStudentNum(studentNum);
        classGroup.setCreateTime(date);
        classGroup.setUpdateTime(date);
        insert(classGroup);

        List<ImGroupMember> imGroupMemberList = new ArrayList<>();
        List<ClassGroupStudentMapper> mixClassGroupStudentMappers = new ArrayList<>();
        List<ClassGroupStudentMapper> classGroupStudentMappers = classGroupStudentMapperDao.findByClassGroups(classGroupIdList);

        for (ClassGroupStudentMapper classGroupStudentMapper : classGroupStudentMappers) {
            ClassGroupStudentMapper mixClassGroupStudentMapper = new ClassGroupStudentMapper();
            if (classGroupStudentMapper.getStatus().equals(ClassGroupStudentStatusEnum.QUIT)) continue;
            mixClassGroupStudentMapper.setMusicGroupId(musicGroupId);
            mixClassGroupStudentMapper.setClassGroupId(classGroup.getId());
            mixClassGroupStudentMapper.setUserId(classGroupStudentMapper.getUserId());
            mixClassGroupStudentMapper.setCreateTime(date);
            mixClassGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);
            mixClassGroupStudentMappers.add(mixClassGroupStudentMapper);

            imGroupMemberList.add(new ImGroupMember(classGroupStudentMapper.getUserId().toString()));
        }
        //插入学生关联关系
        classGroupStudentMapperDao.classGroupStudentsInsert(mixClassGroupStudentMappers);

        //2、插入班级关联关系
        List<ClassGroupRelation> classGroupRelationList = new ArrayList<>();
        for (ClassGroup cGroup : classGroups) {
            ClassGroupRelation classGroupRelation = new ClassGroupRelation();
            classGroupRelation.setClassGroupId(classGroup.getId());
            classGroupRelation.setSubClassGroupId(cGroup.getId());
            classGroupRelation.setCreateTime(date);
            classGroupRelationList.add(classGroupRelation);
        }
        classGroupRelationService.classGroupRelationsInsert(classGroupRelationList);
        imGroupMemberList.addAll(musicGroupService.queryMusicGroupTeachers(classGroup.getMusicGroupId()));
        ImGroupMember[] imGroupMembers = imGroupMemberList.toArray(new ImGroupMember[imGroupMemberList.size()]);
        // 创建群组
        imFeignService.groupCreate(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, classGroup.getName()));
        return classGroup;
    }

    @Override
    public List<ClassGroup> findClassGroupByIds(String ids) {
        return classGroupDao.findClassGroupByIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<HighClassGroupDto> addHighClassGroup(List<HighClassGroupDto> highClassGroupList) throws Exception {

        Integer schoolId = null;

        for (HighClassGroupDto highClassGroup : highClassGroupList) {

            if (schoolId == null) {
                MusicGroup musicGroup = musicGroupDao.get(highClassGroup.getMusicGroupId());
                if (musicGroup != null) {
                    schoolId = musicGroup.getSchoolId();
                }
            }

            //1、插入班级信息
            Date date;
            date = new Date();
            highClassGroup.setCreateTime(date);
            highClassGroup.setUpdateTime(date);
            highClassGroup.setType(ClassGroupTypeEnum.HIGH);
            highClassGroup.setTotalClassTimes(highClassGroup.getCourseTimes());
            highClassGroup.setCurrentClassTimes(0);
            insert(highClassGroup);

            //2、插入班级关联老师
            ClassGroupTeacherMapper classGroupTeacherMapper = new ClassGroupTeacherMapper();
            classGroupTeacherMapper.setMusicGroupId(highClassGroup.getMusicGroupId());
            classGroupTeacherMapper.setClassGroupId(highClassGroup.getId());
            classGroupTeacherMapper.setTeacherRole(TeachTypeEnum.BISHOP);
            classGroupTeacherMapper.setUserId(highClassGroup.getUserId());
            classGroupTeacherMapper.setCreateTime(date);
            classGroupTeacherMapper.setUpdateTime(date);
            classGroupTeacherMapperService.insert(classGroupTeacherMapper);

            //3、插入班级排课信息
            List<CourseSchedule> courseScheduleList = new ArrayList<>();
            int times = 0;
            //LocalDateTime now = LocalDateTime.ofInstant(highClassGroup.getStartDate().toInstant(), ZoneId.systemDefault());
            LocalDateTime now = LocalDate.parse(highClassGroup.getStartDate(), DateTimeFormatter.ofPattern("yyyy-MM-dd")).atStartOfDay();


            List<Subject> subjectList = subjectService.findBySubjectByIdList(highClassGroup.getSubjectIdList());

            String courseScheduleName = subjectList.stream().map(subject -> subject.getName()).collect(Collectors.joining("/"));

            while (true) {
                int dayOfWeek = now.getDayOfWeek().getValue();
                if (highClassGroup.getDayOfWeek().equals(dayOfWeek)) {
                    CourseSchedule courseSchedule = new CourseSchedule();
                    Instant instant = now.atZone(ZoneId.systemDefault()).toInstant();
                    Date classDate = Date.from(instant);
                    String startClassTime = DateUtil.getDate(classDate) + " " + highClassGroup.getStartClassTime() + ":00";
                    String endClassTime = DateUtil.getDate(classDate) + " " + highClassGroup.getEndClassTime() + ":00";

                    courseSchedule.setSchoolId(schoolId);
                    courseSchedule.setClassGroupId(highClassGroup.getId());
                    courseSchedule.setStatus(CourseStatusEnum.NOT_START);
                    courseSchedule.setClassDate(classDate);
                    courseSchedule.setStartClassTime(DateUtil.stringToDate(startClassTime));
                    courseSchedule.setEndClassTime(DateUtil.stringToDate(endClassTime));
                    courseSchedule.setTeacherId(highClassGroup.getUserId());
                    courseSchedule.setActualTeacherId(highClassGroup.getUserId());
                    courseSchedule.setCreateTime(date);
                    courseSchedule.setUpdateTime(date);
                    courseSchedule.setTeachMode(TeachModeEnum.OFFLINE);
                    courseSchedule.setType(CourseSchedule.CourseScheduleType.HIGH);
                    courseSchedule.setName(courseScheduleName + "-" + CourseSchedule.CourseScheduleType.HIGH.getMsg());
                    courseScheduleList.add(courseSchedule);
                    times++;
                }
                now = now.plusDays(1);
                if (highClassGroup.getCourseTimes().equals(times)) {
                    break;
                }
            }
            //检测新排课冲突
            courseScheduleService.checkNewCourseSchedules(courseScheduleList, false);
            courseScheduleDao.batchAddCourseSchedules(courseScheduleList);
            List<ImGroupMember> groupMembers = musicGroupService.queryMusicGroupTeachers(highClassGroup.getMusicGroupId());
            groupMembers.add(new ImGroupMember(classGroupTeacherMapper.getUserId().toString()));
            ImGroupMember[] imGroupMembers = groupMembers.toArray(new ImGroupMember[groupMembers.size()]);
            // 创建群组
            imFeignService.groupCreate(new ImGroupModel(highClassGroup.getId().toString(), imGroupMembers, highClassGroup.getName()));
        }
        return highClassGroupList;
    }

    @Override
    public PageInfo<TeacherMusicClassInfoDto> getTeacherMusicClass(QueryInfo queryInfo) {
        PageInfo<TeacherMusicClassInfoDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<TeacherMusicClassInfoDto> dataList = null;
        int count = classGroupDao.countGroupCourses(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = classGroupDao.queryGroupCourses(params);
            HashMap<String, Object> param = new HashMap<>();
            param.put("teacherId", queryInfo.getSearch());
            param.put("attendanceStatus", YesOrNoEnum.YES);
            dataList.forEach(e -> {
                param.put("musicGroupId", e.getMusicGroupId());
                e.setAttendanceNum(teacherAttendanceDao.getTeacherPersonalAttendancesCount(param));
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<TeacherVipClassInfoDto> getTeacherVipClass(VipClassQueryInfo queryInfo) {
        PageInfo<TeacherVipClassInfoDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<TeacherVipClassInfoDto> dataList = null;
        int count = vipGroupDao.countTeacherVipClass(queryInfo.getTeacherId());
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = vipGroupDao.getTeacherVipClass(params);
//            Set<Integer> activityIds = dataList.stream().map(e -> e.getActivityId()).collect(Collectors.toSet());
//            List<Map<Integer, String>> names = vipGroupActivityDao.queryNamesById(StringUtils.join(activityIds, ","));
//            Map<Integer, String> nameMap = MapUtil.convertMybatisMap(names);
            //计算月度平均消耗值(已上课时 / 消耗月份)
            List<Long> vipGroupIds = dataList.stream().map(vipGroup -> vipGroup.getVipClassId()).collect(Collectors.toList());
            List<Map<Long, Integer>> vipGroupOverCourses = vipGroupDao.countVipGroupOverCourse(vipGroupIds);
            Map<Long, Integer> vipGroupOverCourseMaps = MapUtil.convertIntegerMap(vipGroupOverCourses);
            dataList.forEach(vipGroup -> {
                Integer overCourses = vipGroupOverCourseMaps.get(vipGroup.getVipClassId());
                if (Objects.isNull(overCourses)) {
                    overCourses = 0;
                }
                Integer totalClassTimes = vipGroup.getTotalClassTimes();
                if (Objects.isNull(totalClassTimes)) {
                    vipGroup.setMonthAvg(Float.parseFloat("0.0"));
                } else {
                    if (overCourses <= 0) {
                        vipGroup.setMonthAvg(0);
                    } else {
                        BigDecimal overCoursesOfMonth = new BigDecimal(overCourses), allClassTimes = new BigDecimal(totalClassTimes);

                        Float monthConsumeRate = overCoursesOfMonth.divide(overCoursesOfMonth, 2, BigDecimal.ROUND_HALF_UP).floatValue();

                        vipGroup.setMonthAvg(monthConsumeRate);
                    }

                }
            });

        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<ClassGroupTeachersDto> getClassGroupAndTeachers(String musicGroupId, String type, Integer classGroupId) {
        List<ClassGroupTeachersDto> classGroupAndTeachersList = classGroupDao.findClassGroupByMusicGroupId(musicGroupId, type, classGroupId);

        List<ClassGroupTeacherMapper> classGroupTeacherMapperList = classGroupTeacherMapperService.findClassGroupTeachers(classGroupAndTeachersList);
        //获取当前乐团的合奏班id列表map
        HashMap<Integer, Integer> idMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(classGroupRelationDao.queryClassRelationIdMap(musicGroupId))), HashMap.class);
        HashMap<Integer, String> nameMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertMybatisMap(classGroupRelationDao.queryClassRelationNameMap(musicGroupId))), HashMap.class);

        for (ClassGroupTeachersDto classGroupAndTeachers : classGroupAndTeachersList) {
            List<ClassGroupTeacherMapper> oneClassGroupTeacherMapperList = new ArrayList<>();
            for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
                if (classGroupTeacherMapper.getClassGroupId().equals(classGroupAndTeachers.getId())) {
                    oneClassGroupTeacherMapperList.add(classGroupTeacherMapper);
                }
            }
            classGroupAndTeachers.setMixClassName(nameMap.get(classGroupAndTeachers.getId()));
            classGroupAndTeachers.setMixClassId(idMap.get(classGroupAndTeachers.getId()));
            classGroupAndTeachers.setClassGroupTeacherMapperList(oneClassGroupTeacherMapperList);
        }
        return classGroupAndTeachersList;
    }

    @Override
    public List<ClassGroupTeachersDto> getClassGroupAndTeacherSalary(String musicGroupId, SalarySettlementTypeEnum type, Integer classGroupId) throws Exception {
        //乐团班级及班级老师
        List<ClassGroupTeachersDto> classGroupAndTeachers = getClassGroupAndTeachers(musicGroupId, "NORMAL,MIX,HIGH", classGroupId);
        BigDecimal zeroSalary = new BigDecimal(0);
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        for (ClassGroupTeachersDto classGroupAndTeacher : classGroupAndTeachers) {
            //班级的教师列表
            List<ClassGroupTeacherMapper> classGroupTeacherMapperList = classGroupAndTeacher.getClassGroupTeacherMapperList();
            for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
                //获取老师的默认课酬
                String courseType = classGroupAndTeacher.getType().equals(ClassGroupTypeEnum.NORMAL) ? "SINGLE" : classGroupAndTeacher.getType().getCode();

                if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.CLASSROOM_SALARY)) {
                    courseType = "CLASSROOM";
                }

                List<TeacherDefaultMusicGroupSalary> teacherSalaryByUserIdAndType = teacherDefaultMusicGroupSalaryService.getTeacherSalaryByUserIdAndType(classGroupTeacherMapper.getUserId(), courseType, type);
                //老师课酬没有设置，设置成0
                if (teacherSalaryByUserIdAndType == null || teacherSalaryByUserIdAndType.size() == 0) {
                    TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalary = new TeacherDefaultMusicGroupSalary();
                    teacherDefaultMusicGroupSalary.setAssistantTeacher30MinSalary(zeroSalary);
                    teacherDefaultMusicGroupSalary.setMainTeacher30MinSalary(zeroSalary);
                    teacherDefaultMusicGroupSalary.setUserId(classGroupTeacherMapper.getUserId());
                    teacherSalaryByUserIdAndType.add(teacherDefaultMusicGroupSalary);
                }
                classGroupTeacherMapper.setTeacherDefaultMusicGroupSalaryList(teacherSalaryByUserIdAndType);
            }

        }
        return classGroupAndTeachers;
    }

    @Override
    public PageInfo<CourseListDto> queryCoursePage(QueryInfo queryInfo, Integer userId) {
        queryInfo.setSearch(userId.toString());
        PageInfo<CourseListDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        params.put("userId", userId);
        List<CourseListDto> courseListDtos = null;
        //获取分页前的所有的乐团编号和vip编号
        params.put("offset", pageInfo.getOffset());
        List<String> musicGroupIds = courseScheduleDao.queryMusicGroupIds(params);
        courseListDtos = new ArrayList<>();
        //获取学员所购买的vip课列表详情
        List<CourseListDto> vipCourses = classGroupDao.queryStudentVipCourses(StringUtils.join(musicGroupIds, ","),userId);
        //获取vip课的学生列表
        Set<String> vipIds = vipCourses.stream().map(CourseListDto::getMusicGroupId).collect(Collectors.toSet());
        Map<String, String> vipStuNames = MapUtil.convertMybatisMap(classGroupDao.findVipStuNames(StringUtils.join(vipIds, ",")));
        //获取学员的乐团列表详情
        courseListDtos.addAll(classGroupDao.queryStudentMusicGroupCourses(StringUtils.join(musicGroupIds, ","),userId));
        courseListDtos.addAll(vipCourses);
        //检测对应乐团是否已申请退费
        List<Map<String, Integer>> isApplyMaps;
        if(musicGroupIds != null && musicGroupIds.size() > 0){
            isApplyMaps = studentApplyRefundsDao.checkIsApplyRefund(musicGroupIds, userId);
        }else {
            isApplyMaps = new ArrayList<>();
        }
        Map<String, Integer> isApplyMap = MapUtil.convertIntegerMap(isApplyMaps);
        //获取所有教学点名称列表
        Set<Integer> schoolIds = courseListDtos.stream().map(CourseListDto::getTeacherSchoolId).collect(Collectors.toSet());
        Map<Integer, String> schoolNames = MapUtil.convertMybatisMap(schoolDao.queryNameByIds(StringUtils.join(schoolIds, ",")));

        courseListDtos.forEach(e -> {
            e.setAddress(schoolNames.get(e.getTeacherSchoolId()));
            e.setIsApply(isApplyMap.get(e.getMusicGroupId()));
            if (ClassGroupTypeEnum.VIP == e.getType()) {
                e.setStudentNames(vipStuNames.get(e.getMusicGroupId()));
                if (e.getOffLineClassesNum() > 0 && e.getOffLineClassesNum() <= 0) {
                    e.setClassMode(1);
                } else if (e.getOffLineClassesNum() > 0 && e.getOffLineClassesNum() > 0) {
                    e.setClassMode(3);
                } else if (e.getOnlineClassesNum() < 0 && e.getOffLineClassesNum() >= 0) {
                    e.setClassMode(2);
                } else {
                    e.setClassMode(0);
                }
            }
        });
        pageInfo.setTotal(courseListDtos.size());
        if (courseListDtos.size() == 0) {
            courseListDtos = new ArrayList<>();
        }
        pageInfo.setRows(courseListDtos);
        return pageInfo;
    }

    @Override
    public List<ImGroupModel> queryUserGroups(Integer userId, String search) {
        /*List<ImGroupModel> imGroupModels = musicGroupDao.queryUserGroups(userId, search);

        if (imGroupModels != null && imGroupModels.size() > 0) {
            //获取相关的所有乐团
            Set<String> musicGroupIds = imGroupModels.stream().map(e -> e.getId()).collect(Collectors.toSet());
            List<MusicGroup> musicGroups = musicGroupDao.queryListByIds(musicGroupIds);

            //获取乐团群用户总数map
            List<Map<String, BigDecimal>> musicGroupUserNums = musicGroupDao.countMusicGroupUserNum(StringUtils.join(musicGroupIds, ","));
            Map<String, BigDecimal> musicGroupUserNumMap = MapUtil.convertMybatisMap(musicGroupUserNums);
            MusicGroup musicGroup;
            Set<Integer> teachers;
            for (ImGroupModel e : imGroupModels) {
                teachers = e.getName() == null ? new HashSet<>() : new HashSet(Arrays.asList(e.getName().split(",")));
                musicGroup = musicGroups.stream().filter(mg -> mg.getId().equals(e.getId())).findFirst().get();
                //获取教学老师id列表
                teachers.add(musicGroup.getTeamTeacherId());
                teachers.add(musicGroup.getEducationalTeacherId());
                teachers.add(musicGroup.getOperatorUserId());
                teachers.add(musicGroup.getDirectorUserId());
                teachers.removeAll(Collections.singleton(null));
                //获取乐团群用户总数
                BigDecimal count = musicGroupUserNumMap.get(e.getId());
                e.setCount(count == null ? 0 : count.intValue() + teachers.size());
            }
        } else {
            imGroupModels = new ArrayList<>();
        }*/
        List<ImGroupModel> imGroupModels = classGroupDao.queryUserGroups(userId, search);
        if (imGroupModels != null && imGroupModels.size() > 0) {
            Set<String> classGroupIds = imGroupModels.stream().map(e -> e.getId()).collect(Collectors.toSet());
            //获取班级老师总数map
            Map<Integer, Integer> numsMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(classGroupDao.countTeacherNums(StringUtils.join(classGroupIds, ",")))),HashMap.class);
            imGroupModels.forEach(e -> {
                Integer num = numsMap.get(Integer.parseInt(e.getId()));
                e.setCount((num == null ? 0 : num) + e.getCount());
            });
        }
        return imGroupModels;
    }

    @Override
    public List<ImUserModel> findGroupUsers(Integer groupId) {
        List<ImUserModel> students = classGroupDao.findGroupUsers(groupId);
        if (students == null) {
            students = new ArrayList<>();
        }
        List<ImUserModel> teachers = classGroupDao.findGroupTeacher(groupId);
        students.addAll(teachers);
        return students;
    }

    @Override
    public PageInfo<StudentPayInfoDto> queryStudentPayLog(StudentPayLogQueryInfo queryInfo) {
        PageInfo<StudentPayInfoDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<StudentPayInfoDto> dataList = null;
        int count = classGroupDao.countStudentPayLog(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = classGroupDao.queryStudentPayLog(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup addStudent(ClassGroup classGroup, Integer userId) {
        Date date = new Date();
        //1、增加班级人数
        classGroup.setCreateTime(date);
        classGroupDao.addStudentNum(classGroup);
        //2、插入关联关系
        ClassGroupStudentMapper classGroupStudentMapper = new ClassGroupStudentMapper();
        classGroupStudentMapper.setMusicGroupId(classGroup.getMusicGroupId());
        classGroupStudentMapper.setClassGroupId(classGroup.getId());
        classGroupStudentMapper.setUserId(userId);
        classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);
        classGroupStudentMapper.setCreateTime(date);
        classGroupStudentMapperDao.insert(classGroupStudentMapper);

        //3、学生加入级未开始课程
        List<CourseSchedule> courseScheduleList = courseScheduleService.findNoStartCoursesByClassGroupId(classGroup.getId());
        BigDecimal coursePrice = new BigDecimal("0");
        List<CourseScheduleStudentPayment> courseScheduleStudentPayments = new ArrayList<>();
        for (CourseSchedule courseSchedule : courseScheduleList) {
            CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
            courseScheduleStudentPayment.setCourseScheduleId(courseSchedule.getId());
            courseScheduleStudentPayment.setUserId(userId);
            courseScheduleStudentPayment.setExpectPrice(coursePrice);
            courseScheduleStudentPayment.setClassGroupId(classGroup.getId());
            courseScheduleStudentPayment.setCreateTime(date);
            courseScheduleStudentPayment.setUpdateTime(date);
            courseScheduleStudentPayments.add(courseScheduleStudentPayment);
        }

        if (courseScheduleStudentPayments.size() != 0) {
            courseScheduleStudentPaymentService.batchInsert(courseScheduleStudentPayments);
        }

        //3、加入班级群组
        ImGroupMember[] imGroupMembers = new ImGroupMember[]{new ImGroupMember(userId.toString())};
        imFeignService.groupJoin(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, classGroup.getName()));

        return classGroup;
    }

    @Override
    public List<ClassGroup> findNoClassSubjects(String musicGroupId) {
        return classGroupDao.findNoClassSubjects(musicGroupId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delSingle(Integer classGroupId) {
        //删除学员班级关联关系
        classGroupStudentMapperDao.deleteByClassId(classGroupId);
        studentRegistrationDao.deleteClassGroup(classGroupId);
        //删除班级跟合奏班关系
        classGroupRelationDao.deleteBySubClassId(classGroupId);
        //删除班级
        classGroupDao.delete(classGroupId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delMix(Integer classGroupId) {
        //删除学员班级关联关系
        classGroupStudentMapperDao.deleteByClassId(classGroupId);
        //删除班级关系
        classGroupRelationDao.deleteByClassId(classGroupId);
        //删除合奏班
        classGroupDao.delete(classGroupId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addMusicGroupTeam(Integer teacherId, String musicGroupId, Integer improventClassesNum) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            throw new Exception("用户信息获取失败");
        }
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        if (musicGroup == null) {
            throw new Exception("乐团信息不存在");
        }
        if (musicGroup.getStatus() != MusicGroupStatusEnum.PREPARE) {
            throw new Exception("当前乐团状态不允许此操作");
        }
        //所有学员都已经分配
        List<StudentRegistration> students = studentRegistrationDao.getNoClassStuBySubjectId(musicGroupId, null);
        if (students != null && students.size() > 0) {
            throw new Exception("还有学员未分配班级");
        }
        //所有单技班都有合奏班
        List<ClassGroup> classGroupList = classGroupDao.findNoClassSubjects(musicGroupId);
        if (classGroupList != null && classGroupList.size() > 0) {
            throw new Exception("还有单技班未分配合奏");
        }
        //设置教学主管
        musicGroup.setDirectorUserId(teacherId);
        musicGroup.setImproventClassesNum(improventClassesNum);
        musicGroup.setUpdateTime(new Date());
        musicGroupDao.update(musicGroup);
        musicGroupBuildLogDao.insert(new MusicGroupBuildLog(musicGroupId, "成团确认（学员、班级、教学主管已设置）", sysUser.getId(), ""));
    }

    @Override
    public List<TeacherClassGroupDto> findTeacherClassGroups(String type) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (Objects.isNull(user)) {
            throw new BizException("请登录");
        }

        if (StringUtils.isBlank(type)) {
            throw new BizException("请指定课程类型");
        }

        List<TeacherClassGroupDto> teacherMusicClassGroup;
        switch (type) {
            case "VIP":
                teacherMusicClassGroup = classGroupDao.findTeacherVipClassGroup(user.getId().longValue());
                break;
            case "MUSIC_GROUP":
                teacherMusicClassGroup = classGroupDao.findTeacherMusicClassGroup(user.getId().longValue());
                break;
            default:
                throw new BizException("课程类型错误");
        }
        if (CollectionUtils.isEmpty(teacherMusicClassGroup)) {
            return teacherMusicClassGroup;
        }

        List<String> subjectIdsList = teacherMusicClassGroup.stream().map(teacherClassGroupDto -> teacherClassGroupDto.getSubjectIdList()).collect(Collectors.toList());
        Map<Long, String> subjectMapBySubjectIdsList = subjectService.findSubjectMapBySubjectIdsList(subjectIdsList);

        List<Long> musicGroupIds = teacherMusicClassGroup.stream().map(teacherClassGroupDto -> teacherClassGroupDto.getMusicGroupId()).collect(Collectors.toList());
        List<Map<Long, Integer>> musicGroupIdAndNumList = musicGroupStudentFeeDao.countContinuosAbsenteeismStudentNum(musicGroupIds);
        Map<Long, Integer> musicGroupIdAndNum = MapUtil.convertIntegerMap(musicGroupIdAndNumList);

        List<Map<Long, Integer>> vipGroupOverCourses = vipGroupDao.countVipGroupOverCourse(musicGroupIds);
        Map<Long, Integer> vipGroupOverCourseMaps = MapUtil.convertIntegerMap(vipGroupOverCourses);

        teacherMusicClassGroup.forEach(teacherClassGroupDto -> {

            if (teacherClassGroupDto.getType() == ClassGroupTypeEnum.VIP) {
                String[] studentNums = classGroupStudentMapperDao.findStudentNumByClassGroupId(teacherClassGroupDto.getClassGroupId());
                teacherClassGroupDto.setStudentNames(StringUtils.join(studentNums, ","));
                if (teacherClassGroupDto.getOffLineClassesNum() > 0 && teacherClassGroupDto.getOffLineClassesNum() <= 0) {
                    teacherClassGroupDto.setClassMode(1);
                } else if (teacherClassGroupDto.getOnlineClassesNum() > 0 && teacherClassGroupDto.getOffLineClassesNum() > 0) {
                    teacherClassGroupDto.setClassMode(3);
                } else if (teacherClassGroupDto.getOnlineClassesNum() <= 0 && teacherClassGroupDto.getOffLineClassesNum() > 0) {
                    teacherClassGroupDto.setClassMode(2);
                } else {
                    teacherClassGroupDto.setClassMode(0);
                }

                Integer overCourses = vipGroupOverCourseMaps.get(teacherClassGroupDto.getMusicGroupId());
                if (Objects.isNull(overCourses)) {
                    overCourses = 0;
                }
                Integer totalClassTimes = teacherClassGroupDto.getTotalClassTimes();
                if (Objects.isNull(totalClassTimes) || totalClassTimes == 0) {
                    teacherClassGroupDto.setAvgClassTimesOfMonth(0.0);
                } else {
                    BigDecimal overCoursesOfMonth = new BigDecimal(overCourses), allClassTimes = new BigDecimal(totalClassTimes);

                    Double monthConsumeRate = overCoursesOfMonth.divide(allClassTimes, 1, BigDecimal.ROUND_HALF_UP).doubleValue();
                    teacherClassGroupDto.setAvgClassTimesOfMonth(monthConsumeRate);
                }

            }
            String subjectIdList = teacherClassGroupDto.getSubjectIdList();
            if (StringUtils.isNotEmpty(subjectIdList)) {
                if (Objects.nonNull(musicGroupIdAndNum) && Objects.nonNull(musicGroupIdAndNum.get(teacherClassGroupDto.getMusicGroupId()))) {
                    teacherClassGroupDto.setThreeClassNoAttendanceStudentNum(musicGroupIdAndNum.get(teacherClassGroupDto.getMusicGroupId()));
                }
                String[] subjectIds = subjectIdList.split(",");
                List<String> subjectNameList = new ArrayList<>();
                for (String subjectId : subjectIds) {
                    subjectNameList.add(subjectMapBySubjectIdsList.get(Integer.parseInt(subjectId)));
                }
                teacherClassGroupDto.setSubjectNames(StringUtils.join(subjectNameList, ","));
            }
        });

        return teacherMusicClassGroup;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public boolean updateClassStudentNum(Long classGroupId, Integer num) {
        ClassGroup classGroup = classGroupDao.lockClassGroup(classGroupId);
        if (Objects.isNull(classGroup)) {
            throw new BizException("班级不存在");
        }
        classGroup.setStudentNum(classGroup.getStudentNum() + num);
        classGroup.setUpdateTime(new Date());

        classGroupDao.update(classGroup);

        return true;
    }

    @Override
    public TeacherClassHeadInfo findTeacherClassGroupInfo(Long classGroupId) {
        if (Objects.isNull(classGroupId)) {
            throw new BizException("请指定班级");
        }

        ClassGroup classGroup = classGroupDao.get(classGroupId.intValue());

        if (Objects.isNull(classGroup)) {
            throw new BizException("未找到该班级");
        }

        TeacherClassHeadInfo teacherClassHeadInfo = new TeacherClassHeadInfo();
        teacherClassHeadInfo.setStudentNum(classGroup.getStudentNum());
        teacherClassHeadInfo.setCurrentClassTimes(classGroup.getCurrentClassTimes());
        teacherClassHeadInfo.setTotalClassTimes(classGroup.getTotalClassTimes());
        ClassGroupTeacherMapper bishopTeacher = classGroupTeacherMapperDao.findByClassGroupAndRole(classGroupId, TeachTypeEnum.BISHOP);
        if (Objects.nonNull(bishopTeacher)) {
            teacherClassHeadInfo.setBishopTeacherId(bishopTeacher.getId());
            teacherClassHeadInfo.setBishopTeacher(bishopTeacher.getUserName());
        }
        ClassGroupTeacherMapper teachingTeacher = classGroupTeacherMapperDao.findByClassGroupAndRole(classGroupId, TeachTypeEnum.TEACHING);
        if (Objects.nonNull(teachingTeacher)) {
            teacherClassHeadInfo.setTeachingTeacherId(teachingTeacher.getId());
            teacherClassHeadInfo.setTeachingTeacher(teachingTeacher.getUserName());
        }

        if (classGroup.getType() == ClassGroupTypeEnum.VIP) {
            String[] studentNums = classGroupStudentMapperDao.findStudentNumByClassGroupId(classGroupId);
            teacherClassHeadInfo.setStudentNames(StringUtils.join(studentNums, ","));
        }

        return teacherClassHeadInfo;
    }

    @Override
    public PageInfo findTeacherClassStudents(CourseScheduleQueryInfo queryInfo) {
        PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List dataList = null;
        int count = classGroupStudentMapperDao.findClassStudentNum(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = classGroupStudentMapperDao.findClassStudent(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<StudentAttendanceViewDto> findAttendanceStudentByCourse(Long courseScheduleId) {
        if (Objects.isNull(courseScheduleId)) {
            throw new BizException("请指定课程编号");
        }
        List<StudentAttendanceViewDto> studentByCourse = classGroupStudentMapperDao.findStudentByCourse(courseScheduleId);
        List<StudentAttendanceViewDto> leaveStudent = studentByCourse.stream()
                .filter(studentAttendanceViewDto -> StudentAttendanceStatusEnum.LEAVE.equals(studentAttendanceViewDto.getStatus()))
                .collect(Collectors.toList());
        List<StudentAttendanceViewDto> truantStudent = studentByCourse.stream()
                .filter(studentAttendanceViewDto -> StudentAttendanceStatusEnum.TRUANT.equals(studentAttendanceViewDto.getStatus()))
                .collect(Collectors.toList());
        List<StudentAttendanceViewDto> otherStudent = studentByCourse.stream()
                .filter(studentAttendanceViewDto -> !StudentAttendanceStatusEnum.LEAVE.equals(studentAttendanceViewDto.getStatus())
                        && !StudentAttendanceStatusEnum.TRUANT.equals(studentAttendanceViewDto.getStatus()))
                .collect(Collectors.toList());
        truantStudent.addAll(otherStudent);
        truantStudent.addAll(leaveStudent);
        return truantStudent;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup classGroupAdjust(ClassGroup4MixDto classGroup4MixDto) throws Exception {
        Date date = new Date();
        String musicGroupId = classGroup4MixDto.getMusicGroupId();
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        if (musicGroup == null) {
            throw new Exception("乐团不存在");
        }
        Integer schoolId = musicGroup.getSchoolId();

        List<Integer> subjectIdSet = new ArrayList<>();

        List<Integer> studentIdList = classGroup4MixDto.getStudents();
        List<StudentRegistration> studentList = studentRegistrationService.findStudentListByUserIdList(musicGroupId, studentIdList);
        for (StudentRegistration student : studentList) {
            subjectIdSet.add(student.getSubjectId());
        }
        List<Subject> subjectList = subjectService.findBySubjectByIdList(subjectIdSet);

        String subjectIds = subjectList.stream().map(subject -> subject.getId().toString()).collect(Collectors.joining(","));
        String subjectNames = subjectList.stream().map(subject -> subject.getName()).collect(Collectors.joining("/"));

        //1、新建班级
        ClassGroup classGroup = new ClassGroup();
        classGroup.setMusicGroupId(classGroup4MixDto.getMusicGroupId());
        classGroup.setSubjectIdList(subjectIds);
        classGroup.setName(classGroup4MixDto.getClassGroupName());
        classGroup.setExpectStudentNum(studentList.size());
        classGroup.setStudentNum(studentList.size());
        classGroup.setType(ClassGroupTypeEnum.NORMAL);
        classGroup.setDelFlag(YesOrNoEnum.NO);
        classGroup.setCurrentClassTimes(0);
        classGroup.setTotalClassTimes(classGroup4MixDto.getCourseTimes());
        classGroup.setCreateTime(date);
        classGroup.setUpdateTime(date);
        classGroupDao.insert(classGroup);


        //2、将学生加入新班级（学生注册表，关联表
        List<ClassGroupStudentMapper> classGroupStudentMapperList = new ArrayList<>();
        for (StudentRegistration studentRegistration : studentList) {
            studentRegistration.setClassGroupId(classGroup.getId());
            studentRegistrationService.update(studentRegistration);

            ClassGroupStudentMapper classGroupStudentMapper = new ClassGroupStudentMapper();
            classGroupStudentMapper.setMusicGroupId(musicGroupId);
            classGroupStudentMapper.setClassGroupId(classGroup.getId());
            classGroupStudentMapper.setUserId(studentRegistration.getUserId());
            classGroupStudentMapper.setCreateTime(date);
            classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);

            classGroupStudentMapperList.add(classGroupStudentMapper);
        }
        classGroupStudentMapperDao.classGroupStudentsInsert(classGroupStudentMapperList);


        //3、将老师加入关联关系
        List<ClassGroupTeacherMapper> classGroupTeacherMapperList = classGroup4MixDto.getClassGroupTeacherMapperList();


        for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
            classGroupTeacherMapper.setClassGroupId(classGroup.getId());
            classGroupTeacherMapper.setMusicGroupId(classGroup4MixDto.getMusicGroupId());
        }
        classGroupTeacherMapperDao.classGroupTeachersInsert(classGroupTeacherMapperList);

        //5、插入班级排课信息
        LocalDateTime now = LocalDate.parse(classGroup4MixDto.getStartDate(), DateTimeFormatter.ofPattern("yyyy-MM-dd")).atStartOfDay();

        //计算每节课的课酬
        List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();
        List<CourseScheduleStudentPayment> courseScheduleStudentPaymentList = new ArrayList<>();
        BigDecimal expectPrice = new BigDecimal("0");
        int times = 0;

        Map<String, Integer> holidayDays = new HashMap<>();
        if (classGroup4MixDto.getHoliday()) {
            Map<Integer, Map<String, Integer>> holiday = jiaRiFeignService.query(now.getYear());
            holidayDays = holiday.get(now.getYear());
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        WhileNode:
        while (true) {
            if (classGroup4MixDto.getHoliday() && holidayDays.containsKey(now.format(DateTimeFormatter.ofPattern("MM-dd")))) {
                now = now.plusDays(1);
                continue;
            }
            int dayOfWeek = now.getDayOfWeek().getValue();//当前星期
            for (CourseTimeDto courseTimeDto : classGroup4MixDto.getCourseTimeDtoList()) {
                if (!courseTimeDto.getDayOfWeek().equals(dayOfWeek)) continue;

                //课时长度
                long classCourseDuration = Duration.between(LocalDateTime.parse(classGroup4MixDto.getStartDate() + " " + courseTimeDto.getStartClassTime() + ":00", formatter),
                        LocalDateTime.parse(classGroup4MixDto.getStartDate() + " " + courseTimeDto.getEndClassTime() + ":00", formatter))
                        .toMinutes();


                Date classDate = DateConvertor.toDate(now);
                String startClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getStartClassTime() + ":00";
                String endClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getEndClassTime() + ":00";

                CourseSchedule courseSchedule = new CourseSchedule();
                courseSchedule.setSchoolId(schoolId);
                courseSchedule.setClassGroupId(classGroup.getId());
                courseSchedule.setStatus(CourseStatusEnum.NOT_START);
                courseSchedule.setClassDate(classDate);
                courseSchedule.setStartClassTime(DateUtil.stringToDate(startClassTime));
                courseSchedule.setEndClassTime(DateUtil.stringToDate(endClassTime));
                courseSchedule.setCreateTime(date);
                courseSchedule.setUpdateTime(date);
                courseSchedule.setTeachMode(TeachModeEnum.OFFLINE);
                courseSchedule.setType(classGroup4MixDto.getCourseType());
                courseSchedule.setName(subjectNames + "-" + CourseSchedule.CourseScheduleType.SINGLE.getMsg());

                courseScheduleDao.insert(courseSchedule);

                times++;

                for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
                    List<TeacherDefaultMusicGroupSalary> teacherSalaryList = teacherDefaultMusicGroupSalaryService.getTeacherSalaryByUserIdAndType(classGroupTeacherMapper.getUserId(), "SINGLE", musicGroup.getSettlementType());
                    BigDecimal salary = new BigDecimal("0");

                    Map<CourseSchedule.CourseScheduleType, TeacherDefaultMusicGroupSalary> collect = teacherSalaryList.stream().collect(Collectors.toMap(TeacherDefaultMusicGroupSalary::getCourseScheduleType, teacherDefaultMusicGroupSalary -> teacherDefaultMusicGroupSalary));

                    TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalary = collect.get(classGroup4MixDto.getCourseType());

                    //对应基准课酬
                    BigDecimal baseSalary = new BigDecimal(0), classTimeDuty = new BigDecimal(0);

                    //基准课酬
                    if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.TEACHER_DEFAULT) && teacherDefaultMusicGroupSalary != null) {
                        //课程时长与结算单位时长占比
                        classTimeDuty = new BigDecimal(classCourseDuration).divide(new BigDecimal(30));
                        baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher30MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher30MinSalary();
                    }
                    //阶梯课酬
                    if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.GRADIENT_SALARY) && teacherDefaultMusicGroupSalary != null) {
                        //课程时长与结算单位时长占比
                        classTimeDuty = new BigDecimal(classCourseDuration).divide(new BigDecimal(90));
                        baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher90MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher90MinSalary();
                    }
                    salary = baseSalary.multiply(classTimeDuty).setScale(2, BigDecimal.ROUND_HALF_UP);

                    CourseScheduleTeacherSalary courseScheduleTeacherSalary = new CourseScheduleTeacherSalary();
                    courseScheduleTeacherSalary.setCourseScheduleId(courseSchedule.getId());
                    courseScheduleTeacherSalary.setTeacherRole(classGroupTeacherMapper.getTeacherRole());
                    courseScheduleTeacherSalary.setUserId(classGroupTeacherMapper.getUserId());
                    courseScheduleTeacherSalary.setExpectSalary(salary);
                    courseScheduleTeacherSalary.setClassGroupId(classGroup.getId());
                    courseScheduleTeacherSalary.setCreateTime(date);
                    courseScheduleTeacherSalary.setUpdateTime(date);
                    courseScheduleTeacherSalaryList.add(courseScheduleTeacherSalary);
                }
                for (StudentRegistration studentRegistration : studentList) {
                    CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
                    courseScheduleStudentPayment.setCourseScheduleId(courseSchedule.getId());
                    courseScheduleStudentPayment.setUserId(studentRegistration.getUserId());
                    courseScheduleStudentPayment.setExpectPrice(expectPrice);
                    courseScheduleStudentPayment.setCreateTime(date);
                    courseScheduleStudentPayment.setUpdateTime(date);
                    courseScheduleStudentPayment.setClassGroupId(classGroup.getId());
                    courseScheduleStudentPaymentList.add(courseScheduleStudentPayment);
                }
                if (classGroup4MixDto.getCourseTimes().equals(times)) {
                    break WhileNode;
                }
            }
            now = now.plusDays(1);
        }
        //老师结算表
        courseScheduleTeacherSalaryService.batchInsert(courseScheduleTeacherSalaryList);

        //加入合奏班
        ClassGroupRelation classGroupRelation = new ClassGroupRelation();
        classGroupRelation.setClassGroupId(classGroup4MixDto.getMixClassGroupId());
        classGroupRelation.setSubClassGroupId(classGroup.getId());
        classGroupRelation.setCreateTime(date);
        classGroupRelationService.insert(classGroupRelation);

        //6、添加学生未上合奏课课程
        List<CourseSchedule> noStartCourses = courseScheduleDao.findNoStartCoursesByClassGroupId(classGroup4MixDto.getMixClassGroupId());
        for (CourseSchedule noStartCourse : noStartCourses) {
            for (StudentRegistration studentRegistration : studentList) {
                CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
                courseScheduleStudentPayment.setCourseScheduleId(noStartCourse.getId());
                courseScheduleStudentPayment.setUserId(studentRegistration.getUserId());
                courseScheduleStudentPayment.setExpectPrice(expectPrice);
                courseScheduleStudentPayment.setCreateTime(date);
                courseScheduleStudentPayment.setUpdateTime(date);
                courseScheduleStudentPayment.setClassGroupId(noStartCourse.getClassGroupId());
                courseScheduleStudentPaymentList.add(courseScheduleStudentPayment);
            }
        }
        //学生结算表
        courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPaymentList);
        return classGroup;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup classGroupUpdate(ClassGroup4MixDto classGroup4MixDto) throws Exception {
        Date date = new Date();
        String musicGroupId = classGroup4MixDto.getMusicGroupId();
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        if (musicGroup == null) {
            throw new Exception("乐团不存在");
        }
        ClassGroup classgroup = classGroupDao.get(classGroup4MixDto.getClassGroupId());
        if (classgroup == null) {
            throw new Exception("班级不存在");
        }

        Integer schoolId = musicGroup.getSchoolId();

        List<Integer> subjectIdSet = new ArrayList<>();

        List<Integer> studentIdList = classGroup4MixDto.getStudents();
        List<StudentRegistration> studentList = studentRegistrationService.findStudentListByUserIdList(musicGroupId, studentIdList);
        for (StudentRegistration student : studentList) {
            subjectIdSet.add(student.getSubjectId());
        }
        List<Subject> subjectList = subjectService.findBySubjectByIdList(subjectIdSet);

        String subjectIds = subjectList.stream().map(subject -> subject.getId().toString()).collect(Collectors.joining(","));
        String subjectNames = subjectList.stream().map(subject -> subject.getName()).collect(Collectors.joining("/"));

        //1、新建班级
        ClassGroup classGroup = new ClassGroup();
        classGroup.setMusicGroupId(classGroup4MixDto.getMusicGroupId());
        classGroup.setSubjectIdList(subjectIds);
        classGroup.setName(classGroup4MixDto.getClassGroupName());
        classGroup.setExpectStudentNum(studentList.size());
        classGroup.setStudentNum(studentList.size());
        classGroup.setType(ClassGroupTypeEnum.NORMAL);
        classGroup.setDelFlag(YesOrNoEnum.NO);
        classGroup.setCreateTime(date);
        classGroup.setUpdateTime(date);
        classGroupDao.insert(classGroup);


        //2、将学生加入新班级（学生注册表，关联表
        List<ClassGroupStudentMapper> classGroupStudentMapperList = new ArrayList<>();
        for (StudentRegistration studentRegistration : studentList) {
            studentRegistration.setClassGroupId(classGroup.getId());
            studentRegistrationService.update(studentRegistration);

            ClassGroupStudentMapper classGroupStudentMapper = new ClassGroupStudentMapper();
            classGroupStudentMapper.setClassGroupId(classGroup.getId());
            classGroupStudentMapper.setUserId(studentRegistration.getUserId());
            classGroupStudentMapper.setCreateTime(date);
            classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);

            classGroupStudentMapperList.add(classGroupStudentMapper);
        }
        classGroupStudentMapperDao.classGroupStudentsInsert(classGroupStudentMapperList);


        //3、将老师加入关联关系
        List<ClassGroupTeacherMapper> classGroupTeacherMapperList = classGroup4MixDto.getClassGroupTeacherMapperList();


        for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
            classGroupTeacherMapper.setClassGroupId(classGroup.getId());
            classGroupTeacherMapper.setMusicGroupId(classGroup4MixDto.getMusicGroupId());
        }
        classGroupTeacherMapperDao.classGroupTeachersInsert(classGroupTeacherMapperList);

        //5、插入班级排课信息
        LocalDateTime now = LocalDate.parse(classGroup4MixDto.getStartDate(), DateTimeFormatter.ofPattern("yyyy-MM-dd")).atStartOfDay();

        //计算每节课的课酬
        List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();
        List<CourseScheduleStudentPayment> courseScheduleStudentPaymentList = new ArrayList<>();
        BigDecimal expectPrice = new BigDecimal("0");
        int times = 0;
        Map<String, Integer> holidayDays = new HashMap<>();
        if (classGroup4MixDto.getHoliday()) {
            Map<Integer, Map<String, Integer>> holiday = jiaRiFeignService.query(now.getYear());
            holidayDays = holiday.get(now.getYear());
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        WhileNode:
        while (true) {
            if (classGroup4MixDto.getHoliday() && holidayDays.containsKey(now.format(DateTimeFormatter.ofPattern("MM-dd")))) {
                now = now.plusDays(1);
                continue;
            }
            int dayOfWeek = now.getDayOfWeek().getValue();
            for (CourseTimeDto courseTimeDto : classGroup4MixDto.getCourseTimeDtoList()) {
                if (!courseTimeDto.getDayOfWeek().equals(dayOfWeek)) continue;

                //课时长度
                long classCourseDuration = Duration.between(LocalDateTime.parse(classGroup4MixDto.getStartDate() + " " + courseTimeDto.getStartClassTime() + ":00", formatter),
                        LocalDateTime.parse(classGroup4MixDto.getStartDate() + " " + courseTimeDto.getEndClassTime() + ":00", formatter))
                        .toMinutes();


                Date classDate = DateConvertor.toDate(now);
                String startClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getStartClassTime() + ":00";
                String endClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getEndClassTime() + ":00";

                CourseSchedule courseSchedule = new CourseSchedule();
                courseSchedule.setSchoolId(schoolId);
                courseSchedule.setClassGroupId(classGroup.getId());
                courseSchedule.setStatus(CourseStatusEnum.NOT_START);
                courseSchedule.setClassDate(classDate);
                courseSchedule.setStartClassTime(DateUtil.stringToDate(startClassTime));
                courseSchedule.setEndClassTime(DateUtil.stringToDate(endClassTime));
                courseSchedule.setCreateTime(date);
                courseSchedule.setUpdateTime(date);
                courseSchedule.setTeachMode(TeachModeEnum.OFFLINE);
                courseSchedule.setType(classGroup4MixDto.getCourseType());
                courseSchedule.setName(subjectNames + "-" + CourseSchedule.CourseScheduleType.SINGLE.getMsg());

                courseScheduleDao.insert(courseSchedule);

                times++;

                for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
                    List<TeacherDefaultMusicGroupSalary> teacherSalaryList = teacherDefaultMusicGroupSalaryService.getTeacherSalaryByUserIdAndType(classGroupTeacherMapper.getUserId(), "SINGLE", musicGroup.getSettlementType());
                    BigDecimal salary = new BigDecimal("0");

                    Map<CourseSchedule.CourseScheduleType, TeacherDefaultMusicGroupSalary> collect = teacherSalaryList.stream().collect(Collectors.toMap(TeacherDefaultMusicGroupSalary::getCourseScheduleType, teacherDefaultMusicGroupSalary -> teacherDefaultMusicGroupSalary));

                    TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalary = collect.get(classGroup4MixDto.getCourseType());

                    //对应基准课酬
                    BigDecimal baseSalary = new BigDecimal(0), classTimeDuty = new BigDecimal(0);

                    //基准课酬
                    if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.TEACHER_DEFAULT) && teacherDefaultMusicGroupSalary != null) {
                        //课程时长与结算单位时长占比
                        classTimeDuty = new BigDecimal(classCourseDuration).divide(new BigDecimal(30));
                        baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher30MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher30MinSalary();
                    }
                    //阶梯课酬
                    if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.GRADIENT_SALARY) && teacherDefaultMusicGroupSalary != null) {
                        //课程时长与结算单位时长占比
                        classTimeDuty = new BigDecimal(classCourseDuration).divide(new BigDecimal(90));
                        baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher90MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher90MinSalary();
                    }
                    salary = baseSalary.multiply(classTimeDuty).setScale(2, BigDecimal.ROUND_HALF_UP);

                    CourseScheduleTeacherSalary courseScheduleTeacherSalary = new CourseScheduleTeacherSalary();
                    courseScheduleTeacherSalary.setCourseScheduleId(courseSchedule.getId());
                    courseScheduleTeacherSalary.setTeacherRole(classGroupTeacherMapper.getTeacherRole());
                    courseScheduleTeacherSalary.setUserId(classGroupTeacherMapper.getUserId());
                    courseScheduleTeacherSalary.setExpectSalary(salary);
                    courseScheduleTeacherSalary.setClassGroupId(classGroup.getId());
                    courseScheduleTeacherSalary.setCreateTime(date);
                    courseScheduleTeacherSalary.setUpdateTime(date);
                    courseScheduleTeacherSalaryList.add(courseScheduleTeacherSalary);
                }
                for (StudentRegistration studentRegistration : studentList) {
                    CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
                    courseScheduleStudentPayment.setCourseScheduleId(courseSchedule.getId());
                    courseScheduleStudentPayment.setUserId(studentRegistration.getUserId());
                    courseScheduleStudentPayment.setExpectPrice(expectPrice);
                    courseScheduleStudentPayment.setCreateTime(date);
                    courseScheduleStudentPayment.setUpdateTime(date);
                    courseScheduleStudentPayment.setClassGroupId(classGroup.getId());
                    courseScheduleStudentPaymentList.add(courseScheduleStudentPayment);
                }
                if (classGroup4MixDto.getCourseTimes().equals(times)) {
                    break WhileNode;
                }
            }

            now = now.plusDays(1);
            if (classGroup4MixDto.getCourseTimes().equals(times)) {
                break;
            }
        }
        //老师结算表
        courseScheduleTeacherSalaryService.batchInsert(courseScheduleTeacherSalaryList);

        //加入合奏班
        ClassGroupRelation classGroupRelation = new ClassGroupRelation();
        classGroupRelation.setClassGroupId(classGroup4MixDto.getClassGroupId());
        classGroupRelation.setSubClassGroupId(classGroup.getId());
        classGroupRelation.setCreateTime(date);
        classGroupRelationService.insert(classGroupRelation);

        //6、添加学生未上合奏课课程
        List<CourseSchedule> noStartCourses = courseScheduleDao.findNoStartCoursesByClassGroupId(classGroup4MixDto.getClassGroupId());
        for (CourseSchedule noStartCourse : noStartCourses) {
            for (StudentRegistration studentRegistration : studentList) {
                CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
                courseScheduleStudentPayment.setCourseScheduleId(noStartCourse.getId());
                courseScheduleStudentPayment.setUserId(studentRegistration.getUserId());
                courseScheduleStudentPayment.setExpectPrice(expectPrice);
                courseScheduleStudentPayment.setCreateTime(date);
                courseScheduleStudentPayment.setUpdateTime(date);
                courseScheduleStudentPayment.setClassGroupId(noStartCourse.getClassGroupId());
                courseScheduleStudentPaymentList.add(courseScheduleStudentPayment);
            }
        }
        //学生结算表
        courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPaymentList);
        return classGroup;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ClassGroup classGroupSnap(ClassGroup4MixDto classGroup4MixDto) throws Exception {
        Date date = new Date();
        String musicGroupId = classGroup4MixDto.getMusicGroupId();
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        if (musicGroup == null) {
            throw new Exception("乐团不存在");
        }
        Integer schoolId = musicGroup.getSchoolId();

        List<Integer> studentIdList = classGroup4MixDto.getStudents();

        List<StudentRegistration> studentList = studentRegistrationService.findStudentListByUserIdList(musicGroupId, studentIdList);

        List<Integer> subjectIdList = studentList.stream().map(studentRegistration -> studentRegistration.getActualSubjectId()).collect(Collectors.toList());

        List<Subject> subjectList = subjectService.findBySubjectByIdList(subjectIdList);

        String subjectIds = subjectList.stream().map(subject -> subject.getId().toString()).collect(Collectors.joining(","));
        String subjectNames = subjectList.stream().map(subject -> subject.getName()).collect(Collectors.joining("/"));

        //1、新建班级
        ClassGroup classGroup = new ClassGroup();
        classGroup.setMusicGroupId(classGroup4MixDto.getMusicGroupId());
        classGroup.setSubjectIdList(subjectIds);
        classGroup.setName(classGroup4MixDto.getClassGroupName());
        classGroup.setExpectStudentNum(studentList.size());
        classGroup.setStudentNum(studentList.size());
        classGroup.setType(ClassGroupTypeEnum.SNAP);
        classGroup.setDelFlag(YesOrNoEnum.NO);
        classGroup.setTotalClassTimes(classGroup4MixDto.getCourseTimes());
        classGroup.setCurrentClassTimes(0);
        classGroup.setCreateTime(date);
        classGroup.setUpdateTime(date);
        classGroupDao.insert(classGroup);


        //2、将学生加入新班级，关联表
        List<ClassGroupStudentMapper> classGroupStudentMapperList = new ArrayList<>();
        for (StudentRegistration studentRegistration : studentList) {
            ClassGroupStudentMapper classGroupStudentMapper = new ClassGroupStudentMapper();
            classGroupStudentMapper.setMusicGroupId(musicGroupId);
            classGroupStudentMapper.setClassGroupId(classGroup.getId());
            classGroupStudentMapper.setUserId(studentRegistration.getUserId());
            classGroupStudentMapper.setCreateTime(date);
            classGroupStudentMapper.setStatus(ClassGroupStudentStatusEnum.NORMAL);

            classGroupStudentMapperList.add(classGroupStudentMapper);
        }
        classGroupStudentMapperDao.classGroupStudentsInsert(classGroupStudentMapperList);

        //3、将老师加入关联关系
        List<ClassGroupTeacherMapper> classGroupTeacherMapperList = classGroup4MixDto.getClassGroupTeacherMapperList();


        for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
            classGroupTeacherMapper.setClassGroupId(classGroup.getId());
            classGroupTeacherMapper.setMusicGroupId(classGroup4MixDto.getMusicGroupId());
        }
        classGroupTeacherMapperDao.classGroupTeachersInsert(classGroupTeacherMapperList);

        //5、插入班级排课信息
        LocalDateTime now = LocalDate.parse(classGroup4MixDto.getStartDate(), DateTimeFormatter.ofPattern("yyyy-MM-dd")).atStartOfDay();

        //计算每节课的课酬
        List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();
        List<CourseScheduleStudentPayment> courseScheduleStudentPaymentList = new ArrayList<>();
        BigDecimal expectPrice = new BigDecimal("0");
        int times = 0;
        Map<String, Integer> holidayDays = new HashMap<>();
        if (classGroup4MixDto.getHoliday()) {
            Map<Integer, Map<String, Integer>> holiday = jiaRiFeignService.query(now.getYear());
            holidayDays = holiday.get(now.getYear());
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        WhileNode:
        while (true) {
            if (classGroup4MixDto.getHoliday() && holidayDays.containsKey(now.format(DateTimeFormatter.ofPattern("MM-dd")))) {
                now = now.plusDays(1);
                continue;
            }

            int dayOfWeek = now.getDayOfWeek().getValue();
            for (CourseTimeDto courseTimeDto : classGroup4MixDto.getCourseTimeDtoList()) {
                if (!courseTimeDto.getDayOfWeek().equals(dayOfWeek)) continue;

                //课时长度
                long classCourseDuration = Duration.between(LocalDateTime.parse(classGroup4MixDto.getStartDate() + " " + courseTimeDto.getStartClassTime() + ":00", formatter),
                        LocalDateTime.parse(classGroup4MixDto.getStartDate() + " " + courseTimeDto.getEndClassTime() + ":00", formatter))
                        .toMinutes();


                Date classDate = DateConvertor.toDate(now);
                String startClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getStartClassTime() + ":00";
                String endClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getEndClassTime() + ":00";

                CourseSchedule courseSchedule = new CourseSchedule();
                courseSchedule.setSchoolId(schoolId);
                courseSchedule.setClassGroupId(classGroup.getId());
                courseSchedule.setStatus(CourseStatusEnum.NOT_START);
                courseSchedule.setClassDate(classDate);
                courseSchedule.setStartClassTime(DateUtil.stringToDate(startClassTime));
                courseSchedule.setEndClassTime(DateUtil.stringToDate(endClassTime));
                courseSchedule.setCreateTime(date);
                courseSchedule.setUpdateTime(date);
                courseSchedule.setTeachMode(TeachModeEnum.OFFLINE);
                courseSchedule.setType(classGroup4MixDto.getCourseType());
                courseSchedule.setName(subjectNames + "-" + CourseSchedule.CourseScheduleType.SINGLE.getMsg());

                courseScheduleDao.insert(courseSchedule);

                times++;

                for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
                    List<TeacherDefaultMusicGroupSalary> teacherSalaryList = teacherDefaultMusicGroupSalaryService.getTeacherSalaryByUserIdAndType(classGroupTeacherMapper.getUserId(), "SINGLE", musicGroup.getSettlementType());
                    BigDecimal salary = new BigDecimal("0");

                    Map<CourseSchedule.CourseScheduleType, TeacherDefaultMusicGroupSalary> collect = teacherSalaryList.stream().collect(Collectors.toMap(TeacherDefaultMusicGroupSalary::getCourseScheduleType, teacherDefaultMusicGroupSalary -> teacherDefaultMusicGroupSalary));

                    TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalary = collect.get(classGroup4MixDto.getCourseType());

                    //对应基准课酬
                    BigDecimal baseSalary = new BigDecimal(0), classTimeDuty = new BigDecimal(0);

                    //基准课酬
                    if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.TEACHER_DEFAULT) && teacherDefaultMusicGroupSalary != null) {
                        //课程时长与结算单位时长占比
                        classTimeDuty = new BigDecimal(classCourseDuration).divide(new BigDecimal(30));
                        baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher30MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher30MinSalary();
                    }
                    //阶梯课酬
                    if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.GRADIENT_SALARY) && teacherDefaultMusicGroupSalary != null) {
                        //课程时长与结算单位时长占比
                        classTimeDuty = new BigDecimal(classCourseDuration).divide(new BigDecimal(90));
                        baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher90MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher90MinSalary();
                    }
                    salary = baseSalary.multiply(classTimeDuty).setScale(2, BigDecimal.ROUND_HALF_UP);

                    CourseScheduleTeacherSalary courseScheduleTeacherSalary = new CourseScheduleTeacherSalary();
                    courseScheduleTeacherSalary.setCourseScheduleId(courseSchedule.getId());
                    courseScheduleTeacherSalary.setTeacherRole(classGroupTeacherMapper.getTeacherRole());
                    courseScheduleTeacherSalary.setUserId(classGroupTeacherMapper.getUserId());
                    courseScheduleTeacherSalary.setExpectSalary(salary);
                    courseScheduleTeacherSalary.setClassGroupId(classGroup.getId());
                    courseScheduleTeacherSalary.setCreateTime(date);
                    courseScheduleTeacherSalary.setUpdateTime(date);
                    courseScheduleTeacherSalaryList.add(courseScheduleTeacherSalary);
                }
                for (StudentRegistration studentRegistration : studentList) {
                    CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
                    courseScheduleStudentPayment.setCourseScheduleId(courseSchedule.getId());
                    courseScheduleStudentPayment.setUserId(studentRegistration.getUserId());
                    courseScheduleStudentPayment.setExpectPrice(expectPrice);
                    courseScheduleStudentPayment.setCreateTime(date);
                    courseScheduleStudentPayment.setUpdateTime(date);
                    courseScheduleStudentPayment.setClassGroupId(classGroup.getId());
                    courseScheduleStudentPaymentList.add(courseScheduleStudentPayment);
                }
                if (classGroup4MixDto.getCourseTimes().equals(times)) {
                    break WhileNode;
                }
            }

            now = now.plusDays(1);
        }
        //加入合奏班
        ClassGroupRelation classGroupRelation = new ClassGroupRelation();
        classGroupRelation.setClassGroupId(classGroup4MixDto.getMixClassGroupId());
        classGroupRelation.setSubClassGroupId(classGroup.getId());
        classGroupRelation.setCreateTime(date);
        classGroupRelationService.insert(classGroupRelation);

        //老师结算表
        courseScheduleTeacherSalaryService.batchInsert(courseScheduleTeacherSalaryList);

        //学生结算表
        courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPaymentList);
        return classGroup;
    }

    @Override
    public PageInfo<MusicGroupClassGroupDto> queryMusicGroupClassGroup(QueryInfo queryInfo) {
        PageInfo<MusicGroupClassGroupDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<MusicGroupClassGroupDto> dataList = null;
        int count = classGroupDao.countMusicGroupClassGroup(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = classGroupDao.queryMusicGroupClassGroup(params);
            Set<Integer> classGroups = dataList.stream().map(e -> e.getClassGroupId()).collect(Collectors.toSet());
            String join = StringUtils.join(classGroups, ",");
            //获取主教老师
            Map<Integer, String> masterTeachers = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertMybatisMap(classGroupDao.countStudentNum(join, "BISHOP"))), HashMap.class);
            //获取助教老师
            Map<Integer, String> subTeachers = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertMybatisMap(classGroupDao.countStudentNum(join, "TEACHING"))), HashMap.class);
            //获取在读人数
            Map<Integer, Integer> studyNums = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(classGroupDao.countStudyNum(join))), HashMap.class);
            //获取退班人数
            Map<Integer, Integer> quitNums = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(classGroupDao.countQuitNum(join))), HashMap.class);
            dataList.forEach(e -> {
                e.setMasterTeacher(masterTeachers.get(e.getClassGroupId()));
                e.setSubTeacher(subTeachers.get(e.getClassGroupId()));
                e.setStudyNum(studyNums.get(e.getClassGroupId()));
                e.setQuitNum(quitNums.get(e.getClassGroupId()));
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<MusicGroupCourseScheduleDto> queryMusicGroupCourseSchedule(queryMusicGroupCourseScheduleQueryInfo queryInfo) {
        PageInfo<MusicGroupCourseScheduleDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<MusicGroupCourseScheduleDto> dataList = null;
        int count = classGroupDao.countMusicGroupCourseSchedule(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = classGroupDao.queryMusicGroupCourseSchedule(params);
            //获取当前合奏班班级编号列表
            Set<Integer> mixClassGroupIds = dataList.stream().map(e -> e.getMixClassGroupId()).collect(Collectors.toSet());
            String mixClassGroupIdsStr = StringUtils.join(mixClassGroupIds, ",");
            //所有课程编号的列表
            List<Integer> couseScheduleIds = dataList.stream().map(MusicGroupCourseScheduleDto::getCourseScheduleId).distinct().collect(Collectors.toList());
            //获取合奏班名称
            Map<Integer, String> classGroupNames = MapUtil.convertMybatisMap(classGroupDao.findNameById(mixClassGroupIdsStr));

            //主教老师
            List<CourseScheduleTeacherSalary> bishopTeachers = courseScheduleTeacherSalaryDao.findByCourseScheduleIdsAndTeacherRole(couseScheduleIds, "BISHOP");
            Map<Long, CourseScheduleTeacherSalary> courseScheduleBishopTeacherMaps = bishopTeachers.stream().collect(Collectors.toMap(CourseScheduleTeacherSalary::getCourseScheduleId, courseScheduleTeacherSalary -> courseScheduleTeacherSalary));

            //助教老师
            List<CourseScheduleTeacherSalary> teachingTeachers = courseScheduleTeacherSalaryDao.findByCourseScheduleIdsAndTeacherRole(couseScheduleIds, "TEACHING");
            Map<Long, List<CourseScheduleTeacherSalary>> courseScheduleTeachingTeacherMaps = teachingTeachers.stream().collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId));

            dataList.forEach(e -> {
                e.setMixClassGroupName(classGroupNames.get(e.getMixClassGroupId()));
                CourseScheduleTeacherSalary currentBishopTeacher = courseScheduleBishopTeacherMaps.get(e.getCourseScheduleId().longValue());
                if (!Objects.isNull(currentBishopTeacher)) {
                    e.setMasterTeacherId(currentBishopTeacher.getUserId());
                    e.setMasterTeacherName(currentBishopTeacher.getUserName());
                }
                List<CourseScheduleTeacherSalary> currentCourseTeachingTeachers = courseScheduleTeachingTeacherMaps.get(e.getCourseScheduleId().longValue());
                if (CollectionUtils.isEmpty(currentCourseTeachingTeachers)) {
                    currentCourseTeachingTeachers = new ArrayList<>();
                    if(currentBishopTeacher != null){
                        currentCourseTeachingTeachers.add(currentBishopTeacher);
                    }
                    e.setTeacherNum(currentCourseTeachingTeachers.size());
                    e.setTeachingTeachers(currentCourseTeachingTeachers);
                } else {
                    if(currentBishopTeacher != null){
                        currentCourseTeachingTeachers.add(currentBishopTeacher);
                    }
                    e.setTeachingTeachers(currentCourseTeachingTeachers);
                    e.setTeacherNum(currentCourseTeachingTeachers.size());
                }
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public boolean batchIncreaseClassTimes(List<Integer> classGroupList) {
        classGroupDao.batchIncreaseClassTimes(classGroupList);
        return true;
    }

    @Override
    public List<ClassGroup> highClassGroupList(String musicGroupId) {
        List<ClassGroup> classGroupList = classGroupDao.findClassGroupByMusicGroupIdAndType(musicGroupId, ClassGroupTypeEnum.HIGH);
        for (ClassGroup classGroup : classGroupList) {
            List<String> subjects = subjectService.findBySubIds(classGroup.getSubjectIdList());
            String subjectName = StringUtils.join(subjects, ",");
            classGroup.setSubjectName(subjectName);
        }
        return classGroupList;
    }

    @Override
    public List<ClassGroupTeachersDto> classGroupAndTeacher(Integer classGroupId) {
        List<ClassGroupTeachersDto> classGroups = classGroupDao.findClassGroupByMixClassGroupId(classGroupId);
        String classGroupIds = "";
        for (ClassGroupTeachersDto classGroupTeachersDto : classGroups) {
            classGroupIds += classGroupTeachersDto.getId() + ",";
        }
        classGroupIds = classGroupIds.substring(0, classGroupIds.length() - 1);

        List<ClassGroupTeacherMapper> classGroupTeachers = classGroupTeacherMapperDao.findClassGroupTeachers(classGroupIds);
        for (ClassGroupTeachersDto classGroup : classGroups) {
            List<ClassGroupTeacherMapper> classGroupTeacherMappers = new ArrayList<>();
            for (ClassGroupTeacherMapper classGroupTeacher : classGroupTeachers) {
                if (!classGroup.getId().equals(classGroupTeacher.getClassGroupId())) continue;
                classGroupTeacherMappers.add(classGroupTeacher);
            }
            classGroup.setClassGroupTeacherMapperList(classGroupTeacherMappers);
        }
        return classGroups;
    }

    @Override
    public List<HighClassGroupDto> findClassGroupByMusicGroupIdAndSubjectId(String musicGroupId, Integer subjectId) {
        List<HighClassGroupDto> highClassGroupList = classGroupDao.findClassGroupByMusicGroupIdAndSubjectId(musicGroupId, subjectId);
        for (HighClassGroupDto highClassGroup : highClassGroupList) {
            ClassGroupTeacherMapper bishopTeacher = classGroupTeacherMapperDao.findByClassGroupAndRole(highClassGroup.getId().longValue(), TeachTypeEnum.BISHOP);
            highClassGroup.setTeacherName(bishopTeacher.getUserName());
            CourseSchedule courseSchedule = courseScheduleDao.findOneCourseScheduleByClassGroupId(highClassGroup.getId());
            highClassGroup.setStartClassTime(DateUtil.format(courseSchedule.getStartClassTime(), "HH:mm:ss"));
            highClassGroup.setEndClassTime(DateUtil.format(courseSchedule.getEndClassTime(), "HH:mm:ss"));

            Calendar calendar = Calendar.getInstance();
            calendar.setTime(courseSchedule.getClassDate());
            int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
            highClassGroup.setDayOfWeek(dayOfWeek - 1);

        }
        return highClassGroupList;
    }

    @Override
    public int batchUpdateClassCourseTimes(List<Integer> classGroupIds) {
        List<Map<Integer, Integer>> classGroupCourseTimesMaps = courseScheduleDao.countClassGroupCourseTimes(classGroupIds);
        Map<Integer, Integer> classGroupCourseTimesMap = MapUtil.convertIntegerMap(classGroupCourseTimesMaps);
        return classGroupDao.batchUpdateClassGroupCourseTimes(classGroupCourseTimesMap);
    }
}
