package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ClassGroupTeachersDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.SalarySettlementTypeEnum;
import com.ym.mec.biz.dal.enums.TeachTypeEnum;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupMember;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collector;
import java.util.stream.Collectors;

@Service
public class ClassGroupTeacherMapperServiceImpl extends BaseServiceImpl<Long, ClassGroupTeacherMapper> implements ClassGroupTeacherMapperService {

    @Autowired
    private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;
    @Autowired
    private CourseScheduleService courseScheduleService;
    @Autowired
    private CourseScheduleTeacherSalaryDao courseScheduleTeacherSalaryDao;
    @Autowired
    private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
    @Autowired
    private TeacherDefaultMusicGroupSalaryDao teacherDefaultMusicGroupSalaryDao;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private ImFeignService imFeignService;
    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private TeacherAttendanceDao teacherAttendanceDao;

    @Override
    public BaseDAO<Long, ClassGroupTeacherMapper> getDAO() {
        return classGroupTeacherMapperDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int classGroupTeachersInsert(List<ClassGroupTeacherMapper> classGroupTeacherMapperList) {
        //老师加入群组
        List<ImGroupMember> imGroupMemberList = new ArrayList<>();
        String classGroupIds = classGroupTeacherMapperList.stream().map(classGroupTeacherMapper -> classGroupTeacherMapper.getClassGroupId().toString()).collect(Collectors.joining(","));

        List<ClassGroup> classGroups = classGroupService.findClassGroupByIds(classGroupIds);

        List<ClassGroupTeacherMapper> oldClassGroupTeacherMapper = classGroupTeacherMapperDao.getClassGroupTeacherMapper(classGroupTeacherMapperList);

        for (ClassGroup classGroup : classGroups) {
            //原设置的老师删除群组
            for (ClassGroupTeacherMapper classGroupTeacherMapper : oldClassGroupTeacherMapper) {
                if (!classGroup.getId().equals(classGroupTeacherMapper.getClassGroupId())) continue;
                imGroupMemberList.add(new ImGroupMember(classGroupTeacherMapper.getUserId().toString()));
            }
            if(imGroupMemberList.size()>0) {
                ImGroupMember[] imGroupMembers = imGroupMemberList.toArray(new ImGroupMember[imGroupMemberList.size()]);
                imFeignService.groupQuit(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, classGroup.getName()));
            }
            // 新设置的老师加入群组
            imGroupMemberList.clear();
            for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
                if (!classGroup.getId().equals(classGroupTeacherMapper.getClassGroupId())) continue;
                imGroupMemberList.add(new ImGroupMember(classGroupTeacherMapper.getUserId().toString()));
            }
            ImGroupMember[] imGroupMembers = imGroupMemberList.toArray(new ImGroupMember[imGroupMemberList.size()]);
            imFeignService.groupJoin(new ImGroupModel(classGroup.getId().toString(), imGroupMembers, classGroup.getName()));
        }
        classGroupTeacherMapperDao.delMusicGroupTeacherMapper(classGroupTeacherMapperList);

        return classGroupTeacherMapperDao.classGroupTeachersInsert(classGroupTeacherMapperList);
    }


    @Override
    public List<ClassGroupTeacherMapper> findClassGroupTeachers(List<ClassGroupTeachersDto> ClassGroupList) {
        Set<Integer> collect = ClassGroupList.stream().map(ClassGroupTeachersDto::getId).collect(Collectors.toSet());
        return classGroupTeacherMapperDao.findClassGroupTeachers(StringUtils.join(collect, ","));
    }

    public boolean classGroupTeacherMapperUpdate(List<ClassGroupTeacherMapper> classGroupTeacherMapperList) throws Exception {
        Date date = new Date();

        List<ClassGroupTeacherSalary> classGroupTeacherSalaryList = new ArrayList<>();
        List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();

        String musicGroupId = classGroupTeacherMapperList.get(0).getMusicGroupId();
        MusicGroup musicGroup = musicGroupService.get(musicGroupId);

        List<CourseSchedule> classGroupNoStartCourseSchedules = courseScheduleService.findClassGroupNoStartCourseSchedules(classGroupTeacherMapperList);

        if(classGroupNoStartCourseSchedules == null || classGroupNoStartCourseSchedules.size()==0){
            return true;
        }
        //删除未开始的课程老师课酬
        List<Long> courseScheduleIds = classGroupNoStartCourseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());
        courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
        teacherAttendanceDao.batchDeleteByCourseSchedules(courseScheduleIds);

        for (CourseSchedule classGroupNoStartCourseSchedule : classGroupNoStartCourseSchedules) {

            if (!classGroupNoStartCourseSchedule.getType().equals(CourseSchedule.CourseScheduleType.SINGLE) &&
                    !classGroupNoStartCourseSchedule.getType().equals(CourseSchedule.CourseScheduleType.MIX) &&
                    !classGroupNoStartCourseSchedule.getType().equals(CourseSchedule.CourseScheduleType.HIGH)
            ) {
                continue;
            }
            //课时长度
            int duration = DateUtil.minutesBetween(classGroupNoStartCourseSchedule.getStartClassTime(),classGroupNoStartCourseSchedule.getEndClassTime());
            for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMapperList) {
                if (!classGroupTeacherMapper.getClassGroupId().equals(classGroupNoStartCourseSchedule.getClassGroupId())) {
                    continue;
                }
                BigDecimal salary = new BigDecimal("0");
                List<TeacherDefaultMusicGroupSalary> teacherDefaultMusicGroupSalaryList = classGroupTeacherMapper.getTeacherDefaultMusicGroupSalaryList();

                Map<CourseSchedule.CourseScheduleType, TeacherDefaultMusicGroupSalary> collect = teacherDefaultMusicGroupSalaryList.stream().collect(Collectors.toMap(TeacherDefaultMusicGroupSalary::getCourseScheduleType, teacherDefaultMusicGroupSalary -> teacherDefaultMusicGroupSalary));

                TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalary=collect.get(classGroupNoStartCourseSchedule.getType());

                //对应基准课酬
                BigDecimal baseSalary = new BigDecimal(0),classTimeDuty=new BigDecimal(0);

                //基准课酬
                if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.TEACHER_DEFAULT)) {
                    //课程时长与结算单位时长占比
                    classTimeDuty = new BigDecimal(duration).divide(new BigDecimal(30),6,BigDecimal.ROUND_HALF_UP);
                    baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher30MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher30MinSalary();
                }
                //阶梯课酬
                if (musicGroup.getSettlementType().equals(SalarySettlementTypeEnum.GRADIENT_SALARY)) {
                    //课程时长与结算单位时长占比
                    classTimeDuty = new BigDecimal(duration).divide(new BigDecimal(90),6,BigDecimal.ROUND_HALF_UP);
                    baseSalary = classGroupTeacherMapper.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? teacherDefaultMusicGroupSalary.getMainTeacher90MinSalary() : teacherDefaultMusicGroupSalary.getAssistantTeacher90MinSalary();
                }
                salary = baseSalary.multiply(classTimeDuty).setScale(2, BigDecimal.ROUND_HALF_UP);

                //课程与老师薪水表
                CourseScheduleTeacherSalary courseScheduleTeacherSalary = new CourseScheduleTeacherSalary();
                courseScheduleTeacherSalary.setCourseScheduleId(classGroupNoStartCourseSchedule.getId());
                courseScheduleTeacherSalary.setTeacherRole(classGroupTeacherMapper.getTeacherRole());
                courseScheduleTeacherSalary.setUserId(classGroupTeacherMapper.getUserId());
                courseScheduleTeacherSalary.setExpectSalary(salary);
                courseScheduleTeacherSalary.setCreateTime(date);
                courseScheduleTeacherSalary.setUpdateTime(date);
                courseScheduleTeacherSalary.setClassGroupId(classGroupTeacherMapper.getClassGroupId());
                courseScheduleTeacherSalaryList.add(courseScheduleTeacherSalary);
            }
        }
        courseScheduleTeacherSalaryService.batchInsert(courseScheduleTeacherSalaryList);
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createCourseScheduleTeacherSalary(List<CourseSchedule> courseSchedules, Map<Long, CourseSchedule> oldCourseSchedules) {
        List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();

        BigDecimal teacherSalaryTimeUnit = new BigDecimal(sysConfigDao.findConfigValue(SysConfigService.TEACHER_SALARY_TIME_UNIT));

        //课程编号列表
        List<Long> courseScheduleIds = courseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());
        //主教老师列表
        List<Integer> allTeacherIds = courseSchedules.stream().map(courseSchedule -> courseSchedule.getActualTeacherId().intValue()).collect(Collectors.toList());
        //助教老师列表
        List<Integer> teachingTeacherIds = new ArrayList<>();
        courseSchedules.forEach(courseSchedule -> teachingTeacherIds.addAll(courseSchedule.getTeachingTeacherIdList()));
        allTeacherIds.addAll(teachingTeacherIds);
        //所有老师列表
        allTeacherIds = allTeacherIds.stream().distinct().collect(Collectors.toList());

        //所有老师默认乐团课酬
        List<TeacherDefaultMusicGroupSalary> teacherDefaultMusicGroupSalaries = teacherDefaultMusicGroupSalaryDao.findByTeacher(allTeacherIds);
        Map<CourseSchedule.CourseScheduleType, List<TeacherDefaultMusicGroupSalary>> teacherDefaultMusicGroupSalariesGroupByCourseType = teacherDefaultMusicGroupSalaries.stream().collect(Collectors.groupingBy(TeacherDefaultMusicGroupSalary::getCourseScheduleType));

        //获取课程对应教学点补贴
        List<Map<Integer, BigDecimal>> courseSubsidyByCourses = courseScheduleTeacherSalaryDao.findCourseSubsidyByCourses(courseScheduleIds);
        Map<Integer, BigDecimal> courseSubsidyMap = MapUtil.convertIntegerMap(courseSubsidyByCourses);

        //课程对应乐团结算方式集合
        List<Map<String, String>> musicGroupSettlementTypeByCourse = courseScheduleTeacherSalaryDao.findMusicGroupSettlementTypeByCourse(courseScheduleIds);
        Map<String, String> musicGroupSettlementsMap = MapUtil.convertMybatisMap(musicGroupSettlementTypeByCourse);

        courseSchedules.forEach(courseSchedule -> {
            if (Objects.isNull(courseSchedule.getType())) {
                courseSchedule.setType(oldCourseSchedules.get(courseSchedule.getId()).getType());
            }

            //获取结算方式
            String settlementType = musicGroupSettlementsMap.get(courseSchedule.getId());
            if (StringUtils.isBlank(settlementType)) {
                throw new BizException("存在未指定结算方式的课程");
            }

            //根据课程类型获取对应的默认课酬设置列表
            List<TeacherDefaultMusicGroupSalary> teacherDefaultMusicGroupSalariesWithCourseType = teacherDefaultMusicGroupSalariesGroupByCourseType.get(courseSchedule.getType());
            if (CollectionUtils.isEmpty(teacherDefaultMusicGroupSalariesWithCourseType)) {
                throw new BizException("存在未指定结算方式的课程类型");
            }

            //根据教师编号将对应结算方式下的默认课酬设置分组
            Map<Integer, TeacherDefaultMusicGroupSalary> teacherDefaultMusicGroupSalariesGroupByTeacherId = teacherDefaultMusicGroupSalariesWithCourseType.stream().collect(Collectors.toMap(TeacherDefaultMusicGroupSalary::getUserId,teacherDefaultMusicGroupSalary -> teacherDefaultMusicGroupSalary));

            if (!courseSchedule.isBishopTeacherSalaryNotCreate()) {
                //根据课程获取主教的默认课酬设置列表
                TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalaryWithTeacherId = teacherDefaultMusicGroupSalariesGroupByTeacherId.get(courseSchedule.getActualTeacherId());
                courseScheduleTeacherSalaryList.add(countMusicGroupTeacherSalary(
                        courseSchedule,
                        TeachTypeEnum.BISHOP,
                        settlementType,
                        courseSubsidyMap.get(courseSchedule.getId()),
                        courseSchedule.getActualTeacherId().intValue(),
                        teacherDefaultMusicGroupSalaryWithTeacherId));
            }

            //计算助教课程薪酬
            //助教编号列表
            List<Integer> teachingTeacherIdList = courseSchedule.getTeachingTeacherIdList();
            if (CollectionUtils.isEmpty(teachingTeacherIdList)) {
                return;
            }
            teachingTeacherIdList.forEach(teacherId -> {
                TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalaryWithTeacherId = teacherDefaultMusicGroupSalariesGroupByTeacherId.get(teacherId);
                courseScheduleTeacherSalaryList.add(countMusicGroupTeacherSalary(
                        courseSchedule,
                        TeachTypeEnum.TEACHING,
                        settlementType,
                        courseSubsidyMap.get(courseSchedule.getId()),
                        teacherId,
                        teacherDefaultMusicGroupSalaryWithTeacherId));
            });
        });
        courseScheduleTeacherSalaryService.batchInsert(courseScheduleTeacherSalaryList);
    }

    /**
     * @param courseSchedule: 课程信息
     * @param teachType: 教师类型
     * @param settlementType: 结算方式
     * @param subsidy: 补贴
     * @param teacherId: 教师编号
     * @param teacherDefaultMusicGroupSalaryWithTeacherId: 教师课酬
     * @return com.ym.mec.biz.dal.entity.CourseScheduleTeacherSalary
     * @describe 计算教师课酬
     * @author Joburgess
     * @date 2019/10/24
     */
    private CourseScheduleTeacherSalary countMusicGroupTeacherSalary(CourseSchedule courseSchedule,
                                                                     TeachTypeEnum teachType,
                                                                     String settlementType,
                                                                     BigDecimal subsidy,
                                                                     Integer teacherId,
                                                                     TeacherDefaultMusicGroupSalary teacherDefaultMusicGroupSalaryWithTeacherId) {
        if (Objects.isNull(teacherDefaultMusicGroupSalaryWithTeacherId)) {
            throw new BizException("存在未设置默认课酬的教师");
        }
        //教师课酬
        BigDecimal teacherSalary = new BigDecimal(0);

        //课时长度
        int duration = DateUtil.minutesBetween(courseSchedule.getEndClassTime(), courseSchedule.getStartClassTime());

        //如果结算方式是教师默认课酬
        if (settlementType.equals(SalarySettlementTypeEnum.TEACHER_DEFAULT)) {
            //课程时长与结算单位时长占比
            BigDecimal classTimeDuty = new BigDecimal(duration).divide(new BigDecimal(30));
            BigDecimal teacherDefaultSalary = teachType == TeachTypeEnum.BISHOP ? teacherDefaultMusicGroupSalaryWithTeacherId.getMainTeacher30MinSalary() : teacherDefaultMusicGroupSalaryWithTeacherId.getAssistantTeacher30MinSalary();
            teacherSalary = teacherDefaultSalary.multiply(classTimeDuty).setScale(2, BigDecimal.ROUND_HALF_UP);
        } else if (settlementType.equals(SalarySettlementTypeEnum.GRADIENT_SALARY)) {
            //课程时长与结算单位时长占比
            BigDecimal classTimeDuty = new BigDecimal(duration).divide(new BigDecimal(90));
            BigDecimal teacherDefaultSalary = teachType == TeachTypeEnum.BISHOP ? teacherDefaultMusicGroupSalaryWithTeacherId.getMainTeacher90MinSalary() : teacherDefaultMusicGroupSalaryWithTeacherId.getAssistantTeacher90MinSalary();
            teacherSalary = teacherDefaultSalary.multiply(classTimeDuty).setScale(2, BigDecimal.ROUND_HALF_UP);
        }
        return new CourseScheduleTeacherSalary(
                courseSchedule.getId(),
                teachType,
                teacherId,
                teacherSalary,
                subsidy,
                courseSchedule.getClassGroupId());
    }
}
