package com.ym.mec.biz.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ym.mec.biz.dal.dao.CourseScheduleRewardsRulesDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.entity.CourseSchedule.CourseScheduleType;
import com.ym.mec.biz.dal.entity.CourseScheduleRewardsRules;
import com.ym.mec.biz.dal.entity.CourseScheduleRewardsRules.RewardMode;
import com.ym.mec.biz.service.CourseScheduleRewardsRulesService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.json.JsonUtil;

@Service
public class CourseScheduleRewardsRulesServiceImpl extends BaseServiceImpl<Integer, CourseScheduleRewardsRules> implements CourseScheduleRewardsRulesService {

	@Autowired
	private CourseScheduleRewardsRulesDao courseScheduleRewardsDao;
	
	@Autowired
	private OrganizationDao organizationDao;

	@Override
	public BaseDAO<Integer, CourseScheduleRewardsRules> getDAO() {
		return courseScheduleRewardsDao;
	}

	@Override
	public PageInfo<CourseScheduleRewardsRules> queryPage(QueryInfo queryInfo) {
		PageInfo<CourseScheduleRewardsRules> pageInfo = new PageInfo<CourseScheduleRewardsRules>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);
		
		List<CourseScheduleRewardsRules> dataList = null;
		int count = this.findCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = this.getDAO().queryPage(params);
			for(CourseScheduleRewardsRules rules : dataList){
				rules.setOrganNameList(organizationDao.findByOrganIds(rules.getOrganIdList()).stream().collect(Collectors.joining("、")));
			}
		}
		if (count == 0) {
			dataList = new ArrayList<CourseScheduleRewardsRules>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public BigDecimal queryRewardsAmount(Integer organId, CourseScheduleType courseScheduleType, int coursesNum) {

		List<CourseScheduleRewardsRules> CourseScheduleRewardsRulesList = courseScheduleRewardsDao.query(organId, courseScheduleType);

		if (CourseScheduleRewardsRulesList == null || CourseScheduleRewardsRulesList.size() == 0) {
			return new BigDecimal(0);
		}

		CourseScheduleRewardsRules courseScheduleRewardsRules = CourseScheduleRewardsRulesList.get(0);

		String json = courseScheduleRewardsRules.getRewardsRulesJson();

		List<RewardsRules> rewardsRulesList = JsonUtil.parseArray(json, RewardsRules.class);

		Collections.sort(rewardsRulesList);

		RewardsRules rewardsRules = new RewardsRules();

		for (RewardsRules rule : rewardsRulesList) {
			if (rule.min >= coursesNum && rule.max <= coursesNum) {
				rewardsRules = rule;
				break;
			}
		}

		if (courseScheduleRewardsRules.getRewardMode() == RewardMode.TOTAL) {
			return rewardsRules.getAmount();
		} else if (courseScheduleRewardsRules.getRewardMode() == RewardMode.STAIR) {
			BigDecimal amount = new BigDecimal(0);
			for (RewardsRules rule : rewardsRulesList) {
				if (coursesNum <= rule.max) {
					amount = amount.add(new BigDecimal(coursesNum * rule.amount.doubleValue()));
				} else {
					amount = amount.add(new BigDecimal(rule.max * rule.amount.doubleValue()));
					coursesNum = coursesNum - rule.max;
				}
			}

			return amount;
		}

		return rewardsRules.getAmount().multiply(new BigDecimal(coursesNum));
	}

	class RewardsRules implements Comparable<RewardsRules> {

		private int min;

		private int max;

		private BigDecimal amount = new BigDecimal(0);

		public int getMin() {
			return min;
		}

		public void setMin(int min) {
			this.min = min;
		}

		public int getMax() {
			return max;
		}

		public void setMax(int max) {
			this.max = max;
		}

		public BigDecimal getAmount() {
			return amount;
		}

		public void setAmount(BigDecimal amount) {
			this.amount = amount;
		}

		@Override
		public int compareTo(RewardsRules rules) {
			if (this.min > rules.min) {
				return 1;
			}
			return -1;
		}
	}
}