package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.entity.SysUserRole;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.CourseScheduleQueryInfo;
import com.ym.mec.biz.dal.page.StudentCourseScheduleRecordQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.collections.ListUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class CourseScheduleServiceImpl extends BaseServiceImpl<Long, CourseSchedule>  implements CourseScheduleService {
	
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private CourseScheduleComplaintsDao courseScheduleComplaintsDao;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private StudentAttendanceDao studentAttendanceDao;
	@Autowired
	private ClassGroupStudentMapperDao classGroupStudentMapperDao;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
	@Autowired
	private VipGroupDao vipGroupDao;
	@Autowired
	private TeacherDefaultVipGroupSalaryDao teacherDefaultVipGroupSalaryDao;
	@Autowired
	private CourseScheduleTeacherSalaryDao courseScheduleTeacherSalaryDao;
	@Autowired
	private ClassGroupTeacherMapperService classGroupTeacherMapperService;
	@Autowired
	private CourseScheduleStudentPaymentService courseScheduleStudentPaymentService;
	@Autowired
	private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
	@Autowired
	private TeacherAttendanceDao teacherAttendanceDao;
	@Autowired
	private ClassGroupService classGroupService;
	@Autowired
	private ClassGroupDao classGroupDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private MusicGroupDao musicGroupDao;
	@Autowired
	private SubjectDao subjectDao;

	@Override
	public BaseDAO<Long, CourseSchedule> getDAO() {
		return courseScheduleDao;
	}

	@Override
	public TeacherAttendanceDto getCurrentCourseDetail(Long courseID) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(null==user){
			throw new BizException("获取用户信息失败");
		}
		if(Objects.isNull(courseID)){
			throw new BizException("参数错误!");
		}
		TeacherAttendanceDto currentCourseDetail = courseScheduleDao.getCurrentCourseDetail(courseID);
		if(Objects.isNull(currentCourseDetail)){
		    throw new BizException("课程不存在");
        }

		currentCourseDetail.setOnlyNormal(enableOnlyNormalAttendance(currentCourseDetail.getStartClassTime(),
				user.getId().longValue(),
				true,
				currentCourseDetail.getSchoolId().intValue()));

		currentCourseDetail.setCurrentTime(new Date());
		currentCourseDetail.setAdvanceSignInMinutes(Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.ADVANCE_SIGN_IN_MINUTES)));
		currentCourseDetail.setAttendanceRange(Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.ATTENDANCE_RANGE)));
		return currentCourseDetail;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void deleteCourseSchedules(List<Long> courseScheduleIds) {
		courseScheduleDao.batchDeleteCourseSchedules(courseScheduleIds);
		courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
		courseScheduleStudentPaymentDao.deleteByCourseSchedule(courseScheduleIds);
		teacherAttendanceDao.batchDeleteByCourseSchedules(courseScheduleIds);
	}

	@Override
	public YesOrNoEnum enableOnlyNormalAttendance(Date startClassTime,Long userId,boolean checkSchool,Integer schoolId){
		Date endTime=startClassTime;
		Date startTime=DateUtil.addHours(endTime,-1);
		TeacherAttendanceDto teacherCourseByDateSpeed = courseScheduleDao.getTeacherCourseByDateSpeed(startTime, endTime, userId);
		if(Objects.isNull(teacherCourseByDateSpeed)){
			return YesOrNoEnum.NO;
		}else{
			int i = studentAttendanceDao.countByCourseSchedule(teacherCourseByDateSpeed.getCourseScheduleId());
			if(i>0&&checkSchool&&schoolId.equals(teacherCourseByDateSpeed.getSchoolId())){
				return YesOrNoEnum.YES;
			}else{
				return YesOrNoEnum.NO;
			}
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void createCourseSchedules(List<CourseSchedule> courseSchedules) {
		//添加课程计划
		batchAddCourseSchedule(courseSchedules);
		//更新课次
        List<Integer> classGroupIds = courseSchedules.stream().map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
        classGroupService.batchUpdateClassCourseTimes(classGroupIds);
        //创建学生单节课的缴费记录，乐团课的缴费为0
		courseScheduleStudentPaymentService.createCourseScheduleStudentPaymentByCourseSchedules(courseSchedules);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void batchAddCourseSchedule(List<CourseSchedule> courseSchedules) {
		if(Objects.isNull(courseSchedules)&&courseSchedules.size()<=0){
			throw new BizException("参数错误!");
		}
		checkNewCourseSchedules(courseSchedules,false);
        createCourseScheduleName(courseSchedules);
		courseScheduleDao.batchAddCourseSchedules(courseSchedules);
	}

	@Override
	public void batchUpdateCourseSchedule(List<CourseSchedule> courseSchedules, Long musicGroupID) {
		if(Objects.isNull(courseSchedules)&&courseSchedules.size()<=0){
			throw new BizException("请选择需要调整的课程!");
		}
		if(Objects.isNull(musicGroupID)){
			throw new BizException("请指定乐团!");
		}
		checkNewCourseSchedules(courseSchedules,false);
        courseScheduleDao.deleteCourseSchedulesByMusicGroupID(musicGroupID);
        courseScheduleDao.batchAddCourseSchedules(courseSchedules);
	}

	@Override
	public List<CourseScheduleDto> getCourseSchedulesWithDate(Date classDate) {
		if(Objects.isNull(classDate)){
			throw new BizException("请选择日期!");
		}
		return courseScheduleDao.getCourseSchedulesWithDate(classDate);
	}

	@Override
	public List<Date> getCourseScheduleDateByMonth(Long musicGroupID, Date month) {
		if(Objects.isNull(month)||Objects.isNull(musicGroupID)){
			throw new BizException("请选择日期并指定乐团!");
		}
		return courseScheduleDao.getCourseScheduleDateByMonth(musicGroupID,month);
	}

	@Override
	public List<Date> getCourseScheduleDateByStudent(Date month, Integer isAttend) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(null==user){
			throw new BizException("获取用户信息失败");
		}
		return courseScheduleDao.getStudentCourseScheduleDate(user.getId().longValue(),month,isAttend);
	}

	@Override
	public List<Date> getCourseScheduleDateByTeacher(Date month,String type) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(null==user){
			throw new BizException("获取用户信息失败");
		}
		return courseScheduleDao.getTeacherCourseScheduleDate(user.getId().longValue(),month,type);
	}

	@Override
	public List<Date> getCourseScheduleDates(Date month) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(null==user){
			throw new BizException("请登录");
		}
		return courseScheduleDao.getCourseScheduleDate(month,user.getOrganId().longValue());
	}

	@Override
	public Map getStudentCourseSchedulesWithDate(Date classDate) {
		if(Objects.isNull(classDate)){
			throw new BizException("请选择上课日期!");
		}
		SysUser user = sysUserFeignService.queryUserInfo();
		if(null==user){
			throw new BizException("请登录");
		}
		Date now = new Date();
		Map<String,Object> result=new HashMap<>(3);
		result.put("vipAppealDaysRange",sysConfigDao.findConfigValue(SysConfigService.VIP_APPEAL_DAYS_RANGE));
		result.put("advanceLeaveHours",Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.ADVANCE_LEAVE_HOURS)));
		List<CourseScheduleDto> studentCourseSchedulesWithDate = courseScheduleDao.getStudentCourseSchedulesWithDate(classDate, user.getId().longValue());
		List<Long> courseScheduleIds = studentCourseSchedulesWithDate.stream()
				.filter(cs -> cs.getType().equals(CourseSchedule.CourseScheduleType.VIP) || cs.getType().equals(CourseSchedule.CourseScheduleType.DEMO))
				.map(CourseScheduleDto::getId).collect(Collectors.toList());

		Map<Integer, String> subjectNameCourseMap=new HashMap<>();
		if(!CollectionUtils.isEmpty(courseScheduleIds)){
			List<Map<Integer, String>> subjectNameCourseMaps = subjectDao.findSubjectNameCourseMaps(courseScheduleIds);
			subjectNameCourseMap = MapUtil.convertMybatisMap(subjectNameCourseMaps);
		}

		for (CourseScheduleDto courseScheduleDto : studentCourseSchedulesWithDate) {
			if(now.before(courseScheduleDto.getStartClassTime())){
				courseScheduleDto.setStatus(CourseStatusEnum.NOT_START);
			}else if(now.after(courseScheduleDto.getEndClassTime())){
				courseScheduleDto.setStatus(CourseStatusEnum.OVER);
			}else{
				courseScheduleDto.setStatus(CourseStatusEnum.UNDERWAY);
			}
			courseScheduleDto.setSubjectName(subjectNameCourseMap.get(courseScheduleDto.getId()));
		}
		result.put("list",studentCourseSchedulesWithDate);
		return result;
	}

	@Override
	public PageInfo getCourseSchedulesWithDateByEdu(CourseScheduleQueryInfo queryInfo) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(null==user){
			throw new BizException("请登录");
		}
		queryInfo.setOrganId(user.getOrganId().longValue());
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

		if(Objects.isNull(queryInfo.getDate())){
			queryInfo.setDate(new Date());
		}

		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = courseScheduleDao.countCourseSchedulesWithDate(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = courseScheduleDao.getCourseSchedulesWithDate(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public List<CourseScheduleDto> getTeacherCourseSchedulesWithDate(Integer userId,Date classDate,String type) {
		if(Objects.isNull(classDate)){
			throw new BizException("请选择上课日期!");
		}
		Date now=new Date();
		List<CourseScheduleDto> teacherCourseSchedulesWithDate = courseScheduleDao.getTeacherCourseSchedulesWithDate(classDate, userId.longValue(),type);
		List<Long> courseScheduleIds = teacherCourseSchedulesWithDate.stream()
				.filter(cs -> cs.getType().equals(CourseSchedule.CourseScheduleType.VIP) || cs.getType().equals(CourseSchedule.CourseScheduleType.DEMO))
				.map(CourseScheduleDto::getId).collect(Collectors.toList());

		Map<Integer, String> subjectNameCourseMap=new HashMap<>();
		if(!CollectionUtils.isEmpty(courseScheduleIds)){
			List<Map<Integer, String>> subjectNameCourseMaps = subjectDao.findSubjectNameCourseMaps(courseScheduleIds);
			subjectNameCourseMap = MapUtil.convertMybatisMap(subjectNameCourseMaps);
		}
		for (CourseScheduleDto courseScheduleDto : teacherCourseSchedulesWithDate) {
			if(now.before(courseScheduleDto.getStartClassTime())){
				courseScheduleDto.setStatus(CourseStatusEnum.NOT_START);
			}else if(now.after(courseScheduleDto.getEndClassTime())){
				courseScheduleDto.setStatus(CourseStatusEnum.OVER);
			}else{
				courseScheduleDto.setStatus(CourseStatusEnum.UNDERWAY);
			}
			if(Objects.nonNull(courseScheduleDto.getClassGroupId())){
				String[] studentNums = classGroupStudentMapperDao.findStudentNumByClassGroupId(courseScheduleDto.getClassGroupId().longValue());
				courseScheduleDto.setStudentNames(org.apache.commons.lang3.StringUtils.join(studentNums, ","));
			}
			courseScheduleDto.setSubjectName(subjectNameCourseMap.get(courseScheduleDto.getId()));
		}
		return teacherCourseSchedulesWithDate;
	}

	@Override
	public void checkNewCourseSchedules(List<CourseSchedule> courseSchedules,boolean checkExistCourseSchedule){
		System.out.println(JSON.toJSONString(courseSchedules));
		//第一节课
		CourseSchedule firstCourseSchedule = courseSchedules.stream().min(Comparator.comparing(CourseSchedule::getStartClassTime)).get();
		//最后一节课
		CourseSchedule latestCourseSchedule = courseSchedules.stream().max(Comparator.comparing(CourseSchedule::getStartClassTime)).get();
		//获取第一节课和最后一节课所包含的时间段内已存在的课程
		List<CourseSchedule> existCourseSchedules = courseScheduleDao.findByDateZone(firstCourseSchedule.getStartClassTime(), latestCourseSchedule.getEndClassTime());

		//只需要调整课程信息的课程编号列表
		List<Long> updateCourseScheduleIds = courseSchedules
				.stream()
				.filter(courseSchedule -> Objects.nonNull(courseSchedule.getId()))
				.map(CourseSchedule::getId)
				.collect(Collectors.toList());

		//排除只需调整的课程
		existCourseSchedules=existCourseSchedules.stream()
				.filter(courseSchedule -> !updateCourseScheduleIds.contains(courseSchedule.getId()))
				.collect(Collectors.toList());
		//新课程对应的班级编号列表
		List<Integer> newCourseScheduleClassGroupIds = courseSchedules
				.stream()
				.map(CourseSchedule::getClassGroupId)
				.distinct()
				.collect(Collectors.toList());

		List<Long> existCourseScheduleIds = existCourseSchedules.stream()
				.map(CourseSchedule::getId)
				.collect(Collectors.toList());

		//合并新课程和已存在的课程
		List<CourseSchedule> allCourseSchedules;
		if(!CollectionUtils.isEmpty(existCourseSchedules)){
			allCourseSchedules = ListUtils.sum(courseSchedules, existCourseSchedules);
		}else{
			allCourseSchedules = courseSchedules;
		}
		//所有课程的班级编号
		List<Integer> classGroupIds = allCourseSchedules
				.stream()
				.map(CourseSchedule::getClassGroupId)
				.distinct()
				.collect(Collectors.toList());
		//班级与学生的关联记录
		List<ClassGroupStudentMapper> classGroupStudentMappers = classGroupStudentMapperDao.findByClassGroups(classGroupIds);
		Map<Integer, List<ClassGroupStudentMapper>> classGroupStudentsMap = classGroupStudentMappers
				.stream()
				.collect(Collectors.groupingBy(ClassGroupStudentMapper::getClassGroupId));

		//根据课程获取助教id关联集合
		List<IntegerAndIntegerListDto> courseScheduleTeachingTeacherIdList = new ArrayList<>();
		if(!CollectionUtils.isEmpty(existCourseScheduleIds)){
			courseScheduleTeachingTeacherIdList = courseScheduleDao.findCourseScheduleIdAndUserIdsMap(existCourseScheduleIds, TeachTypeEnum.TEACHING.getCode());
		}
		Map<Integer, IntegerAndIntegerListDto> courseScheduleTeachingTeacherMap = courseScheduleTeachingTeacherIdList.stream()
				.collect(Collectors.toMap(IntegerAndIntegerListDto::getId, integerAndIntegerListDto -> integerAndIntegerListDto));

		//班级助教关联ID集合
		List<IntegerAndIntegerListDto> classGroupAndUserIdsMap = courseScheduleDao.findClassGroupAndUserIdsMap(newCourseScheduleClassGroupIds, TeachTypeEnum.TEACHING.getCode());
		Map<Integer, IntegerAndIntegerListDto> classGroupTeachingTeacherMap = classGroupAndUserIdsMap.stream()
				.collect(Collectors.toMap(IntegerAndIntegerListDto::getId, integerAndIntegerListDto -> integerAndIntegerListDto));

		//将课程计划按照开课时间排序
		allCourseSchedules.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
		if(allCourseSchedules.size()>1){
		    //记录连续冲突的次数
		    Integer repeatTimes=1;
			for (int i=1;i<allCourseSchedules.size();i++){
				for(int j=repeatTimes;j>0;j--){
                    //当前课程
                    CourseSchedule preCourseSchedule = allCourseSchedules.get(i-j);
                    //后面一节课程
                    CourseSchedule backCourseSchedule = allCourseSchedules.get(i);
                    if(!checkExistCourseSchedule
						&&existCourseScheduleIds.contains(preCourseSchedule.getId())
						&&existCourseScheduleIds.contains(backCourseSchedule.getId())){
                    	continue;
					}
                    //判断前后两节课是否存在冲突
                    if(backCourseSchedule.getStartClassTime().before(preCourseSchedule.getEndClassTime())){
                        //提示信息
                        StringBuffer errInfo = new StringBuffer("在");
                        errInfo.append(DateUtil.dateToString(preCourseSchedule.getStartClassTime(),"yyyy-MM-dd HH:mm"));
                        errInfo.append("至");
                        errInfo.append(DateUtil.dateToString(backCourseSchedule.getEndClassTime(),"yyyy-MM-dd HH:mm"));
                        errInfo.append("的时间段内");
                        //如果存在时间重叠，则需要判断前后两节课的教师和学生是否存在冲突
                        //主教冲突检测
                        if(Objects.nonNull(preCourseSchedule.getActualTeacherId())
                                &&preCourseSchedule.getActualTeacherId().equals(backCourseSchedule.getActualTeacherId())){
                            errInfo.append("安排的主教有课程冲突");
                            throw new BizException(errInfo.toString());
                        }
                        //助教冲突检测
						if(Objects.isNull(preCourseSchedule.getId())){
							IntegerAndIntegerListDto integerAndIntegerListDto = classGroupTeachingTeacherMap.get(preCourseSchedule.getClassGroupId());
							if(Objects.nonNull(integerAndIntegerListDto)){
								preCourseSchedule.setTeachingTeacherIdList(integerAndIntegerListDto.getIds());
							}
						}else if(existCourseScheduleIds.contains(preCourseSchedule.getId())){
							IntegerAndIntegerListDto integerAndIntegerListDto = courseScheduleTeachingTeacherMap.get(preCourseSchedule.getId());
							if(Objects.nonNull(integerAndIntegerListDto)){
								preCourseSchedule.setTeachingTeacherIdList(integerAndIntegerListDto.getIds());
							}
						}
						if(Objects.isNull(backCourseSchedule.getId())){
							IntegerAndIntegerListDto integerAndIntegerListDto = classGroupTeachingTeacherMap.get(backCourseSchedule.getClassGroupId());
							if(Objects.nonNull(integerAndIntegerListDto)){
								backCourseSchedule.setTeachingTeacherIdList(integerAndIntegerListDto.getIds());
							}
						}else if(existCourseScheduleIds.contains(backCourseSchedule.getId())){
							IntegerAndIntegerListDto integerAndIntegerListDto = courseScheduleTeachingTeacherMap.get(backCourseSchedule.getId());
							if(Objects.nonNull(integerAndIntegerListDto)){
								backCourseSchedule.setTeachingTeacherIdList(integerAndIntegerListDto.getIds());
							}
						}
						if(!CollectionUtils.isEmpty(preCourseSchedule.getTeachingTeacherIdList())
							&&!CollectionUtils.isEmpty(backCourseSchedule.getTeachingTeacherIdList())){
							List<Integer> repeatIds = preCourseSchedule.getTeachingTeacherIdList()
									.stream().filter(backCourseSchedule.getTeachingTeacherIdList()::contains)
									.collect(Collectors.toList());
							if(!CollectionUtils.isEmpty(repeatIds)){
								errInfo.append("安排的助教存在冲突");
								throw new BizException(errInfo.toString());
							}
						}
                        //学生冲突检测
                        if(preCourseSchedule.getClassGroupId().equals(backCourseSchedule.getClassGroupId())){
                            //如果班级相同，则学生肯定存在冲突
                            errInfo.append("安排的课程存在学生冲突");
                            throw new BizException(errInfo.toString());
                        }
                        //如果班级不同，则需要检测两个班级是否存在重复的学生
                        List<ClassGroupStudentMapper> preClassGroupStudents=classGroupStudentsMap.get(preCourseSchedule.getClassGroupId());
                        List<ClassGroupStudentMapper> backClassGroupStudents=classGroupStudentsMap.get(backCourseSchedule.getClassGroupId());
                        //如果有一个存在没有学生的班级则不存在冲突
                        if(CollectionUtils.isEmpty(preClassGroupStudents)||CollectionUtils.isEmpty(backClassGroupStudents)){
                            continue;
                        }
                        //当前课程所在班级的学生编号列表
                        List<Integer> preClassGroupStudentIds = preClassGroupStudents.stream()
                                .map(ClassGroupStudentMapper::getUserId)
                                .collect(Collectors.toList());
                        //后面一节课程所在班级的学生编号列表
                        List<Integer> backClassGroupStudentIds = backClassGroupStudents.stream()
                                .map(ClassGroupStudentMapper::getUserId)
                                .collect(Collectors.toList());
                        List<Integer> repeatStudentIds = preClassGroupStudentIds.stream()
                                .filter(backClassGroupStudentIds::contains)
                                .collect(Collectors.toList());
                        if(!CollectionUtils.isEmpty(repeatStudentIds)){
                            errInfo.append("安排的课程存在学生冲突");
                            throw new BizException(errInfo.toString());
                        }
						System.out.print(j==repeatTimes);
                        System.out.println(" "+j);
                        if(j==repeatTimes){
                            repeatTimes+=1;
                        }
                    }else{
                        if(j==repeatTimes){
                            repeatTimes=1;
                        }
                    }
                }
			}
		}
	}

	/**
	 * 判断课程安排中是否存在冲突
	 * 计算课程课次
	 * @param courseSchedules
	 * @return
	 */
	private void checkCourseSchedule(List<CourseSchedule> courseSchedules){
		List<Date> classDates = courseSchedules.stream().map(CourseSchedule::getClassDate).collect(Collectors.toList());
		List<CourseSchedule> byClassGroupAndDate = courseScheduleDao.findByClassGroupAndDate(classDates, null);
		List<CourseSchedule> collect3 = Stream.of(courseSchedules, byClassGroupAndDate).flatMap(Collection::stream).collect(Collectors.toList());
		Map<Date, List<CourseSchedule>> collect = collect3.stream().collect(Collectors.groupingBy(CourseSchedule::getClassDate));
		for(Date keyDate:collect.keySet()){
			List<CourseSchedule> currentCourses=collect.get(keyDate);
			Map<Date, List<CourseSchedule>> collect1 = currentCourses.stream().collect(Collectors.groupingBy(CourseSchedule::getStartClassTime));
			if(currentCourses.size()==1&&currentCourses.size()==collect1.size()){
				continue;
			}else if(currentCourses.size()>1&&currentCourses.size()==collect1.size()){
				ArrayList<CourseSchedule> collect2 = currentCourses
						.stream()
						.collect(Collectors.collectingAndThen(Collectors.toCollection(()
								-> new TreeSet<CourseSchedule>(Comparator.comparing(o -> o.getStartClassTime() + "-" + o.getEndClassTime()))), ArrayList::new));
				if(collect2.size()!=currentCourses.size()){
					throw new BizException("建课失败，当前课程安排与现有课程冲突，请调整相关设置");
				}
			}else{
				for(Date keyDate2:collect1.keySet()){
					List<CourseSchedule> currentCourses1=collect1.get(keyDate2);
					if(currentCourses1.size()>ParamEnum.ONE.getCode()){
						Map<Integer, List<CourseSchedule>> collect2 = currentCourses1.stream().collect(
								Collectors.groupingBy(CourseSchedule::getTeacherId));
						for (Integer teacherID:collect2.keySet()){
							if(collect2.get(teacherID).size()>ParamEnum.ONE.getCode()){
								throw new BizException(new SimpleDateFormat("yyyy-MM-dd").format(keyDate)+"日的课程安排在教师分配上有冲突!");
							}
						}
						List<Integer> repeatClassGroupIds = currentCourses1.stream().map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
						Integer[] counts=courseScheduleDao.countStudentInClass(repeatClassGroupIds);
						for (Integer i:counts){
							if (i>ParamEnum.ONE.getCode()){
								throw new BizException(new SimpleDateFormat("yyyy-MM-dd").format(keyDate)+"日的课程安排有学生重复!");
							}
						}
					}
				}
			}
		}
	}

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/29
	 * 生成课程名
	 */
	private void createCourseScheduleName(List<CourseSchedule> courseSchedules){
        List<Integer> classGroupIds = courseSchedules.stream().map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
        HashSet<Integer> temp=new HashSet<>(classGroupIds);
        classGroupIds=new ArrayList<>(temp);
        Map<Integer,String> classIdSubjectNameMap=convertToMap(courseScheduleDao.findClassSubjects(classGroupIds));
        courseSchedules.forEach(courseSchedule -> {
            StringBuilder className=new StringBuilder(classIdSubjectNameMap.get(courseSchedule.getClassGroupId()));
            className.append("-");
            className.append(courseSchedule.getType().getMsg());
            courseSchedule.setName(className.toString());
            courseSchedule.setStatus(CourseStatusEnum.NOT_START);
        });
    }

	private Map<Integer,String> convertToMap(String[] result){
		Map<Integer,String> map=new HashMap<>();
		for(String s:result){
			map.put(Integer.valueOf(s.split("-")[0]),s.split("-")[1].replace(",","/"));
		}
		return map;
	}

	@Override
	public void courseAdjust(List<CourseSchedule> newCourseSchedules) {
		checkNewCourseSchedules(newCourseSchedules,false);
		classStartDateAdjust(newCourseSchedules);
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void classStartDateAdjust(List<CourseSchedule> newCourseSchedules) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		List<Long> newCourseScheduleIds = newCourseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());
		//获取对应未修改的课程列表
		List<CourseSchedule> oldCourseScheduleList = courseScheduleDao.findByCourseScheduleIds(newCourseScheduleIds);
		if(oldCourseScheduleList.size()!=newCourseSchedules.size()){
			throw new BizException("课程信息错误");
		}
		Integer courseScheduleId = null;

		Date now=new Date();
		//明天日期
		Date tomorrow = DateUtil.addDays1(now, 1);

		//需要生成课程课酬的课程计划
		List<CourseSchedule> needCreateCourseScheduleTeacherSalaryCourse=new ArrayList<>();

		//课程对应助教集合
		List<CourseScheduleTeacherSalary> teachingTeacherSalarys = courseScheduleDao.findTeachingTeachers(newCourseScheduleIds);
		Map<Long,List<CourseScheduleTeacherSalary>> courseTeachingTeacherMaps = teachingTeacherSalarys.stream().collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId));

		Map<Long, CourseSchedule> oldCourseSchedules = oldCourseScheduleList.stream().collect(Collectors.toMap(CourseSchedule::getId,courseSchedule -> courseSchedule));
		newCourseSchedules.forEach(newCourseSchedule -> {

			if(newCourseSchedule.getStartClassTime().before(tomorrow)&&sysUser.getUserType().equals(SysUserType.TEACHER)){
			    throw new BizException("调整时间必须为明天及以后");
            }

			CourseSchedule oldCourseSchedule = oldCourseSchedules.get(newCourseSchedule.getId());

            if(oldCourseSchedule.getStartClassTime().before(now)&&sysUser.getUserType().equals(SysUserType.TEACHER)){
                throw new BizException("不能调整今天及以前的课程");
            }

			oldCourseSchedule.setClassDate(newCourseSchedule.getClassDate());
			oldCourseSchedule.setStartClassTime(newCourseSchedule.getStartClassTime());
			oldCourseSchedule.setEndClassTime(newCourseSchedule.getEndClassTime());
			courseScheduleDao.update(oldCourseSchedule);

			//被修改的教师编号列表，需要将预计薪水置零
			List<Integer> beReplaceTeacherIds=new ArrayList<>();

			if(oldCourseSchedule.getType()== CourseSchedule.CourseScheduleType.VIP){
				if(Objects.isNull(oldCourseSchedule.getActualTeacherId())
						||(!oldCourseSchedule.getActualTeacherId().equals(newCourseSchedule.getActualTeacherId())
						&&Objects.nonNull(newCourseSchedule.getActualTeacherId()))){
					VipGroup byCourseSchedule = vipGroupDao.findByCourseSchedule(newCourseSchedule.getId().intValue());
					TeacherDefaultVipGroupSalary byTeacherAndCategory = teacherDefaultVipGroupSalaryDao.findByTeacherAndCategory(newCourseSchedule.getActualTeacherId(), byCourseSchedule.getVipGroupCategoryId());
					List<CourseSchedule> courseSchedules=new ArrayList<>();
					courseSchedules.add(newCourseSchedule);
					courseScheduleTeacherSalaryService.createCourseScheduleTeacherVipSalary(byCourseSchedule,
							courseSchedules,
							byTeacherAndCategory.getOnlineClassesSalary(),
							byTeacherAndCategory.getOfflineClassesSalary());
					if(Objects.nonNull(oldCourseSchedule.getActualTeacherId())){
						beReplaceTeacherIds.add(oldCourseSchedule.getActualTeacherId().intValue());
					}
				}
			}else{
				if(Objects.isNull(oldCourseSchedule.getActualTeacherId())
						||!oldCourseSchedule.getActualTeacherId().equals(newCourseSchedule.getActualTeacherId())){
					if(Objects.nonNull(oldCourseSchedule.getActualTeacherId())){
						beReplaceTeacherIds.add(oldCourseSchedule.getActualTeacherId().intValue());
					}
				}else{
					//如果主教未修改则不需要调整课酬记录
					newCourseSchedule.setBishopTeacherSalaryNotCreate(true);
				}
				List<CourseScheduleTeacherSalary> courseTeachingTeacherList=courseTeachingTeacherMaps.get(newCourseSchedule.getId());
				//当前设置的课程助教编号列表
				List<Integer> currentCourseTeachingTeacherIds=new ArrayList<>();
				if(!CollectionUtils.isEmpty(courseTeachingTeacherList)){
					currentCourseTeachingTeacherIds = courseTeachingTeacherList.stream().map(CourseScheduleTeacherSalary::getUserId).collect(Collectors.toList());
				}
				//修改后的课程助教编号列表
				List<Integer> newTeachingTeacherIdList = newCourseSchedule.getTeachingTeacherIdList();
				//找出重复的助教编号
				List<Integer> foundInCurrentCourseTeachingTeacherIds = newTeachingTeacherIdList.stream().filter(currentCourseTeachingTeacherIds::contains).collect(Collectors.toList());

				//需要重新生成课酬的助教列表
				List<Integer> newCreateSalaryTeachingTeacherIds=new ArrayList<>();

				newTeachingTeacherIdList.forEach(newTeachingTeacherId->{
					//如果不在重复列表中，则需要生成对应的课酬记录
					if(!foundInCurrentCourseTeachingTeacherIds.contains(newTeachingTeacherId)){
						//计算修改后的助教的课酬
						newCreateSalaryTeachingTeacherIds.add(newTeachingTeacherId);

						if(Objects.nonNull(newTeachingTeacherId)){
							beReplaceTeacherIds.add(newTeachingTeacherId);
						}
					}
				});
				newCourseSchedule.setTeachingTeacherIdList(newCreateSalaryTeachingTeacherIds);
				if(!newCourseSchedule.isBishopTeacherSalaryNotCreate()||!CollectionUtils.isEmpty(newCreateSalaryTeachingTeacherIds)){
					//如果主教或者助教发生了变化，则需要重新生成课酬
					needCreateCourseScheduleTeacherSalaryCourse.add(newCourseSchedule);
				}
			}
			//将被修改的教师预计薪水置零
			if(!CollectionUtils.isEmpty(beReplaceTeacherIds)){
				courseScheduleTeacherSalaryDao.batchUpdateTeacherExpectSalary(newCourseSchedule.getId().intValue(),beReplaceTeacherIds);
			}
		});
		//生成修改后的教师课酬
		if(!CollectionUtils.isEmpty(needCreateCourseScheduleTeacherSalaryCourse)){
			classGroupTeacherMapperService.createCourseScheduleTeacherSalary(needCreateCourseScheduleTeacherSalaryCourse,oldCourseSchedules);
		}
		try {
			//推送课时调整
			Set<Integer> teacherIds = newCourseSchedules.stream().map(e -> e.getActualTeacherId()).collect(Collectors.toSet());
			if(teacherIds == null){
				teacherIds = new HashSet<>();
			}
			teacherIds.addAll(oldCourseScheduleList.stream().map(e -> e.getActualTeacherId()).collect(Collectors.toSet()));
			Map<Integer,String> map = new HashMap<>(teacherIds.size());
			teacherIds.forEach(e->{
				map.put(e,e.toString());
			});
//		HashMap<Integer,String> map = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertMybatisMap(teacherDao.queryPhoneByIds(StringUtils.join(teacherIds, ",")))), HashMap.class);
			if(map != null && map.size() > 0){
				sysMessageService.batchSendMessage(MessageSender.JIGUANG,MessageTypeEnum.TEACHER_PUSH_COURSE_SCHEDULE_CHANGE_RESULT,
						map,null,0,"7");
			}
		}catch (Exception e){
			e.printStackTrace();
		}
//		Set<Integer> roleIds = new HashSet<>(1);
//		roleIds.add(SysUserRole.SECTION_MANAGER);
//		classGroupDao.get();
//		sysMessageService.batchSeoMessage(musicGroupDao.queryUserIdByRoleId(roleIds),MessageTypeEnum.BACKSTAGE_COURSE_CHANGE,
//				teacher.getRealName());
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public void courseSwap(Long courseScheduleId1, Long courseScheduleId2) {
		if(courseScheduleId1.equals(courseScheduleId2)){
			throw new BizException("请选择不同的课程");
		}
		CourseSchedule courseSchedule1 = courseScheduleDao.get(courseScheduleId1);
		CourseSchedule courseSchedule2 = courseScheduleDao.get(courseScheduleId2);

		if(Objects.isNull(courseSchedule1)||Objects.isNull(courseSchedule2)){
			throw new BizException("未找到对应课程");
		}

		//获取两个课程的主教、助教
		List<Long> courseScheduleIds=new ArrayList<>();
		courseScheduleIds.add(courseScheduleId1);
		courseScheduleIds.add(courseScheduleId2);
		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseScheduleIds);
		Map<Long, List<CourseScheduleTeacherSalary>> courseScheduleTeacherSalaryByCourse = courseScheduleTeacherSalaries.stream()
				.collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId));
		//课程1的主教、助教
		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries1 = courseScheduleTeacherSalaryByCourse.get(courseScheduleId1);
		//课程2的主教、助教
		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries2 = courseScheduleTeacherSalaryByCourse.get(courseScheduleId2);

		Map<TeachTypeEnum, List<CourseScheduleTeacherSalary>> ct1 = courseScheduleTeacherSalaries1.stream()
				.collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getTeacherRole));

		Map<TeachTypeEnum, List<CourseScheduleTeacherSalary>> ct2 = courseScheduleTeacherSalaries2.stream()
				.collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getTeacherRole));

		//交换两节课的主教、助教
		if(Objects.nonNull(ct1.get(TeachTypeEnum.BISHOP))){
			courseSchedule1.setActualTeacherId(ct1.get(TeachTypeEnum.BISHOP).get(0).getUserId());
		}else{
			courseSchedule1.setActualTeacherId(null);
		}
		if(Objects.nonNull(ct2.get(TeachTypeEnum.BISHOP))){
			courseSchedule2.setActualTeacherId(ct2.get(TeachTypeEnum.BISHOP).get(0).getUserId());
		}else{
			courseSchedule2.setActualTeacherId(null);
		}
		List<CourseScheduleTeacherSalary> ct1t = ct1.get(TeachTypeEnum.TEACHING);
		if(CollectionUtils.isEmpty(ct1t)){
			courseSchedule1.setTeachingTeacherIdList(null);
		}else{
			List<Integer> collect = ct1t.stream().map(CourseScheduleTeacherSalary::getUserId).collect(Collectors.toList());
			collect.stream().distinct();
			courseSchedule1.setTeachingTeacherIdList(collect);
		}
		List<CourseScheduleTeacherSalary> ct2t = ct2.get(TeachTypeEnum.TEACHING);
		if(CollectionUtils.isEmpty(ct2t)){
			courseSchedule2.setTeachingTeacherIdList(null);
		}else{
			List<Integer> collect = ct2t.stream().map(CourseScheduleTeacherSalary::getUserId).collect(Collectors.toList());
			collect.stream().distinct();
			courseSchedule2.setTeachingTeacherIdList(collect);
		}
		courseSchedule1.setId(courseScheduleId2);
		courseSchedule2.setId(courseScheduleId1);
		List<CourseSchedule> courseSchedules=new ArrayList<>();
		courseSchedules.add(courseSchedule1);
		courseSchedules.add(courseSchedule2);
		classStartDateAdjust(courseSchedules);
	}

	@Override
	public void courseSwapWithDay(Date day1, Date day2) {

	}

	@Override
    public List<CourseSchedule> findClassGroupNoStartCourseSchedules(List<ClassGroupTeacherMapper> classGroupTeacherMapperList) {
		return courseScheduleDao.findClassGroupNoStartCourseSchedules(classGroupTeacherMapperList);
    }

    /**
	 * @Author: Joburgess
	 * @Date: 2019/9/24
	 * 检测学生是否存在重复
	 */
	private void checkStudentIsRepeat(List<Integer> studentIds){
		Integer[] counts=courseScheduleDao.countStudentInClass(studentIds);
		for (Integer i:counts){
			if (i>ParamEnum.ONE.getCode()){
				throw new BizException("所选课程安排日期有学生重复!");
			}
		}
	}

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/24
	 * 根据课时调整信息增加新课程
	 */
	private void createNewCourseByAdjust(List<Date> dates,ClassDateAdjustDto classDateAdjustDto){
		CourseSchedule schedule=courseScheduleDao.get(classDateAdjustDto.getId());
		List<CourseSchedule> courseSchedules=new ArrayList<>();
		courseScheduleDao.delete(classDateAdjustDto.getId());
		for (Date date:dates){
			CourseSchedule courseSchedule=new CourseSchedule();
			BeanUtils.copyProperties(schedule,courseSchedule);
			courseSchedule.setClassDate(date);
			courseSchedule.setStartClassTime(classDateAdjustDto.getStartClassTime());
			courseSchedule.setEndClassTime(classDateAdjustDto.getEndClassTime());
			courseSchedule.setTeacherId(classDateAdjustDto.getTeacherId());
			courseSchedules.add(courseSchedule);
		}
		courseScheduleDao.batchAddCourseSchedules(courseSchedules);
	}

	@Override
	public Object findStudentCourseScheduleRecords(StudentCourseScheduleRecordQueryInfo queryInfo) {
		SysUser user = sysUserFeignService.queryUserInfo();
		queryInfo.setUserId(user.getId().longValue());
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

		if(queryInfo.getIsAttend()== YesOrNoEnum.YES){
			pageInfo.setRows(courseScheduleDao.findStudentCourseScheduleNotStartRecords(user.getId().longValue(),
					queryInfo.getDate()));
			return pageInfo;
		}

		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List dataList = null;
		int count = courseScheduleDao.countStudentCourseScheduleRecords(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = courseScheduleDao.findStudentCourseScheduleRecords(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public void courseScheduleCommplaint(CourseScheduleComplaints courseScheduleComplaints) {
		SysUser user = sysUserFeignService.queryUserInfo();

		if(Objects.isNull(user)){
			throw new BizException("请登录");
		}

		if(Objects.isNull(courseScheduleComplaints.getCourseScheduleId())){
			throw new BizException("请指定需要投诉的课程");
		}

		if(StringUtils.isBlank(courseScheduleComplaints.getReason())){
			throw new BizException("请说明投诉原因");
		}

		if(StringUtils.length(courseScheduleComplaints.getReason())>150){
			throw new BizException("字符长度超限");
		}

		courseScheduleComplaints.setUserId(user.getId());

		CourseScheduleComplaints byUserAndCourse = courseScheduleComplaintsDao.findByUserAndCourse(courseScheduleComplaints.getUserId().longValue(), courseScheduleComplaints.getCourseScheduleId());
		if(byUserAndCourse!=null){
			throw new BizException("您已经对该课程进行过申诉操作!");
		}

		CourseScheduleDto courseSchedules = courseScheduleDao.getCourseSchedules(courseScheduleComplaints.getCourseScheduleId());
		Date now=new Date();
		if(!courseSchedules.getStatus().equals(CourseStatusEnum.OVER)){
			throw new BizException("该课程还未结束");
		}
		int i = DateUtil.daysBetween(courseSchedules.getEndClassTime(), now);
		if(i>Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.VIP_APPEAL_DAYS_RANGE))){
			throw new BizException("该课程已超过可申诉时间范围!");
		}
		courseScheduleComplaints.setUserId(user.getId());
		courseScheduleComplaints.setStatus(AuditStatusEnum.ING);
		courseScheduleComplaintsDao.insert(courseScheduleComplaints);
		//学员提起申诉通知
		CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleComplaints.getCourseScheduleId());
		Set<Integer> roleIds = new HashSet<>(1);
		roleIds.add(SysUserRole.EDUCATIONAL_TEACHER);
		sysMessageService.batchSeoMessage(musicGroupDao.queryUserIdByRoleId(roleIds),MessageTypeEnum.BACKSTAGE_STUDENT_APPEAL,"",courseSchedule.getName(),user.getUsername());
	}

	@Override
	public void courseScheduleCommplaintAudit(Long id, Double teacherLiabilityRatio, Double studentLiabilityRatio, AuditStatusEnum status) {

		if(status == AuditStatusEnum.ING){
			throw new BizException("审批状态不正确");
		}
		
		CourseScheduleComplaints courseScheduleComplaints = courseScheduleComplaintsDao.get(id);
		if (Objects.isNull(courseScheduleComplaints)) {
			throw new BizException("投诉记录不存在");
		}
		courseScheduleComplaints.setTeacherLiabilityRatio(teacherLiabilityRatio);
		courseScheduleComplaints.setStudentLiabilityRatio(studentLiabilityRatio);
		courseScheduleComplaints.setStatus(status);
		courseScheduleComplaintsDao.update(courseScheduleComplaints);
	}

	@Override
	public boolean updateCourseScheduleToOverStatus() {
		List<CourseSchedule> list = courseScheduleDao.queryFinishedWithNoUpdateStatus();

		List<CourseSchedule> updateList = new ArrayList<CourseSchedule>();
		Date date = new Date();
		List<Integer> classGroups = new ArrayList<Integer>();
		for (CourseSchedule courseSchedule : list) {
			courseSchedule.setStatus(CourseStatusEnum.OVER);
			courseSchedule.setUpdateTime(date);
			updateList.add(courseSchedule);
			
			if(!classGroups.contains(courseSchedule.getClassGroupId())){
				classGroups.add(courseSchedule.getClassGroupId());
			}
		}

		if (updateList.size() > 0) {
			courseScheduleDao.batchUpdate(updateList);
		}
		
		//更新班级课次
		if(classGroups.size()>0){
			classGroupService.batchIncreaseClassTimes(classGroups);
		}
		
		return true;
	}

	@Override
	public PageInfo queryTeacherClassCourseSchedule(CourseScheduleQueryInfo queryInfo) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(null==user){
			throw new BizException("请登录");
		}
		queryInfo.setUserId(user.getId().longValue());
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

		if(Objects.isNull(queryInfo.getDate())){
			queryInfo.setDate(new Date());
		}

		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<TeacherClassCourseSchudeleDto> dataList = null;
		int count = 0;
		if(Objects.isNull(queryInfo.getOnlyHistory())||queryInfo.getOnlyHistory()==0){
			count = courseScheduleDao.countTeacherClassCourseSchedule(params);
		}else{
			count= courseScheduleDao.countTeacherHistoryClassCourseSchedule(params);
		}
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			if(Objects.isNull(queryInfo.getOnlyHistory())||queryInfo.getOnlyHistory()==0){
				dataList = courseScheduleDao.queryTeacherClassCourseSchedule(params);
			}else{
				dataList = courseScheduleDao.queryTeacherHistoryClassCourseSchedule(params);
			}
			List<Integer> courseIds = dataList.stream().map(teacherClassCourseSchudeleDto -> teacherClassCourseSchudeleDto.getCourseScheduleId().intValue()).collect(Collectors.toList());
			List<Map<Integer, Integer>> currentClassTimesByCourses = studentAttendanceDao.getCurrentClassTimesByCourses(courseIds);
			if(!CollectionUtils.isEmpty(currentClassTimesByCourses)){
				Map<Integer, Integer> currentClassTimesWithCourse = MapUtil.convertIntegerMap(currentClassTimesByCourses);
				dataList.forEach(e->{
					e.setCurrentClassTimes(currentClassTimesWithCourse.get(e.getCourseScheduleId()));
				});
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public CourseAttendanceDetailHeadInfoDto findCourseAttendanceDetailHeadInfo(Long courseScheduleId) {
		if(Objects.isNull(courseScheduleId)){
			throw new BizException("请指定课程");
		}
		CourseAttendanceDetailHeadInfoDto courseAttendanceDetailHeadInfoDto = courseScheduleDao.findByCourse(courseScheduleId);
		courseAttendanceDetailHeadInfoDto.setLatestAttendanceTime(studentAttendanceDao.findLatestAttendanceDate(courseScheduleId));
		return courseAttendanceDetailHeadInfoDto;
	}

	@Override
	public void checkMusicGroupClassCourse(String musicGroupId) {
		if(Objects.isNull(musicGroupId)){
			throw new BizException("请指定乐团");
		}
		List<Map<Integer, Long>> maps = courseScheduleDao.countClassCourseNumByMusicGroup(musicGroupId);
		Map<Integer,Long> classCourseNumMap = MapUtil.convertIntegerMap(maps);
		List<Integer> collect = classCourseNumMap
								.entrySet()
								.stream()
								.filter(integerLongEntry -> integerLongEntry.getValue() <= 0)
								.map(Map.Entry::getKey)
								.collect(Collectors.toList());
		if(!CollectionUtils.isEmpty(collect)){
			List<String> classGroupNamesByClassGroups = classGroupDao.findClassGroupNamesByClassGroups(collect);
			String classNamesStr=StringUtils.join(classGroupNamesByClassGroups,",");
			throw new BizException(classNamesStr+"班级未排课");
		}
	}

	@Override
	public Map<Integer, Integer> findCourseScheduleCurrentTimes(List<Integer> courseSchedules) {
		List<Map<Integer, Integer>> courseScheduleCurrentTimes = courseScheduleDao.findCourseScheduleCurrentTimes(courseSchedules);
		return MapUtil.convertIntegerMap(courseScheduleCurrentTimes);
	}

    @Override
    public List<CourseSchedule> findNoStartCoursesByClassGroupId(Integer classGroupId) {
		return courseScheduleDao.findNoStartCoursesByClassGroupId(classGroupId);
    }

	@Override
	public void studentCoursesScheduleRemind() {
		List<Mapper> mapperList = courseScheduleDao.queryStudentCoursesTimesOfTomorrow();
		for (Mapper mapper : mapperList) {
			Integer userId = Integer.parseInt(mapper.getKey().toString()) ;
			Integer size = Integer.parseInt(mapper.getValue().toString());

			Map<Integer, String> receivers = new HashMap<Integer, String>();
			receivers.put(userId, userId + "");

			sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_HAVE_COURSE_MESSAGE, receivers, null, 0, "2", size);
		}
	}

	@Override
	public void studentSignInRemind() {
		List<Mapper> mapperList = courseScheduleDao.queryStudentNotStartCourseScheduleIn90Mins();
		if(mapperList != null && mapperList.size() > 0){
			for(Mapper mapper : mapperList){
				Map<Integer, String> receivers = new HashMap<Integer, String>();
				receivers.put((Integer) mapper.getKey(), mapper.getKey() + "");
				
				sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_ATTEND_COURSE_MESSAGE, receivers, null, 0, "2", mapper.getValue());
			}
		}
	}

	@Override
	public void teacherCoursesScheduleRemind() {
		List<Mapper> mapperList = courseScheduleDao.queryTeacherCoursesTimesOfTomorrow();
		for (Mapper mapper : mapperList) {
			Integer userId = (Integer) mapper.getKey();
			String teacherName = (String) mapper.getValue();

			Map<Integer, String> receivers = new HashMap<Integer, String>();
			receivers.put(userId, userId + "");

			sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_HAVE_COURSE_MESSAGE,
					receivers, null, 0, "2", teacherName);
		}
	}

	@Override
	public void teacherSignInRemind() {
		List<Mapper> mapperList = courseScheduleDao.queryTeacherNotStartCourseScheduleIn30Mins();
		for (Mapper mapper : mapperList) {
			Integer userId = (Integer) mapper.getKey();
			String scheduleName = (String) mapper.getValue();

			Map<Integer, String> receivers = new HashMap<Integer, String>();
			receivers.put(userId, userId + "");

			sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_ATTEND_CLASS_MESSAGE,
                    receivers, null, 0, "2", scheduleName);
		}
	}

	@Override
	public List<CourseSchedule> findCourseScheduleByClassGroup(Integer classGroupId) {
		if(Objects.isNull(classGroupId)){
			throw new BizException("请指定班级");
		}
		ClassGroup classGroup = classGroupDao.get(classGroupId);
		if(Objects.isNull(classGroup)){
			throw new BizException("指定的班级不存在");
		}
		List<Integer> classGroupIds=new ArrayList<>();
		classGroupIds.add(classGroupId);
		if(classGroup.getType().equals(ClassGroupTypeEnum.MIX)){
			List<ClassGroup> mixClassChildClassGroup = classGroupDao.findMixClassChildClassGroup(classGroupId);
			List<Integer> childClassGroupIds = mixClassChildClassGroup.stream()
					.map(ClassGroup::getId)
					.collect(Collectors.toList());
			classGroupIds.addAll(childClassGroupIds);
		}
		return courseScheduleDao.findByClassGroups(classGroupIds);
	}
}
