package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.TeachModeEnum;
import com.ym.mec.biz.dal.enums.VipGroupActivityTypeEnum;
import com.ym.mec.biz.service.CourseScheduleStudentPaymentService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class CourseScheduleStudentPaymentServiceImpl extends BaseServiceImpl<Long, CourseScheduleStudentPayment>  implements CourseScheduleStudentPaymentService {
	
	@Autowired
	private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
	@Autowired
	private VipGroupDao vipGroupDao;
	@Autowired
	private VipGroupActivityDao vipGroupActivityDao;
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private ClassGroupStudentMapperDao classGroupStudentMapperDao;

	@Override
	public BaseDAO<Long, CourseScheduleStudentPayment> getDAO() {
		return courseScheduleStudentPaymentDao;
	}

	@Override
	public int createCourseScheduleStudentPaymentForVipGroup(Long vipGroupId, Integer userId) {
		VipGroup vipGroupApplyBaseInfoDto=vipGroupDao.get(vipGroupId);

		if(Objects.isNull(vipGroupApplyBaseInfoDto)){
			throw new BizException("未找到相关vip课程");
		}

		//获取活动信息
		VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroupApplyBaseInfoDto.getVipGroupActivityId().intValue());

		List<CourseSchedule> vipGroupCourseSchedules=courseScheduleDao.findVipGroupCourseSchedules(vipGroupId);
		if(CollectionUtils.isEmpty(vipGroupCourseSchedules)){
			throw new BizException("未获取到排课信息");
		}
		List<CourseScheduleStudentPayment> courseScheduleStudentPayments=new ArrayList<>();
		Map<TeachModeEnum, List<CourseSchedule>> courseScheduleGroupByTeachMode = vipGroupCourseSchedules.stream().collect(Collectors.groupingBy(CourseSchedule::getTeachMode));
		for(TeachModeEnum teachModeEnum:courseScheduleGroupByTeachMode.keySet()){
			List<CourseSchedule> courseSchedules = courseScheduleGroupByTeachMode.get(teachModeEnum)
					.stream()
					.sorted(Comparator.comparing(CourseSchedule::getStartClassTime)).collect(Collectors.toList());
			for(int i=0;i<courseSchedules.size();i++) {
				//创建学生课程应缴费记录
				CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
				courseScheduleStudentPayment.setUserId(userId);
				courseScheduleStudentPayment.setCourseScheduleId(courseSchedules.get(i).getId());
				courseScheduleStudentPayment.setClassGroupId(courseSchedules.get(i).getClassGroupId());

				if (teachModeEnum == TeachModeEnum.ONLINE) {
					//学生线上单节课应缴费计算
					switch (vipGroupActivity.getType()) {
						case BASE_ACTIVITY:
							courseScheduleStudentPayment.setExpectPrice(vipGroupApplyBaseInfoDto.getOnlineClassesUnitPrice());
							break;
						case DISCOUNT:
							courseScheduleStudentPayment.setExpectPrice(
									vipGroupApplyBaseInfoDto.getOnlineClassesUnitPrice().multiply(
											new BigDecimal(vipGroupActivity.getAttribute1())
													.divide(new BigDecimal(100)).setScale(2,BigDecimal.ROUND_HALF_UP)
									)
							);
							break;
						case GIVE_CLASS:
							if (vipGroupActivity.getType() == VipGroupActivityTypeEnum.GIVE_CLASS
									&& vipGroupApplyBaseInfoDto.getGiveTeachMode() == teachModeEnum
									&& i >= Integer.parseInt(vipGroupActivity.getAttribute1())) {
								courseScheduleStudentPayment.setExpectPrice(new BigDecimal(0));
							} else {
								courseScheduleStudentPayment.setExpectPrice(vipGroupApplyBaseInfoDto.getOnlineClassesUnitPrice());
							}
							break;
						default:
							throw new BizException("活动类型错误!");
					}
				}else{
					//学生线下单节课应缴费计算
					switch (vipGroupActivity.getType()) {
						case BASE_ACTIVITY:
							courseScheduleStudentPayment.setExpectPrice(vipGroupApplyBaseInfoDto.getOfflineClassesUnitPrice());
							break;
						case DISCOUNT:
							courseScheduleStudentPayment.setExpectPrice(
									vipGroupApplyBaseInfoDto.getOfflineClassesUnitPrice().multiply(
											new BigDecimal(vipGroupActivity.getAttribute1())
													.divide(new BigDecimal(100)).setScale(2,BigDecimal.ROUND_HALF_UP)
									)
							);
							break;
						case GIVE_CLASS:
							if (vipGroupActivity.getType() == VipGroupActivityTypeEnum.GIVE_CLASS
									&& vipGroupApplyBaseInfoDto.getGiveTeachMode() == teachModeEnum
									&& i >= Integer.parseInt(vipGroupActivity.getAttribute1())) {
								courseScheduleStudentPayment.setExpectPrice(new BigDecimal(0));
							} else {
								courseScheduleStudentPayment.setExpectPrice(vipGroupApplyBaseInfoDto.getOfflineClassesUnitPrice());
							}
							break;
						default:
							throw new BizException("活动类型错误!");
					}
				}
				courseScheduleStudentPayments.add(courseScheduleStudentPayment);
			}
		}
		return courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPayments);
	}

    @Override
    public int deleteStudentCourseSchedule(Integer userId, List<CourseSchedule> courseScheduleList) {
		return courseScheduleStudentPaymentDao.deleteStudentCourseSchedule(userId,courseScheduleList);
    }

	@Override
	public void createCourseScheduleStudentPaymentByCourseSchedules(List<CourseSchedule> courseSchedules) {
		if(CollectionUtils.isEmpty(courseSchedules)){
			return;
		}

		List<CourseScheduleStudentPayment> courseScheduleStudentPayments=new ArrayList<>();

		BigDecimal zeroPrice=new BigDecimal(0);

		//班级编号列表
		List<Integer> classGroupIds = courseSchedules.stream().map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
		Map<Integer, List<CourseSchedule>> courseSchedulesByClassGroupMap = courseSchedules.stream().collect(Collectors.groupingBy(CourseSchedule::getClassGroupId));

		//班级学生对应列表
		List<ClassGroupStudentMapper> classGroupStudentMappers = classGroupStudentMapperDao.findByClassGroups(classGroupIds);
		Map<Integer, List<ClassGroupStudentMapper>> studentsByClassMap = classGroupStudentMappers.stream().collect(Collectors.groupingBy(ClassGroupStudentMapper::getClassGroupId));

		courseSchedulesByClassGroupMap.forEach((classGroupId,courseSchedulesByClassGroup)->{
			List<ClassGroupStudentMapper> studentsWithClassGroup = studentsByClassMap.get(classGroupId);
			if(CollectionUtils.isEmpty(studentsWithClassGroup)){
				return;
			}
			studentsWithClassGroup.forEach(classGroupStudent->{
				courseSchedulesByClassGroup.forEach(courseSchedule -> {
					courseScheduleStudentPayments.add(new CourseScheduleStudentPayment(
							courseSchedule.getId(),
							classGroupStudent.getUserId(),
							zeroPrice,
							courseSchedule.getClassGroupId()));
				});
			});
		});
		courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPayments);
	}

	@Override
	public int batchInsert(List<CourseScheduleStudentPayment> courseScheduleStudentPayments) {
		return courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPayments);
	}
}
