package com.ym.mec.biz.service.impl;

import java.util.*;

import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.entity.MusicGroup;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ym.mec.biz.dal.dao.MusicGroupStudentFeeDao;
import com.ym.mec.biz.dal.dao.SysConfigDao;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus;
import com.ym.mec.biz.dal.entity.SysConfig;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.service.MusicGroupStudentFeeService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;

@Service
public class MusicGroupStudentFeeServiceImpl extends BaseServiceImpl<Long, MusicGroupStudentFee> implements MusicGroupStudentFeeService {

	@Autowired
	private MusicGroupStudentFeeDao musicGroupStudentFeeDao;

	@Autowired
	private SysConfigDao sysConfigDao;

	@Autowired
	private MusicGroupDao musicGroupDao;

	@Autowired
	private SysMessageService sysMessageService;

	@Override
	public BaseDAO<Long, MusicGroupStudentFee> getDAO() {
		return musicGroupStudentFeeDao;
	}

	@Override
	public boolean refreshPaymentFeeStatus() {

		int days = Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.REFRESH_PAYMENT_STATUS_EARLY_DAYS));

		Date date = new Date();

		List<MusicGroupStudentFee> updateList = new ArrayList<MusicGroupStudentFee>();

		List<MusicGroupStudentFee> musicGroupStudentFeeList = musicGroupStudentFeeDao.queryWillRenewList(days);

		for (MusicGroupStudentFee musicGroupStudentFee : musicGroupStudentFeeList) {
			if (musicGroupStudentFee.getPaymentStatus() == PaymentStatus.PAID_COMPLETED) {
				musicGroupStudentFee.setPaymentStatus(PaymentStatus.NON_PAYMENT);
				musicGroupStudentFee.setUpdateTime(date);
				updateList.add(musicGroupStudentFee);
			}
		}

		if (updateList.size() > 0) {
			musicGroupStudentFeeDao.batchUpdate(updateList);
			MusicGroup musicGroup = musicGroupDao.get(updateList.get(0).getMusicGroupId());
			Map<Integer, String> push = new HashMap<Integer, String>();
			for (MusicGroupStudentFee sf : updateList) {
				push.put(sf.getUserId(), sf.getUserId() + "");
			}
			String memo = "4?http://mstudev.dayaedu.com/#/renew?musicGroupId=" + musicGroup.getId();
			// 发送续费通知
			sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push, null, 0, memo,musicGroup.getName());
		}

		// int i = DateUtil.daysBetween(new Date(), musicGroupStudentFee.getNextPaymentDate());
		// e.setRenewStatus(i < 8 ? 0 : 1);

		return true;
	}

}