package com.ym.mec.biz.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.huifu.adapay.model.payment.Payment;
import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.OrderDetailTypeEnum;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.biz.service.StudentRegistrationService;
import com.ym.mec.biz.service.VipGroupService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.adapay.Pay;
import com.ym.mec.thirdparty.yqpay.Msg;
import com.ym.mec.thirdparty.yqpay.RsqMsg;
import com.ym.mec.thirdparty.yqpay.YqPayFeignService;
import com.ym.mec.thirdparty.yqpay.YqPayUtil;

@Service
public class StudentPaymentOrderServiceImpl extends BaseServiceImpl<Long, StudentPaymentOrder> implements StudentPaymentOrderService {

    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private YqPayFeignService yqPayFeignService;
    @Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    private VipGroupService vipGroupService;
    @Autowired
    private MusicGroupService musicGroupService;

    @Override
    public BaseDAO<Long, StudentPaymentOrder> getDAO() {
        return studentPaymentOrderDao;
    }

    @Override
    public List<Goods> queryApplyGoodsList(String musicGroupId, OrderDetailTypeEnum type) {
        return studentPaymentOrderDao.queryApplyGoodsList(musicGroupId, type);
    }

    @Override
    public StudentPaymentOrder findMusicGroupApplyOrderByStatus(Integer userId,String musicGroupId, DealStatusEnum status) {
        return studentPaymentOrderDao.findMusicGroupApplyOrderByStatus(userId,musicGroupId, status);
    }

    @Override
    public StudentPaymentOrder findOrderByOrderNo(String orderNo) {
        return studentPaymentOrderDao.findOrderByOrderNo(orderNo);
    }

    @Override
    public int findPayOrderNum() {
        return studentPaymentOrderDao.findPayOrderNum();
    }

    @Override
    public List<StudentPaymentOrder> findOrdersByStatus(DealStatusEnum status,String paymentChannel) {
        return studentPaymentOrderDao.findOrdersByStatus(status,paymentChannel);
    }

    @Override
    public List<StudentPaymentOrder> findOrdersOverTime(List<String> orderNoList, DealStatusEnum status, Date beforeTime) {
        return studentPaymentOrderDao.findOrdersOverTime(orderNoList,status,beforeTime);
    }

    @Override
    public void queryOrderStatus() throws Exception {
           yqPayQuery();
           adaPayQuery();
    }

    private void yqPayQuery() throws Exception {
        List<StudentPaymentOrder> payingOrders = findOrdersByStatus(DealStatusEnum.ING, "YQPAY");

        if(payingOrders.size() ==0){
            return;
        }
        List<String> orderNoList = payingOrders.stream().map(StudentPaymentOrder::getOrderNo).collect(Collectors.toList());
        String merOrderNos = payingOrders.stream().map(StudentPaymentOrder::getOrderNo).collect(Collectors.joining(","));

        String notifyUrl = ""; //回调地址
        Map<String, Object> resultMap = new LinkedHashMap<>();
        resultMap.put("merOrderNoList", merOrderNos);
        Map<String, Object> requestMap = YqPayUtil.getRequestMap(notifyUrl, resultMap);

        RsqMsg rsqMsg = new RsqMsg(requestMap);

        Msg queryRs = yqPayFeignService.orderQuery(rsqMsg);

        if (queryRs.getCode().equals("88")) {
            //更新订单状态
            String[] statusArr = {"0", "1", "7"};
            String responseParameters = queryRs.getResponseParameters();
            List<Map<String, String>> responseList = JSON.parseObject(responseParameters, List.class);
            for (Map<String, String> response : responseList) {
                Map<String, String> rpMap = response;
                String channelType = rpMap.get("channelType").equals("1") ? "WXPay" : (rpMap.get("channelType").equals("2") ? "Alipay" : "quickPay");
                rpMap.put("channelType", channelType);

                if (Arrays.asList(statusArr).contains(rpMap.get("tradeState"))) {
                    updateOrder(rpMap); //更新订单
                }
                if (orderNoList.contains(rpMap.get("merOrderNo"))) {
                    orderNoList.remove(rpMap.get("merOrderNo"));
                }
            }
            closeOrders(orderNoList); //关闭订单
        }
    }

    private void adaPayQuery() throws Exception {
        List<StudentPaymentOrder> payingOrders = findOrdersByStatus(DealStatusEnum.ING, "ADAPAY");
        if(payingOrders.size()==0){
            return;
        }

        List<String> orderNoList = new ArrayList<String>();

        for (StudentPaymentOrder payingOrder : payingOrders) {
            if (payingOrder.getTransNo() == null) {
                orderNoList.add(payingOrder.getOrderNo());
                continue;
            }
            Payment payment = new Pay().queryPayment(payingOrder.getTransNo());
            Map<String, String> rpMap = new HashMap<>();
            rpMap.put("merOrderNo", payingOrder.getOrderNo());
            rpMap.put("remarks", payment.getReason());
            rpMap.put("orderNo", payment.getId());
            rpMap.put("channelType", payment.getPayChannel());
            if (payment.getStatus().equals("succeeded")) {
                rpMap.put("tradeState", "1");
            }
            if (payment.getStatus().equals("failed")) {
                rpMap.put("tradeState", "0");
            }
            if (payment.getStatus().equals("pending")) {
                orderNoList.add(payingOrder.getOrderNo());
            }
        }
        closeOrders(orderNoList);
    }

    private void updateOrder(Map<String, String> rpMap) throws Exception {
        DealStatusEnum status = rpMap.get("tradeState").equals("1") ? DealStatusEnum.SUCCESS : DealStatusEnum.FAILED;
        StudentPaymentOrder order = findOrderByOrderNo(rpMap.get("merOrderNo"));
        if (order == null || !order.getStatus().equals(DealStatusEnum.ING)) {
            return;
        }

        if (status.equals(DealStatusEnum.SUCCESS)) {
            order.setPayTime(new Date());
        } else {
            order.setMemo(rpMap.get("remarks"));
        }
        order.setStatus(status);
        order.setTransNo(rpMap.get("orderNo"));
        order.setPaymentBusinessChannel(rpMap.get("channelType"));

        if (order.getType().equals(OrderTypeEnum.APPLY)) { //报名订单
            studentRegistrationService.updateApplyOrder(order);
        } else if (order.getType().equals(OrderTypeEnum.SMALL_CLASS_TO_BUY)) {
            vipGroupService.orderCallback(order);
        } else if (order.getType().equals(OrderTypeEnum.RENEW)) {
            musicGroupService.renewForCallback(order);
        }
    }

    private void closeOrders(List<String> orderNoList) throws Exception {
        if (orderNoList.size() == 0) {
            return;
        }

        Calendar beforeTime = Calendar.getInstance();
        beforeTime.add(Calendar.MINUTE, -30);// 30分钟之前的时间
        Date beforeDate = beforeTime.getTime();

        List<StudentPaymentOrder> ordersOverTime = findOrdersOverTime(orderNoList, DealStatusEnum.ING, beforeDate);
        for (StudentPaymentOrder order : ordersOverTime) {
            order.setStatus(DealStatusEnum.FAILED);
            order.setMemo("超时未支付关闭");
            if (order.getType().equals(OrderTypeEnum.APPLY)) { //报名订单
                studentRegistrationService.updateApplyOrder(order);
            } else if (order.getType().equals(OrderTypeEnum.SMALL_CLASS_TO_BUY)) {
                vipGroupService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.RENEW)) {
                musicGroupService.renewForCallback(order);
            }
        }

    }
}