package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.entity.SysUserRole;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.api.enums.UserLockFlag;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.StudentApplyDetailDto;
import com.ym.mec.biz.dal.dto.StudentFeeDetailDto;
import com.ym.mec.biz.dal.dto.StudentInfo;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.StudentRegistrationQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImResult;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.http.HttpUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class StudentRegistrationServiceImpl extends BaseServiceImpl<Long, StudentRegistration> implements StudentRegistrationService {

    @Resource
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;
    @Autowired
    private MusicGroupStudentFeeDao musicGroupStudentFeeDao;
    @Autowired
    private SysMessageConfigDao sysMessageConfigDao;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
    @Autowired
    private MusicGroupSubjectPlanDao musicGroupSubjectPlanDao;
    @Autowired
    private SubjectDao subjectDao;
    @Autowired
    private SysUserCashAccountDao sysUserCashAccountDao;
    @Autowired
    private ContractService contractService;
    @Autowired
    private ImFeignService imFeignService;
    @Autowired
    private MusicGroupSubjectPlanService musicGroupSubjectPlanService;
    @Autowired
    private SubjectService subjectService;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private SysUserCashAccountDetailService sysUserCashAccountDetailService;
    @Autowired
    private SysUserCashAccountService sysUserCashAccountService;

    @Override
    public BaseDAO<Long, StudentRegistration> getDAO() {
        return studentRegistrationDao;
    }

    @Override
    public PageInfo<StudentApplyDetailDto> queryStudentDetailPage(StudentRegistrationQueryInfo queryInfo) {
        PageInfo<StudentApplyDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentApplyDetailDto> dataList = null;
        int count = studentRegistrationDao.queryStudentDetailCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentRegistrationDao.queryStudentDetailPage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchUpdateSubject(Integer userId, Integer subId, String musicGroupId) throws Exception {
        //获取当前学员的当前声部
        StudentRegistration studentRegistration = studentRegistrationDao.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if(studentRegistration == null){
            throw new Exception("学员信息不存在");
        }
        //当前专业报名人数减一
        musicGroupSubjectPlanService.addApplyStudentNum(musicGroupId,studentRegistration.getActualSubjectId(),-1);
        //批量调剂（未缴费学员）
        int i = studentRegistrationDao.batchUpdateSubject(userId, subId, musicGroupId);
        //修改专业已报名人数
        musicGroupSubjectPlanService.addApplyStudentNum(musicGroupId,subId,1);
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        Set<Integer> roleIds = new HashSet<>(2);
        roleIds.add(SysUserRole.SECTION_MANAGER);
        roleIds.add(SysUserRole.OPERATION_EXECUTIVE);
        sysMessageService.batchSeoMessage(musicGroupDao.queryUserIdByRoleId(roleIds),MessageTypeEnum.BACKSTAGE_STUDENT_SUBJECT_CHANGE,"",musicGroup.getName());
        return studentRegistrationDao.batchUpdateSubject(userId, subId, musicGroupId);
    }

    @Override
    public StudentFeeDetailDto queryFeeDetail(Integer studentId, String musicGroupId) {
        return studentRegistrationDao.queryFeeDetail(studentId, musicGroupId);
    }

    @Override
    public List<Map<String, Object>> getNoClassStuBySubjectId(String musicGroupId, String actualSubjectId) {
        List<StudentRegistration> students = studentRegistrationDao.getNoClassStuBySubjectId(musicGroupId, actualSubjectId);
        List<Map<String, Object>> mapArrayList = new ArrayList<>();
        if (students != null && students.size() > 0) {
            String[] subjectIds = actualSubjectId.split(",");
            if (subjectIds != null && subjectIds.length > 0) {
                List<Map<Integer, String>> subjectNames = subjectDao.queryNameByIds(actualSubjectId);
                Map<Integer, String> subjectNameMap = MapUtil.convertMybatisMap(subjectNames);
                for (int i = 0; i < subjectIds.length; i++) {
                    Map<String, Object> resultMap = new HashMap<>(3);
                    int subjectId = Integer.parseInt(subjectIds[i]);
                    List<StudentRegistration> collect = students.stream().filter(e -> e.getActualSubjectId().equals(subjectId)).collect(Collectors.toList());
                    resultMap.put("subjectId", subjectId);
                    resultMap.put("subjectName", subjectNameMap.get(subjectId));
                    resultMap.put("rows", collect);
                    mapArrayList.add(resultMap);
                }
            }
        }
        return mapArrayList;
    }

    @Override
    public List<MusicGroupSubjectPlan> getNoClassStuCountByMusicGroupId(String musicGroupId) {
        return studentRegistrationDao.getNoClassStuCountByMusicGroupId(musicGroupId);
    }

    @Override
    public Integer updateByUserIdAndMusicGroupId(StudentRegistration studentRegistration) {
        return studentRegistrationDao.updateByUserIdAndMusicGroupId(studentRegistration);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentRegistration addStudent(StudentRegistration studentRegistration) throws IOException {
        Date date = new Date();
        Integer userId = 0;
        SysUser sysUser = studentRegistrationDao.getSysUserByPhone(studentRegistration.getParentsPhone());
        if (sysUser == null) {
            sysUser = new SysUser();
            sysUser.setPhone(studentRegistration.getParentsPhone());
            sysUser.setCreateTime(date);
            sysUser.setUpdateTime(date);
            sysUser.setLockFlag(UserLockFlag.NORMAL);
            sysUser.setDelFlag(com.ym.mec.auth.api.enums.YesOrNoEnum.NO);
            sysUser.setUserType(SysUserType.STUDENT);
            sysUser.setGender(studentRegistration.getGender());
            sysUser.setOrganId(studentRegistration.getOrganId());
            sysUser.setRealName(studentRegistration.getParentsName());
            sysUser.setUsername(studentRegistration.getName());
            sysUser.setIdCardNo(studentRegistration.getIdCardNo());
            teacherDao.addSysUser(sysUser);
            userId = sysUser.getId();
            //添加用户现金账户
            sysUserCashAccountDao.insert(new SysUserCashAccount(userId, "CNY"));
			// 添加用户电子签章账户
			//contractService.register(userId, sysUser.getRealName(), sysUser.getIdCardNo(), sysUser.getPhone());
            //注册到融云
            if(StringUtils.isEmpty(sysUser.getAvatar())){
                sysUser.setAvatar(sysConfigDao.findConfigValue(SysConfigService.USER_DEFAULT_HEAD_URL));
            }
            ImResult register = imFeignService.register(new ImUserModel(userId.toString(), sysUser.getUsername(), sysUser.getAvatar()));
            sysUser.setImToken(register.getToken());
            teacherDao.updateUser(sysUser);
        }
        studentRegistration.setActualSubjectId(studentRegistration.getSubjectId());
        studentRegistration.setCreateTime(date);
        studentRegistration.setUpdateTime(date);
        studentRegistration.setUserId(sysUser.getId());
        studentRegistrationDao.insert(studentRegistration);
        //增加报名学生数
        musicGroupSubjectPlanService.addApplyStudentNum(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId(),1);
        //报名成功后，发送短信
        String studentApplyUrl = sysConfigDao.findConfigValue(SysConfigService.STUDENT_APPLY_URL) + studentRegistration.getMusicGroupId();
        String serverPhone = sysConfigDao.findConfigValue(SysConfigService.SERVER_PHONE);
        Subject subject = subjectDao.get(studentRegistration.getActualSubjectId());
        MusicGroup musicGroup = musicGroupDao.get(studentRegistration.getMusicGroupId());

        Map<Integer,String> map = new HashMap<>(1);
        map.put(studentRegistration.getUserId(),studentRegistration.getParentsPhone());
        sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,
                MessageTypeEnum.SMS_APPLY_MESSAGE,map,null,0,"",
                studentRegistration.getParentsName(),subject.getName(), HttpUtil.getSortUrl(studentApplyUrl),
                DateUtil.format(musicGroup.getApplyExpireDate(),DateUtil.DATE_FORMAT_MIN),serverPhone);
        return studentRegistration;
    }

    @Override
    public StudentInfo queryStudentInfo(Integer userId) {
        return studentRegistrationDao.queryStudentInfo(userId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StudentPaymentOrder addOrder(StudentRegistration studentRegistration, BigDecimal amount, String orderNo, String paymentChannel, BigDecimal courseFee, List<MusicGroupSubjectGoodsGroup> goodsGroups, List<Goods> goodsList, List<Goods> otherGoodsList) throws Exception {
        Date date = new Date();
        StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
        studentPaymentOrder.setUserId(studentRegistration.getUserId());
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.APPLY);
        studentPaymentOrder.setExpectAmount(amount);
        studentPaymentOrder.setActualAmount(amount);
        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        studentPaymentOrder.setPaymentChannel(paymentChannel);
        studentPaymentOrder.setMusicGroupId(studentRegistration.getMusicGroupId());
        studentPaymentOrderService.insert(studentPaymentOrder);

        ArrayList<StudentPaymentOrderDetail> studentPaymentOrderDetailList = new ArrayList<>();
        StudentPaymentOrderDetail studentPaymentOrderDetail = new StudentPaymentOrderDetail();
        studentPaymentOrderDetail.setType(OrderDetailTypeEnum.COURSE);
        studentPaymentOrderDetail.setPrice(courseFee);
        studentPaymentOrderDetail.setCreateTime(date);
        studentPaymentOrderDetail.setUpdateTime(date);
        studentPaymentOrderDetail.setPaymentOrderId(studentPaymentOrder.getId());
        studentPaymentOrderDetailList.add(studentPaymentOrderDetail);
        //乐器及打包辅件
        if (goodsGroups != null) {
            for (MusicGroupSubjectGoodsGroup goodsGroup : goodsGroups) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goodsGroup = new StudentPaymentOrderDetail();
                OrderDetailTypeEnum type = null;
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT)) {
                    type = OrderDetailTypeEnum.MUSICAL;
                } else if (goodsGroup.getType().equals(GoodsType.ACCESSORIES)) {
                    type = OrderDetailTypeEnum.ACCESSORIES;
                } else if (goodsGroup.getType().equals(GoodsType.OTHER)) {
                    type = OrderDetailTypeEnum.TEACHING;
                }
                studentPaymentOrderDetail4goodsGroup.setType(type);
                studentPaymentOrderDetail4goodsGroup.setPrice(goodsGroup.getPrice());
                studentPaymentOrderDetail4goodsGroup.setGoodsIdList(goodsGroup.getGoodsIdList());
                studentPaymentOrderDetail4goodsGroup.setCreateTime(date);
                studentPaymentOrderDetail4goodsGroup.setUpdateTime(date);
                studentPaymentOrderDetail4goodsGroup.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goodsGroup);
            }
        }

        //单独辅件
        if (goodsList != null) {
            for (Goods goods : goodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goods = new StudentPaymentOrderDetail();
                studentPaymentOrderDetail4goods.setType(OrderDetailTypeEnum.ACCESSORIES);
                studentPaymentOrderDetail4goods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4goods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4goods.setCreateTime(date);
                studentPaymentOrderDetail4goods.setUpdateTime(date);
                studentPaymentOrderDetail4goods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goods);

            }
        }
        //单独教谱
        if (otherGoodsList != null) {
            for (Goods goods : otherGoodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4otherGoods = new StudentPaymentOrderDetail();
                studentPaymentOrderDetail4otherGoods.setType(OrderDetailTypeEnum.TEACHING);
                studentPaymentOrderDetail4otherGoods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4otherGoods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4otherGoods.setCreateTime(date);
                studentPaymentOrderDetail4otherGoods.setUpdateTime(date);
                studentPaymentOrderDetail4otherGoods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4otherGoods);
            }
        }

        studentPaymentOrderDetailService.batchAdd(studentPaymentOrderDetailList);

        //增加缴费学生数
        MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId());
        int paidNum = musicOneSubjectClassPlan.getPaidStudentNum() == null ? 0 : musicOneSubjectClassPlan.getPaidStudentNum();
        musicOneSubjectClassPlan.setPaidStudentNum(paidNum + 1);
        if(musicOneSubjectClassPlan.getPaidStudentNum() > musicOneSubjectClassPlan.getExpectedStudentNum()){
            throw new Exception("乐团人数已满");
        }
        musicGroupSubjectPlanService.update(musicOneSubjectClassPlan);
        return studentPaymentOrder;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public StudentPaymentOrder reAddOrder(Integer userId, BigDecimal amount, String orderNo, String paymentChannel, BigDecimal courseFee, List<MusicGroupSubjectGoodsGroup> goodsGroups, List<Goods> goodsList, List<Goods> otherGoodsList, String musicGroupId, StudentPaymentOrder oldOrder) {
        //关闭老订单
        oldOrder.setStatus(DealStatusEnum.CLOSE);
        studentPaymentOrderService.update(oldOrder);
        Date date = new Date();
        StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
        studentPaymentOrder.setUserId(userId);
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.APPLY);
        studentPaymentOrder.setExpectAmount(amount);
        studentPaymentOrder.setActualAmount(amount);
        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        studentPaymentOrder.setPaymentChannel(paymentChannel);
        studentPaymentOrder.setMusicGroupId(musicGroupId);
        studentPaymentOrderService.insert(studentPaymentOrder);

        ArrayList<StudentPaymentOrderDetail> studentPaymentOrderDetailList = new ArrayList<>();
        StudentPaymentOrderDetail studentPaymentOrderDetail = new StudentPaymentOrderDetail();
        studentPaymentOrderDetail.setType(OrderDetailTypeEnum.COURSE);
        studentPaymentOrderDetail.setPrice(courseFee);
        studentPaymentOrderDetail.setCreateTime(date);
        studentPaymentOrderDetail.setUpdateTime(date);
        studentPaymentOrderDetail.setPaymentOrderId(studentPaymentOrder.getId());
        studentPaymentOrderDetailList.add(studentPaymentOrderDetail);
        //乐器及打包辅件
        if (goodsGroups != null) {
            for (MusicGroupSubjectGoodsGroup goodsGroup : goodsGroups) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goodsGroup = new StudentPaymentOrderDetail();
                OrderDetailTypeEnum type = null;
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT)) {
                    type = OrderDetailTypeEnum.MUSICAL;
                } else if (goodsGroup.getType().equals(GoodsType.ACCESSORIES)) {
                    type = OrderDetailTypeEnum.ACCESSORIES;
                } else if (goodsGroup.getType().equals(GoodsType.OTHER)) {
                    type = OrderDetailTypeEnum.TEACHING;
                }
                studentPaymentOrderDetail4goodsGroup.setType(type);
                studentPaymentOrderDetail4goodsGroup.setPrice(goodsGroup.getPrice());
                studentPaymentOrderDetail4goodsGroup.setGoodsIdList(goodsGroup.getGoodsIdList());
                studentPaymentOrderDetail4goodsGroup.setCreateTime(date);
                studentPaymentOrderDetail4goodsGroup.setUpdateTime(date);
                studentPaymentOrderDetail4goodsGroup.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goodsGroup);
            }
        }

        //单独辅件
        if (goodsList != null) {
            for (Goods goods : goodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4goods = new StudentPaymentOrderDetail();
                studentPaymentOrderDetail4goods.setType(OrderDetailTypeEnum.ACCESSORIES);
                studentPaymentOrderDetail4goods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4goods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4goods.setCreateTime(date);
                studentPaymentOrderDetail4goods.setUpdateTime(date);
                studentPaymentOrderDetail4goods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4goods);

            }
        }
        //单独教谱
        if (otherGoodsList != null) {
            for (Goods goods : otherGoodsList) {
                StudentPaymentOrderDetail studentPaymentOrderDetail4otherGoods = new StudentPaymentOrderDetail();
                studentPaymentOrderDetail4otherGoods.setType(OrderDetailTypeEnum.TEACHING);
                studentPaymentOrderDetail4otherGoods.setPrice(goods.getGroupPurchasePrice());
                studentPaymentOrderDetail4otherGoods.setGoodsIdList(goods.getId().toString());
                studentPaymentOrderDetail4otherGoods.setCreateTime(date);
                studentPaymentOrderDetail4otherGoods.setUpdateTime(date);
                studentPaymentOrderDetail4otherGoods.setPaymentOrderId(studentPaymentOrder.getId());
                studentPaymentOrderDetailList.add(studentPaymentOrderDetail4otherGoods);
            }
        }

        studentPaymentOrderDetailService.batchAdd(studentPaymentOrderDetailList);

        return studentPaymentOrder;
    }

    @Override
    public StudentRegistration queryByUserIdAndMusicGroupId(Integer userId, String musicGroupId) {
        return studentRegistrationDao.queryByUserIdAndMusicGroupId(userId, musicGroupId);
    }

    @Override
    public StudentRegistration getByPhoneAndMusicGroupId(String musicGroupId, String parentsPhone) {
        return studentRegistrationDao.getByPhoneAndMusicGroupId(musicGroupId, parentsPhone);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertStudent(StudentRegistration studentRegistration) throws Exception {
        StudentRegistration phoneAndMusicGroupId = studentRegistrationDao.getByPhoneAndMusicGroupId(studentRegistration.getMusicGroupId(), studentRegistration.getParentsPhone());
        if (phoneAndMusicGroupId != null) {
            throw new Exception("该学员已存在");
        } else {
            SysUser sysUser = studentRegistrationDao.getSysUserByPhone(studentRegistration.getParentsPhone());
            Integer userId;
            if (sysUser == null) {
                //新增user
                sysUser = new SysUser();
                sysUser.setRealName(studentRegistration.getName());
                sysUser.setUsername(studentRegistration.getName());
                sysUser.setGender(studentRegistration.getGender());
                sysUser.setUserType(SysUserType.STUDENT);
                sysUser.setIdCardNo(studentRegistration.getIdCardNo());
                teacherDao.addSysUser(sysUser);
                userId = sysUser.getId();
                //添加用户现金账户
                sysUserCashAccountDao.insert(new SysUserCashAccount(userId, "CNY"));
                //添加用户电子签章账户
                contractService.register(userId, sysUser.getRealName(), sysUser.getIdCardNo(), sysUser.getPhone());
                //注册到融云
                if(StringUtils.isEmpty(sysUser.getAvatar())){
                    sysUser.setAvatar(sysConfigDao.findConfigValue(SysConfigService.USER_DEFAULT_HEAD_URL));
                }
                ImResult register = imFeignService.register(new ImUserModel(userId.toString(), sysUser.getUsername(), sysUser.getAvatar()));
                sysUser.setImToken(register.getToken());
                teacherDao.updateUser(sysUser);
            } else {
                userId = sysUser.getId();
            }
            studentRegistration.setActualSubjectId(studentRegistration.getSubjectId());
            studentRegistration.setUserId(userId);
            studentRegistration.setPaymentStatus(PaymentStatusEnum.YES);
            studentRegistration.setMusicGroupStatus(ClassGroupStudentStatusEnum.NORMAL);
            //学生报名表
            studentRegistrationDao.insertBasic(studentRegistration);
            MusicGroup musicGroup = musicGroupDao.get(studentRegistration.getMusicGroupId());
            Set<Integer> roleIds = new HashSet<>(1);
            roleIds.add(SysUserRole.OPERATION_EXECUTIVE);
            sysMessageService.batchSeoMessage(musicGroupDao.queryUserIdByRoleId(roleIds),MessageTypeEnum.BACKSTAGE_MUSIC_GROUP_ADD_STUDENT,"",musicGroup.getName(),studentRegistration.getName());
            //增加报名学生数
//            musicGroupSubjectPlanService.addApplyStudentNum(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId(),1);
            if (studentRegistration.getClassGroupId() != null) {
                ClassGroup classGroup = classGroupDao.get(studentRegistration.getClassGroupId());
                if (classGroup != null) {
                    if (!classGroup.getMusicGroupId().equals(studentRegistration.getMusicGroupId())) {
                        throw new Exception("班级录入错误");
                    } else {
                        //修改实际学生人数
                        classGroup.setStudentNum(classGroup.getStudentNum() + 1);
                        classGroup.setUpdateTime(new Date());
                        classGroupDao.update(classGroup);
                    }
                } else {
                    throw new Exception("班级不存在");
                }
                //新增班级学生关系
                classGroupStudentMapperDao.insert(new ClassGroupStudentMapper(studentRegistration.getClassGroupId(), userId));
                MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanDao.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getActualSubjectId());
                //获取当前月
                Integer month = Integer.parseInt(DateUtil.getMonth(new Date()));
                //获取下次缴费月份
                List<MusicGroupPaymentCalender> musicGroupPaymentCalenders = musicGroupPaymentCalenderDao.findByMusicGroupId(studentRegistration.getMusicGroupId());
                List<Integer> months = musicGroupPaymentCalenders.stream().map(e -> e.getPaymentMonth()).collect(Collectors.toList());
                for (int i = 0; i < months.size(); i++) {
                    if (months.get(i).equals(month)) {
                        month = i == months.size() - 1 ? months.get(0) : months.get(i + 1);
                    }
                }
                Calendar calendar = Calendar.getInstance();
                calendar.add(Calendar.MONTH, month);
                DateUtil.getFirstDayOfMonth(calendar.getTime());
                //保存乐团学生费用表
                musicGroupStudentFeeDao.insert(new MusicGroupStudentFee(studentRegistration.getMusicGroupId(),
                        userId, studentRegistration.getSubjectId(), musicOneSubjectClassPlan.getFee(),
                        DateUtil.getFirstDayOfMonth(calendar.getTime()), 0, studentRegistration.getTemporaryCourseFee()));
            }
            return userId;
        }
    }

    @Override
    public List<StudentRegistration> findClassGroupStu(String musicGroupId, Integer classGroupId) {
        //获取所有声部
        List<Subject> subjectList = subjectService.findAll(new HashMap<>());
        List<StudentRegistration> classGroupStuList = studentRegistrationDao.findClassGroupStu(musicGroupId, classGroupId);
        for (StudentRegistration studentRegistration : classGroupStuList) {
            for (Subject subject : subjectList) {
                if (studentRegistration.getActualSubjectId().equals(subject.getId())) {
                    studentRegistration.setSubjectName(subject.getName());
                    break;
                }
            }
        }
        return classGroupStuList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentPaymentOrder updateApplyOrder(StudentPaymentOrder studentPaymentOrder) {
        //更新订单状态
        studentPaymentOrderService.update(studentPaymentOrder);
        Date nowDate = new Date();

        //成功报名状态变更
        StudentRegistration studentRegistration = studentRegistrationDao.queryByUserIdAndMusicGroupId(studentPaymentOrder.getUserId(), studentPaymentOrder.getMusicGroupId());
        MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId());

        if (studentPaymentOrder.getStatus().equals(DealStatusEnum.SUCCESS)) {
            studentRegistration.setPaymentStatus(PaymentStatusEnum.YES);
            studentRegistration.setMusicGroupStatus(ClassGroupStudentStatusEnum.NORMAL);
            studentRegistrationDao.update(studentRegistration);

            //获取当前月
            Integer month = Integer.parseInt(DateUtil.getMonth(nowDate));
            //获取下次缴费月份
            List<MusicGroupPaymentCalender> musicGroupPaymentCalenders = musicGroupPaymentCalenderDao.findByMusicGroupId(studentRegistration.getMusicGroupId());
            List<Integer> months = musicGroupPaymentCalenders.stream().map(e -> e.getPaymentMonth()).collect(Collectors.toList());
            for (int i = 0; i < months.size(); i++) {
                if (months.get(i).equals(month)) {
                    month = i == months.size() - 1 ? months.get(0) : months.get(i + 1);
                }
            }
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.MONTH, month);

            //加入缴费周期
            MusicGroupStudentFee musicGroupStudentFee = new MusicGroupStudentFee();
            musicGroupStudentFee.setMusicGroupId(studentRegistration.getMusicGroupId());
            musicGroupStudentFee.setUserId(studentRegistration.getUserId());
            musicGroupStudentFee.setCourseFee(musicOneSubjectClassPlan.getFee());
            musicGroupStudentFee.setCreateTime(nowDate);
            musicGroupStudentFee.setUpdateTime(nowDate);
            musicGroupStudentFee.setNextPaymentDate(DateUtil.getFirstDayOfMonth(calendar.getTime()));
            musicGroupStudentFee.setLatestPaidTime(nowDate);
            musicGroupStudentFee.setSubjectId(studentRegistration.getActualSubjectId());
            musicGroupStudentFee.setPaymentStatus(MusicGroupStudentFee.PaymentStatus.PAID_COMPLETED);
            musicGroupStudentFeeDao.insert(musicGroupStudentFee);

            //插入交易明细
            SysUserCashAccount cashAccount = sysUserCashAccountService.get(studentRegistration.getUserId());
            BigDecimal amount = studentPaymentOrder.getActualAmount();
            //充值
            SysUserCashAccountDetail rechargeDetail = new SysUserCashAccountDetail();
            rechargeDetail.setAmount(amount);
            rechargeDetail.setBalance(cashAccount.getBalance().add(amount));
            rechargeDetail.setComment("缴费前充值");
            rechargeDetail.setCreateTime(nowDate);
            rechargeDetail.setStatus(DealStatusEnum.SUCCESS);
            rechargeDetail.setTransNo(studentPaymentOrder.getTransNo());
            rechargeDetail.setType(PlatformCashAccountDetailTypeEnum.RECHARGE);
            rechargeDetail.setUpdateTime(nowDate);
            rechargeDetail.setUserId(studentRegistration.getUserId());
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            sysUserCashAccountDetailService.insert(rechargeDetail);

            //缴费
            SysUserCashAccountDetail paymentDetail = new SysUserCashAccountDetail();
            paymentDetail.setAmount(amount);
            paymentDetail.setBalance(cashAccount.getBalance());
            paymentDetail.setComment("报名缴费");
            paymentDetail.setCreateTime(nowDate);
            paymentDetail.setStatus(DealStatusEnum.SUCCESS);
            paymentDetail.setTransNo(studentPaymentOrder.getTransNo());
            paymentDetail.setType(PlatformCashAccountDetailTypeEnum.PAY_FEE);
            paymentDetail.setUpdateTime(nowDate);
            paymentDetail.setUserId(studentRegistration.getUserId());
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            sysUserCashAccountDetailService.insert(paymentDetail);

            //发送缴费成功通知（短信 + push）
            Map<Integer,String> map = new HashMap<>(1);
            map.put(studentPaymentOrder.getUserId(),studentRegistration.getParentsPhone());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,
                    MessageTypeEnum.STUDENT_SMS_PAYMENT_SUCCESS,map,null,0,"1",
                    studentRegistration.getParentsName(),studentPaymentOrder.getActualAmount());
            //push
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,
                    MessageTypeEnum.STUDENT_SMS_PAYMENT_SUCCESS,map,null,0,"1",
                    studentRegistration.getParentsName(),studentPaymentOrder.getActualAmount());
        }

        //失败减去已收款金额

        //减去缴费人数
        if (studentPaymentOrder.getStatus().equals(DealStatusEnum.FAILED)) {
            musicOneSubjectClassPlan.setPaidStudentNum(musicOneSubjectClassPlan.getPaidStudentNum() - 1);
            musicGroupSubjectPlanService.update(musicOneSubjectClassPlan);
            //发送缴费成功通知（短信 + push）
            String studentApplyUrl = sysConfigDao.findConfigValue(SysConfigService.STUDENT_APPLY_URL + studentRegistration.getMusicGroupId());

            Map<Integer,String> map = new HashMap<>(1);
            map.put(studentPaymentOrder.getUserId(),studentRegistration.getParentsPhone());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.YIMEI,
                    MessageTypeEnum.STUDENT_SMS_PAYMENT_FAILED,map,null,0,"1",
                    studentApplyUrl);
//            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG,
//                    MessageTypeEnum.STUDENT_SMS_PAYMENT_FAILED,map,null,0,"1",
//                    studentApplyUrl);
        }

        return studentPaymentOrder;
    }

    @Override
    public List<StudentRegistration> findMusicGroupNoClassGroupStudent(String musicGroupId, Integer actualSubjectId) {
        //获取所有声部
        List<Subject> subjectList = subjectService.findAll(new HashMap<>());
        List<StudentRegistration> classGroupStuList = studentRegistrationDao.findMusicGroupNoClassGroupStudent(musicGroupId, actualSubjectId);
        for (StudentRegistration studentRegistration : classGroupStuList) {
            for (Subject subject : subjectList) {
                if (studentRegistration.getActualSubjectId().equals(subject.getId())) {
                    studentRegistration.setSubjectName(subject.getName());
                    break;
                }
            }
        }
        return classGroupStuList;
    }

    @Override
    public List<StudentRegistration> findStudentListByIdList(List<Long> idList) {
        return studentRegistrationDao.findStudentListByIdList(idList);
    }

    @Override
    public int batchInsert(List<StudentRegistration> studentRegistrationList) {
        return studentRegistrationDao.batchInsert(studentRegistrationList);
    }

    @Override
    public StudentRegistration findStudentByClassGroupIdAndUserId(Integer userId, Integer classGroupId) {
        return studentRegistrationDao.findStudentByClassGroupIdAndUserId(userId, classGroupId);
    }

    @Override
    public List<StudentRegistration> findMusicGroupStudent(String musicGroupId, Integer actualSubjectId) {
        List<Subject> subjectList = subjectService.findAll(new HashMap<>());
        List<StudentRegistration> musicGroupStudents = studentRegistrationDao.findMusicGroupStudent(musicGroupId, actualSubjectId);
        for (StudentRegistration musicGroupStudent : musicGroupStudents) {
            for (Subject subject : subjectList) {
                if (musicGroupStudent.getActualSubjectId().equals(subject.getId())) {
                    musicGroupStudent.setSubjectName(subject.getName());
                }
            }
        }
        return musicGroupStudents;
    }

    @Override
    public List<StudentRegistration> findStudentListByUserIdList(String musicGroupId, List<Integer> userIdList) {
        return studentRegistrationDao.findStudentListByUserIdList(musicGroupId, userIdList);
    }

    @Override
    public int openPayment(String ids) {
        return studentRegistrationDao.batchOpenPay(ids, PaymentStatusEnum.OPEN);
    }

    @Override
    public List<StudentRegistration> queryStudentByMusicGroupId(String musicGroupId) {
        return studentRegistrationDao.queryStudentByMusicGroupId(musicGroupId);
    }

    @Override
    public List<Map<Integer, String>> findParentId(String parentPhones) {
        return studentRegistrationDao.findParentId(parentPhones);
    }
}