package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.TeacherCloseQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Service
public class TeacherAttendanceServiceImpl extends BaseServiceImpl<Long, TeacherAttendance>  implements TeacherAttendanceService {
	
	@Autowired
	private TeacherAttendanceDao teacherAttendanceDao;
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private CourseHomeworkService courseHomeworkService;
	@Autowired
	private StudentAttendanceDao studentAttendanceDao;
	@Autowired
	private StudentCourseHomeworkDao studentCourseHomeworkDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private CourseScheduleService courseScheduleService;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private SysConfigService sysConfigService;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private ClassGroupDao classGroupDao;
	@Autowired
	private SubjectDao subjectDao;

	@Override
	public BaseDAO<Long, TeacherAttendance> getDAO() {
		return teacherAttendanceDao;
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public Map<String, Object> addTeacherAttendanceRecord(TeacherSignOutDto teacherSignOutDto) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(Objects.isNull(user)){
			throw new BizException("请登录");
		}

		if(Objects.isNull(teacherSignOutDto.getTeacherAttendanceInfo().getCourseScheduleId())){
			throw new BizException("请选择课程!");
		}

		SysConfig byParamName = sysConfigService.findByParamName(SysConfigService.ADVANCE_SIGN_IN_MINUTES);
		Integer advanceSignInMinutes=Integer.parseInt(byParamName.getParanValue());

		Map<String,Object> result=new HashMap<>();

		TeacherAttendance teacherAttendance=teacherAttendanceDao.findByTeacherAttendanceInfo(user.getId().longValue(),teacherSignOutDto.getTeacherAttendanceInfo().getCourseScheduleId());

		boolean isAttendance=false;

		if(Objects.isNull(teacherAttendance)){
			teacherAttendance=teacherSignOutDto.getTeacherAttendanceInfo();
			teacherAttendance.setTeacherId(user.getId());
		}else{
			isAttendance=true;
		}

		TeacherAttendanceDto currentCourseDetail = courseScheduleDao.getCurrentCourseDetail(teacherSignOutDto.getTeacherAttendanceInfo().getCourseScheduleId());
		if(Objects.isNull(currentCourseDetail)){
			throw new BizException("课程不存在");
		}

		teacherAttendance.setMusicGroupId(currentCourseDetail.getMusicGroupId());
		teacherAttendance.setClassGroupId(currentCourseDetail.getClassId());
		teacherAttendance.setRemark(teacherSignOutDto.getTeacherAttendanceInfo().getRemark());
		Date date = new Date();
		if(teacherSignOutDto.getTeacherAttendanceInfo().getStatus().equals(SignStatusEnum.SIGN_IN.getCode())){

			if (currentCourseDetail.getEndClassTime().before(date)){
				throw new BizException("该课程已结束，不能进行签到!");
			}

			YesOrNoEnum yesOrNoEnum = courseScheduleService.enableOnlyNormalAttendance(currentCourseDetail.getStartClassTime(),
					user.getId().longValue(),
					true,
					currentCourseDetail.getSchoolId().intValue());

			if(yesOrNoEnum==YesOrNoEnum.YES){
				teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
				result.put("signInStatus",YesOrNoEnum.YES.getCode());
			}else{
				Long timeGap=(date.getTime()-currentCourseDetail.getStartClassTime().getTime())/1000/60;
				if(timeGap<= advanceSignInMinutes){
					teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
					result.put("signInStatus",YesOrNoEnum.YES.getCode());
				}else{
					if(StringUtils.isEmpty(teacherAttendance.getRemark())){
						//当前为异常签到，请填写原因!
						result.put("status",2);
						return result;
					}
					teacherAttendance.setSignInStatus(YesOrNoEnum.NO);
					result.put("signInStatus",YesOrNoEnum.NO.getCode());
				}
			}
			teacherAttendance.setSignInTime(date);
			if(Objects.nonNull(teacherAttendance.getId())){
				teacherAttendanceDao.update(teacherAttendance);
			}else{
				teacherAttendanceDao.insert(teacherAttendance);
			}
		}else{
			if (currentCourseDetail.getEndClassTime().after(date)){
				throw new BizException("该课程还未结束，不能进行签退!");
			}
			if(!Objects.isNull(teacherAttendance.getSignOutStatus())){
				throw new BizException("该课程已签退!");
			}
			int i = DateUtil.hoursBetween(currentCourseDetail.getEndClassTime(), date);
			if(i<=1){
				teacherAttendance.setSignOutStatus(YesOrNoEnum.YES);
			}else{
				teacherAttendance.setSignOutStatus(YesOrNoEnum.NO);
			}
			teacherAttendance.setSignOutTime(date);
			teacherAttendance.setCreateTime(date);
			if(isAttendance){
				teacherAttendanceDao.update(teacherAttendance);
			}else{
				teacherAttendanceDao.insert(teacherAttendance);
			}
		}
		if(teacherSignOutDto.getTeacherAttendanceInfo().getStatus().equals(SignStatusEnum.SIGN_OUT.getCode())){
			//新增课堂作业
			CourseHomework courseHomework=teacherSignOutDto.getCourseHomeworkInfo();
			courseHomework.setCourseScheduleId(teacherAttendance.getCourseScheduleId());
			courseHomework.setMusicGroupId(currentCourseDetail.getMusicGroupId());
			courseHomework.setClassGroupId(currentCourseDetail.getClassId().intValue());
			courseHomework.setExpectNum(studentAttendanceDao.countNormalAttendanceStudentNums(teacherAttendance.getCourseScheduleId()));
			courseHomeworkService.insert(courseHomework);
			List<StudentCourseHomework> studentCourseHomeworks = studentCourseHomeworkDao
					.constructInitialStudentHomeworkRecordsWithPayment(teacherAttendance.getCourseScheduleId(),
					courseHomework.getId());
			if(CollectionUtils.isEmpty(studentCourseHomeworks)){
				throw new BizException("此课程没有学生");
			}
			studentCourseHomeworkDao.batchInsertStudentCourseHomeworkRecord(studentCourseHomeworks);
		}

		result.put("status",0);
		result.put("currentTime",date);

		return result;
	}

	@Override
	public void addTeacherAttendanceRecord(Integer courseScheduleId, Integer userId, SignStatusEnum signStatus, boolean updateCourseScheduleStatus) {
		TeacherAttendance teacherAttendance=teacherAttendanceDao.findByTeacherAttendanceInfo(userId.longValue(),courseScheduleId.longValue());
		ClassGroup classGroup = classGroupDao.findByCourseSchedule(courseScheduleId);
		CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleId.longValue());
		boolean isAttendance=false;
		Date now=new Date();

		SysConfig byParamName = sysConfigService.findByParamName(SysConfigService.ADVANCE_SIGN_IN_MINUTES);
		Integer advanceSignInMinutes=Integer.parseInt(byParamName.getParanValue());

		if(Objects.isNull(teacherAttendance)){
			teacherAttendance=new TeacherAttendance();
			teacherAttendance.setCourseScheduleId(courseScheduleId.longValue());
		}else{
			isAttendance=true;
		}
		teacherAttendance.setTeacherId(userId);
		teacherAttendance.setClassGroupId(classGroup.getId());
		teacherAttendance.setMusicGroupId(classGroup.getMusicGroupId());

		int timeGap = DateUtil.minutesBetween(now, courseSchedule.getStartClassTime());

		YesOrNoEnum yesOrNoEnum = courseScheduleService.enableOnlyNormalAttendance(courseSchedule.getStartClassTime(),
				userId.longValue(),
				false,
				null);


		switch (signStatus){
			case SIGN_IN:
				if (courseSchedule.getEndClassTime().after(now)){
					teacherAttendance.setSignInTime(now);
					if(yesOrNoEnum.equals(YesOrNoEnum.YES)||advanceSignInMinutes<=timeGap&&timeGap<=60){
						teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
					}else{
						teacherAttendance.setSignInStatus(YesOrNoEnum.NO);
					}
				}
				int currentClassTimes = teacherAttendanceDao.countClassTime(classGroup.getId().longValue());
				classGroup.setCurrentClassTimes(currentClassTimes);
				if(updateCourseScheduleStatus&&courseSchedule.getStatus().equals(CourseStatusEnum.NOT_START)){
					courseScheduleDao.updateCourscheduleStatus(courseScheduleId,CourseStatusEnum.UNDERWAY.getCode());
				}
				break;
			case SIGN_OUT:
				teacherAttendance.setSignOutTime(now);
				teacherAttendance.setSignOutStatus(YesOrNoEnum.YES);
				break;
			default:

				break;
		}
		if(isAttendance){
			if(SignStatusEnum.SIGN_OUT.equals(signStatus)){
				teacherAttendanceDao.update(teacherAttendance);
				//教师签退后，给未签到学员添加旷课记录
				//封装未签到学员签到列表
				List<StudentAttendance> studentAttendances = studentAttendanceDao.queryNoSignStudentRecord(courseScheduleId);
				if(studentAttendances != null && studentAttendances.size() > 0){
					studentAttendances.forEach(e->{
						e.setTeacherId(userId);
						e.setStatus(StudentAttendanceStatusEnum.TRUANT);
						e.setRemark("学员未到，自动补旷课");
					});
					studentAttendanceDao.addStudentAttendances(studentAttendances);
				}
			}
		}else{
			teacherAttendanceDao.insert(teacherAttendance);
			//教师到课推送
			if(courseSchedule.getTeachMode() == TeachModeEnum.ONLINE){
				//获取所有当前课程的学生列表
				Map<Integer,String> userMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertMybatisMap(courseScheduleDao.findStudentMap(courseScheduleId))),HashMap.class);
				if(userMap != null && userMap.size() > 0){
					String name = subjectDao.findNames(classGroup.getSubjectIdList());
					sysMessageService.batchSendMessage(MessageSender.JIGUANG,MessageTypeEnum.STUDENT_PUSH_ONLINE_COURSE_ACTION,userMap,
							null,0,"6?sealClassId=" + courseScheduleId + "&subjectName=" + name ,courseSchedule.getName());
				}
			}
		}
	}

	@Override
	public PageInfo getTeacherPersonalAttendances(TeacherCloseQueryInfo queryInfo) {
		PageInfo<TeacherPersonalAttendanceDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<TeacherPersonalAttendanceDto> dataList = null;
		int count = teacherAttendanceDao.getTeacherPersonalAttendancesCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherAttendanceDao.getTeacherPersonalAttendances(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public boolean pushNoSignOutMessage() {
		
		Integer minutes = Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.SIGN_OUT_MESSAGE_PUSH_MINUTE));;

		List<Mapper> list = teacherAttendanceDao.queryNoSignOutListByOverMinutes(minutes);

		for (Mapper mapper : list) {
			Map<Integer, String> receivers = new HashMap<Integer, String>();
			Integer userId = (Integer) mapper.getKey();
			receivers.put(userId, userId + "");

			sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_SIGN_OUT_MESSAGE, receivers, null, 0,"5?courseScheduleId=" + mapper.getCourseScheduleId(), mapper.getValue());
		}
		return true;
	}

	@Override
	public List<TeacherAbnormalAttendanceInfoDto> findTeacherAbnormalSignInCourseScheduleRecord(Integer organId, Date date, Integer signInStatus) {
		return teacherAttendanceDao.findTeacherAbnormalSignInCourseScheduleRecord(organId,date,signInStatus);
	}

	@Override
	public List<Date> findTeacherAbnormalSignInDates(Integer organId, Date date) {
		return teacherAttendanceDao.findTeacherAbnormalSignInDates(organId,date);
	}

	@Override
	public Map countTeacherAbnormalSignInClassTimes(Integer organId, Date date) {
		Map<String,Integer> result=new HashMap<>();
		result.put("abnormalSignInNum",teacherAttendanceDao.countTeacherAbnormalSignInClassTimes(organId,date,YesOrNoEnum.NO.getCode()));
		result.put("noSignInNum",teacherAttendanceDao.countTeacherAbnormalSignInClassTimes(organId,date,null));
		return result;
	}
}
