package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.api.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.service.TeacherService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.entity.ImResult;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class TeacherServiceImpl extends BaseServiceImpl<Integer, Teacher>  implements TeacherService {
	
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private SubjectDao subjectDao;
	@Autowired
	private ClassGroupDao classGroupDao;
	@Autowired
	private MusicGroupDao musicGroupDao;
	@Autowired
	private DemoGroupDao demoGroupDao;
	@Autowired
	private OrganizationDao organizationDao;
	@Autowired
	private SchoolDao schoolDao;
	@Autowired
	private SysUserCashAccountDao sysUserCashAccountDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private ImFeignService imFeignService;
	@Autowired
	private SysConfigDao configDao;


	@Override
	public BaseDAO<Integer, Teacher> getDAO() {
		return teacherDao;
	}

	@Override
	public Teacher findByCertificateNum(String certificateNum){
		return teacherDao.findByCertificateNum(certificateNum);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public int add(Teacher teacher) throws Exception {
		SysUser user = sysUserFeignService.queryUserByMobile(teacher.getPhone());
		if(user != null && user.getId() != null){
			Integer id = user.getId();
			Teacher teacher1 = teacherDao.get(id);
			if(teacher1 == null || teacher1.getId() == null){
				teacher.setId(id);
				teacherDao.updateUser(user);
				teacherDao.insert(teacher);
			}else {
				throw new Exception("用户已存在");
			}
			return id;
		}
		teacher.setUserType(SysUserType.TEACHER);
		teacher.setUsername(teacher.getRealName());
		teacherDao.addSysUser(teacher);
		//添加用户现金账户
		sysUserCashAccountDao.insert(new SysUserCashAccount(teacher.getId(),"CNY"));
		//添加用户电子签章账户
//		contractService.register(teacher.getId(), user.getRealName(), user.getIdCardNo(), user.getPhone());
		teacherDao.insert(teacher);
		if(StringUtils.isEmpty(teacher.getAvatar())){
			SysConfig head_url = configDao.findByParamName("user_default_head_url");
			teacher.setAvatar(head_url.getParanValue());
		}
		ImResult imResult = imFeignService.register(new ImUserModel(teacher.getId().toString(), teacher.getUsername(), teacher.getAvatar()));
		if(imResult != null){
			teacher.setImToken(imResult.getToken());
			teacherDao.updateUser(teacher);
		}
		return teacher.getId();
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateTea(Teacher teacher) {
		teacher.setUpdateTime(new Date());
		teacherDao.update(teacher);
		teacherDao.updateUser(teacher);
		if(StringUtils.isEmpty(teacher.getAvatar())){
			teacher.setAvatar("https://daya-online.oss-cn-beijing.aliyuncs.com/201910/RgHwgKc.jpg");
		}
		imFeignService.update(new ImUserModel(teacher.getId().toString(),teacher.getUsername(),teacher.getAvatar()));
	}

	@Override
	public void deleteTeacher(Long userId) {
		SysUser sysUser=new SysUser();
		sysUser.setId(userId.intValue());
		sysUser.setDelFlag(YesOrNoEnum.YES);
		teacherDao.updateUser(sysUser);
	}

	@Override
	public List<ImGroupModel> queryTeacherGroups(Integer userId,String search) {
		//获取教师所在乐团列表
		/*List<ImGroupModel> imGroupModels =  musicGroupDao.queryTeacherGroups(userId,search);

		if(imGroupModels != null && imGroupModels.size() > 0){
			//获取乐团id列表
			Set<String> musicGroupIds = imGroupModels.stream().map(e -> e.getId()).collect(Collectors.toSet());
			List<MusicGroup> musicGroups = musicGroupDao.queryListByIds(musicGroupIds);
			List<Map<String, BigDecimal>> musicGroupUserNums = musicGroupDao.countMusicGroupUserNum(StringUtils.join(musicGroupIds,","));
			Map<String, BigDecimal> musicGroupUserNumMap = MapUtil.convertMybatisMap(musicGroupUserNums);
			imGroupModels.forEach(e -> {
				Set<Integer> teachers = e.getName() == null?new HashSet<>():new HashSet(Arrays.asList(e.getName().split(",")));
				MusicGroup musicGroup = musicGroups.stream().filter(mg -> mg.getId().equals(e.getId())).findFirst().get();
				//获取教学老师id列表
//				Set<Integer> teachers = musicGroupDao.queryTeacherIds(e.getId());
				teachers.add(musicGroup.getTeamTeacherId());
				teachers.add(musicGroup.getEducationalTeacherId());
				teachers.add(musicGroup.getOperatorUserId());
				teachers.add(musicGroup.getDirectorUserId());
				teachers.removeAll(Collections.singleton(null));
				//获取乐团群用户总数
				BigDecimal count = musicGroupUserNumMap.get(e.getId());
				e.setCount(count == null?0:count.intValue() + teachers.size());
				e.setId("mg" + e.getId());
			});
		}else {
			imGroupModels = new ArrayList<>();
		}*/
		//获取教师所在班级列表
		List<ImGroupModel> imGroupModels =  classGroupDao.queryTeacherGroups(userId,search);
		if(imGroupModels != null && imGroupModels.size() > 0){
			Set<String> classGroupIds = imGroupModels.stream().map(e -> e.getId()).collect(Collectors.toSet());
			//获取班级老师总数map
			Map<Integer, Integer> numsMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(classGroupDao.countTeacherNums(StringUtils.join(classGroupIds, ",")))),HashMap.class);
			imGroupModels.forEach(e -> {
				Integer num = numsMap.get(Integer.parseInt(e.getId()));
				e.setCount((num == null ? 0 : num) + e.getCount());
			});
		}
		return imGroupModels;
	}

	@Override
	public List<BasicUserDto> queryGroupStudents(Integer teacherId, String search) {
		List<ImGroupModel> musicGroups =  musicGroupDao.queryTeacherGroups(teacherId,search);
		List<BasicUserDto> teachers = new ArrayList<>();
		if(musicGroups != null && musicGroups.size() > 0){
			Set<String> musicGroupId = musicGroups.stream().map(ImGroupModel::getId).collect(Collectors.toSet());
			List<MusicGroup> musicGroupList = musicGroupDao.queryListByIds(StringUtils.join(musicGroupId, ","));
			Set<Integer> teacherIds = new HashSet<>();
			if(musicGroupList != null && musicGroupList.size() > 0){
				musicGroupList.forEach(e->{
					teacherIds.add(e.getTeamTeacherId());
					teacherIds.add(e.getEducationalTeacherId());
					teacherIds.add(e.getOperatorUserId());
					teacherIds.add(e.getDirectorUserId());
				});
			}
			teacherIds.remove(teacherId);
			teacherIds.removeAll(Collections.singleton(null));
			if(teacherIds.size() > 0){
				teachers = teacherDao.queryMusicGroupStudents(StringUtils.join(teacherIds,","), search);
			}
		}
		List<BasicUserDto> userDtos = teacherDao.queryGroupStudents(teacherId, search);
		teachers.addAll(userDtos);
		return teachers;
	}

    @Override
    public List<Teacher> findTeachers(Integer organId) {
		return teacherDao.findTeachers(organId);
    }

	@Override
	public List<Teacher> findOrganTeacherBySubject(Integer organId, Integer subjectId) {
		return teacherDao.findOrganTeacherBySubject(organId,subjectId);
	}

	@Override
	public PageInfo<TeacherCloseDto> queryCloses(TeacherCloseQueryInfo queryInfo) {
		PageInfo<TeacherCloseDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<TeacherCloseDto> dataList = null;
		int count = teacherDao.countCloses(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherDao.queryCloses(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public BigDecimal findTeacherNoPay(Integer teacherId) {
		return teacherDao.findTeacherNoPay(teacherId);
	}

	@Override
	public PageInfo<MusicGroupTeachersDto> queryMusicGroupTeachers(MusicGroupTeachersQueryInfo queryInfo) {
		PageInfo<MusicGroupTeachersDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<MusicGroupTeachersDto> dataList = null;
		int count = teacherDao.countMusicGroupTeachers(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherDao.queryMusicGroupTeachers(params);
			//获取合奏班id列表
//			Set<Integer> classGroupIds = dataList.stream().map(e -> e.getClassGroupId()).collect(Collectors.toSet());
//			String classGroupIdStr = StringUtils.join(classGroupIds,",");
			//获取教师id列表
			Set<Integer> teacherIds = dataList.stream().map(e -> e.getTeacherId()).collect(Collectors.toSet());
//			String teacherIdStr = StringUtils.join(teacherIds,",");

			//根据教师编号和班级编号，获取老师签到次数
			Map<Integer,Integer> signNumMap =  JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(teacherDao.findSignNum(StringUtils.join(teacherIds,","),queryInfo.getMusicGroupId()))),HashMap.class);
//			Map<Integer,Integer> signNumMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertMybatisMap(signNum)),HashMap.class);

			//根据合奏班编号，获取子班级名称列表
//			List<Map<Integer,String>> classNames =  classGroupDao.queryNamesByClassId(classGroupIdStr);
//			Map<Integer,String> classNameMap = MapUtil.convertMybatisMap(classNames);

			dataList.forEach(e->{
//				e.setCourseScheduleName(classNameMap.get(e.getClassGroupId()));
				e.setNum(signNumMap.get(e.getTeacherId()));
			});
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<MusicGroupTeacherAttendanceDto> queryTeacherAttendance(MusicGroupTeacherAttendanceQueryInfo queryInfo) {
		PageInfo<MusicGroupTeacherAttendanceDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<MusicGroupTeacherAttendanceDto> dataList = null;
		int count = teacherDao.countTeacherAttendance(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherDao.queryTeacherAttendance(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<musicGroupStudentApplyDto> queryMusicGroupStudentApply(queryMusicGroupStudentQueryInfo queryInfo) {
		PageInfo<musicGroupStudentApplyDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<musicGroupStudentApplyDto> dataList = null;
		int count = teacherDao.countMusicGroupStudentApply(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherDao.queryMusicGroupStudentApply(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public List<BasicUserDto> queryMusicGroupTeacher(String musicGroupId) {
		if(Objects.isNull(musicGroupId)){
			throw new BizException("请指定乐团");
		}
		return teacherDao.findTeacherByMusicGroup(musicGroupId);
	}

	@Override
	public PageInfo<Teacher> queryPageDetail(TeacherQueryInfo queryInfo) {
		PageInfo<Teacher> pageInfo = queryPage(queryInfo);
		List<Teacher> rows = pageInfo.getRows();
		if(rows != null && rows.size() > 0){
			Set<Integer> teachers = rows.stream().map(e -> e.getId()).collect(Collectors.toSet());
			String teacherIds = StringUtils.join(teachers, ",");
			//获取小课数量
//			List<Map<Integer,Long>> numList =  classGroupDao.countTeacherVipNum(teacherIds);
			Map<Integer,Integer> vipNumMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(classGroupDao.countTeacherVipNum(teacherIds))),HashMap.class);
			//获取试听课数量
//			List<Map<Integer,Long>> demoNumList =  demoGroupDao.countTeacherDemoGroupNum(teacherIds);
			Map<Integer,Integer> demoNumMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertIntegerMap(demoGroupDao.countTeacherDemoGroupNum(teacherIds))),HashMap.class);

//			Set<Integer> organIds = rows.stream().map(e -> e.getOrganId()).collect(Collectors.toSet());
//			Map<Integer,String> organNames = MapUtil.convertMybatisMap(organizationDao.findOrganNameMap(StringUtils.join(organIds,",")));
			rows.forEach(e->{
//				e.setSubjectName(subjectDao.findBySubIds(e.getSubjectId()));
//				e.setOrganName(organNames.get(e.getOrganId()));
				if(StringUtils.isNotEmpty(e.getSplitSubjectName())){
					e.setSubjectName(Arrays.asList(e.getSplitSubjectName().split(",")));
				}
				Integer num = vipNumMap.get(e.getId());
				e.setVipNum(num == null?0:num);
				Integer demoNum = demoNumMap.get(e.getId());
				e.setDemoNum(demoNum == null?0:demoNum);
			});
		}
		return pageInfo;
	}

	@Override
	public Teacher getDetail(Integer id) {
		Teacher teacher = teacherDao.get(id);
		if(teacher != null){
			//获取老师教学点列表
			List<School> teacherSchools = schoolDao.findByUserId(id);
			teacher.setTeacherSchools(teacherSchools);
			//获取流动范围
			if(StringUtils.isNotEmpty(teacher.getFlowOrganRange())){
				teacher.setFlowOrganRangeId(teacher.getFlowOrganRange());
				List<String> organNames = organizationDao.findByOrganIds(teacher.getFlowOrganRange());
				teacher.setOrganName(organizationDao.get(teacher.getOrganId()).getName());
				teacher.setFlowOrganRange(StringUtils.join(organNames,","));
			}
			//专业技能
			if(StringUtils.isNotEmpty(teacher.getSubjectId())){
				teacher.setSubjectName(subjectDao.findBySubIds(teacher.getSubjectId()));
			}
		}
		return teacher;
	}

}
