package com.ym.mec.student.controller;

import com.huifu.adapay.model.payment.PayChannelEnum;
import com.huifu.adapay.model.payment.Payment;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.MusicGroupStudentFeeDao;
import com.ym.mec.biz.dal.dto.RegisterPayDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.thirdparty.adapay.Pay;
import com.ym.mec.util.string.IdWorker;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@RequestMapping("musicGroup")
@Api(tags = "乐团服务")
@RestController
public class MusicGroupController extends BaseController {

    @Resource
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private MusicGroupSubjectPlanService musicGroupSubjectPlanService;
    @Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    private MusicGroupSubjectGoodsGroupService musicGroupSubjectGoodsGroupService;
    @Autowired
    private GoodsService goodsService;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private PayService payService;
    @Autowired
    private MusicGroupStudentFeeDao musicGroupStudentFeeDao;
    @Autowired
    private IdGeneratorService idGeneratorService;
    @Autowired
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;

    @ApiOperation("获取学生所在乐团列表")
    @GetMapping(value = "/queryUserMusicGroups")
    public Object queryUserMusicGroups() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(musicGroupService.queryUserMusicGroups(sysUser.getId()));
    }

    @ApiOperation("获取学生个人中心乐团信息")
    @GetMapping(value = "/queryPersonalMusicGroups")
    public Object queryPersonalMusicGroups() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(musicGroupService.queryPersonalMusicGroups(sysUser.getId()));
    }

    @ApiOperation(value = "根据学员编号，获取有关联的老师列表")
    @GetMapping("/findTeachersByStuId")
    public Object findTeachersByStuId() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(musicGroupService.findTeachersByStuId(sysUser.getId()));
    }


    @ApiOperation(value = "获取乐团声部费用信息及乐器和辅件")
    @GetMapping("/getSubjectGoodsAndInfo")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "subjectId", value = "声部编号", required = true, dataType = "Integer")})
    public HttpResponseResult getSubjectGoodsAndInfo(String musicGroupId, Integer subjectId) {
        return succeed(musicGroupSubjectPlanService.getSubjectGoodsAndInfo(musicGroupId, subjectId));
    }

    @ApiOperation(value = "乐团报名缴费状态查询")
    @GetMapping("/getMusicGroupStatus")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String")})
    public HttpResponseResult getMusicGroupStatus(String musicGroupId) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Integer userId = sysUser.getId();
        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        if (musicGroup == null) {
            return failed("乐团不存在");
        }
        StudentRegistration studentRegistration = studentRegistrationService.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if (studentRegistration == null) {
            return failed(HttpStatus.CONTINUE, "乐团报名信息不存在");
        }
        if (studentRegistration.getPaymentStatus() != null && studentRegistration.getPaymentStatus().equals(PaymentStatusEnum.YES)) {
            return failed("您已缴费,请等待乐团开启");
        }
        if (!musicGroup.getStatus().equals(MusicGroupStatusEnum.PAY) && (studentRegistration.getPaymentStatus() == null || !studentRegistration.getPaymentStatus().equals(PaymentStatusEnum.OPEN))) {
            return failed("乐团还未开启缴费，请等待通知");
        }
        return succeed(studentRegistration);
    }

    @ApiOperation(value = "查询续费信息")
    @GetMapping("/queryRenewInfo")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String")})
    public HttpResponseResult queryRenewInfo(String musicGroupId) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Integer userId = sysUser.getId();
        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        if (musicGroup == null) {
            return failed("乐团不存在");
        }
        MusicGroupStudentFee musicGroupStudentFee = musicGroupStudentFeeDao.findByUser(userId, musicGroupId);

        if (musicGroupStudentFee == null) {
            throw new BizException("个人信息不存在");
        }
        if (musicGroupStudentFee.getPaymentStatus() != PaymentStatus.NON_PAYMENT) {
            throw new BizException("已缴费");
        }

        BigDecimal amount = musicGroupStudentFee.getTemporaryCourseFee();
        if (amount == null || amount.doubleValue() == 0) {
            amount = musicGroupStudentFee.getCourseFee();
        }

        StudentRegistration studentRegistration = studentRegistrationService.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if (studentRegistration == null) {
            return failed("乐团报名信息找不到");
        }
        MusicGroupSubjectPlan musicGroupSubjectPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(musicGroupId,
                studentRegistration.getActualSubjectId());
        ModelMap model = new ModelMap();
        model.put("musicGroup", musicGroup);
        model.put("amount", amount);
        model.put("musicGroupSubjectPlan", musicGroupSubjectPlan);

        return succeed(model);
    }

    @ApiOperation(value = "续费")
    @GetMapping("/renew")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String")})
    public HttpResponseResult renew(String musicGroupId) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Integer userId = sysUser.getId();
        return succeed(musicGroupService.renew(musicGroupId, userId));
    }

    @ApiOperation(value = "退团")
    @PostMapping("/quitMusicGroup")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String")})
    public HttpResponseResult quitMusicGroup(String musicGroupId) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        return succeed(musicGroupService.applyQuitMusicGroup(musicGroupId, sysUser));
    }


    @ApiOperation(value = "乐团报名支付")
    @PostMapping("/pay")
    @ApiImplicitParams({@ApiImplicitParam(name = "registerPayDto", value = "支付信息", required = true, dataType = "Integer")})
    public HttpResponseResult pay(@RequestBody RegisterPayDto registerPayDto) throws Exception {

        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Integer userId = sysUser.getId();
        StudentRegistration studentRegistration = studentRegistrationService.get(registerPayDto.getRegisterId().longValue());
        if (!studentRegistration.getUserId().equals(userId)) {
            return failed("报名信息有误，请核查");
        }

        StudentPaymentOrder ApplyOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId(), DealStatusEnum.SUCCESS);
        if (ApplyOrder != null) {
            return failed("您已支付成功，请勿重复支付");
        }

        //判断用户是否已存在订单
        ApplyOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId(), DealStatusEnum.ING);
        if (ApplyOrder != null) {
            return failed(HttpStatus.CONTINUE, "您有待支付的订单");
        }

        BigDecimal amount = registerPayDto.getAmount(); //前端获取的价格
        BigDecimal orderAmount = new BigDecimal("0");

        //获取课程价格
        MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId());
        BigDecimal courseFee = musicOneSubjectClassPlan.getFee();
        orderAmount = orderAmount.add(courseFee);


        //乐器及打包辅件
        List<MusicGroupSubjectGoodsGroup> goodsGroups = null;
        if (registerPayDto.getGoodsGroupIds() != null && !registerPayDto.getGoodsGroupIds().equals("")) {
            goodsGroups = musicGroupSubjectGoodsGroupService.findGoodsGroupByIds(registerPayDto.getGoodsGroupIds());
            for (MusicGroupSubjectGoodsGroup goodsGroup : goodsGroups) {
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT) && musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.LEASE)) {
                    orderAmount = orderAmount.add(musicOneSubjectClassPlan.getDepositFee());
                    continue;
                }
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT) && musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.FREE)) {
                    continue;
                }
                orderAmount = orderAmount.add(goodsGroup.getPrice());
                //团购乐器减免课程费用
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT) && goodsGroup.getRemissionCourseFee() != null && musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.GROUP)) {//团购
                    orderAmount = orderAmount.subtract(goodsGroup.getRemissionCourseFee());
                }
            }
        }

        //单独辅件
        List<Goods> goodsList = null;
        if (registerPayDto.getGoodsIds() != null && !registerPayDto.getGoodsIds().equals("")) {
            goodsList = goodsService.findGoodsByIds(registerPayDto.getGoodsIds());
            for (Goods goods : goodsList) {
                orderAmount = orderAmount.add(goods.getGroupPurchasePrice());
            }
        }

        //单独教谱
        List<Goods> otherGoodsList = null;
        if (registerPayDto.getOtherGoodsIds() != null && !registerPayDto.getOtherGoodsIds().equals("")) {
            otherGoodsList = goodsService.findGoodsByIds(registerPayDto.getOtherGoodsIds());
            for (Goods goods : otherGoodsList) {
                orderAmount = orderAmount.add(goods.getGroupPurchasePrice());
            }
        }
        if (amount.compareTo(orderAmount) != 0) {
            return failed("商品价格不符");
        }


        String orderNo = idGeneratorService.generatorId("payment") + "";

        Map payMap = payService.getPayMap(
                orderAmount,
                orderNo,
                "http://mstudev.dayaedu.com/api-student/studentOrder/notify",
                "http://mstudev.dayaedu.com/#/paymentresult?orderNo=" + orderNo,
                "测试订单",
                "测试订单");

        studentRegistrationService.addOrder(studentRegistration, amount, orderNo, (String) payMap.get("type"), courseFee, goodsGroups, goodsList, otherGoodsList);

        return succeed(payMap);
    }

    @ApiOperation(value = "乐团报名重新支付")
    @PostMapping("/rePay")
    @ApiImplicitParams({@ApiImplicitParam(name = "registerPayDto", value = "支付信息", required = true, dataType = "Integer")})
    public HttpResponseResult rePay(@RequestBody RegisterPayDto registerPayDto) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Integer userId = sysUser.getId();
        StudentRegistration studentRegistration = studentRegistrationService.get(registerPayDto.getRegisterId().longValue());
        if (!studentRegistration.getUserId().equals(userId)) {
            return failed("报名信息有误，请核查");
        }

        StudentPaymentOrder ApplyOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId().toString(), DealStatusEnum.SUCCESS);
        if (ApplyOrder != null) {
            return failed("您已支付成功，请勿重复支付");
        }

        //判断用户是否已存在订单
        ApplyOrder = studentPaymentOrderService.findMusicGroupApplyOrderByStatus(userId, studentRegistration.getMusicGroupId().toString(), DealStatusEnum.ING);
        if (ApplyOrder == null) {
            return failed(HttpStatus.CONTINUE, "没有支付中的订单，请勿非法请求");
        }

        BigDecimal amount = registerPayDto.getAmount(); //前端获取的价格
        BigDecimal orderAmount = new BigDecimal("0");

        //获取课程价格
        MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(studentRegistration.getMusicGroupId(), studentRegistration.getSubjectId());
        BigDecimal courseFee = musicOneSubjectClassPlan.getFee();
        orderAmount = orderAmount.add(courseFee);


        //乐器及打包辅件
        List<MusicGroupSubjectGoodsGroup> goodsGroups = null;
        if (registerPayDto.getGoodsGroupIds() != null && !registerPayDto.getGoodsGroupIds().equals("")) {
            goodsGroups = musicGroupSubjectGoodsGroupService.findGoodsGroupByIds(registerPayDto.getGoodsGroupIds());
            for (MusicGroupSubjectGoodsGroup goodsGroup : goodsGroups) {
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT) && musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.LEASE)) {
                    orderAmount = orderAmount.add(musicOneSubjectClassPlan.getDepositFee());
                    continue;
                }
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT) && musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.FREE)) {
                    continue;
                }

                orderAmount = orderAmount.add(goodsGroup.getPrice());
                //团购乐器减免课程费用
                if (goodsGroup.getType().equals(GoodsType.INSTRUMENT) && goodsGroup.getRemissionCourseFee() != null && musicOneSubjectClassPlan.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.GROUP)) {//团购
                    orderAmount = orderAmount.subtract(goodsGroup.getRemissionCourseFee());
                }
            }
        }

        //单独辅件
        List<Goods> goodsList = null;
        if (registerPayDto.getGoodsIds() != null && !registerPayDto.getGoodsIds().equals("")) {
            goodsList = goodsService.findGoodsByIds(registerPayDto.getGoodsIds());
            for (Goods goods : goodsList) {
                orderAmount = orderAmount.add(goods.getGroupPurchasePrice());
            }
        }

        //单独教谱
        List<Goods> otherGoodsList = null;
        if (registerPayDto.getOtherGoodsIds() != null && !registerPayDto.getOtherGoodsIds().equals("")) {
            otherGoodsList = goodsService.findGoodsByIds(registerPayDto.getOtherGoodsIds());
            for (Goods goods : otherGoodsList) {
                orderAmount = orderAmount.add(goods.getGroupPurchasePrice());
            }
        }
        if (amount.compareTo(orderAmount) != 0) {
            return failed("商品价格不符");
        }

        String orderNo = idGeneratorService.generatorId("payment") + "";

        Map payMap = payService.getPayMap(
                orderAmount,
                orderNo,
                "http://mstudev.dayaedu.com/api-student/studentOrder/notify",
                "http://mstudev.dayaedu.com/#/paymentresult?orderNo=" + orderNo,
                "测试订单",
                "测试订单");

        studentRegistrationService.reAddOrder(userId, amount, orderNo, (String) payMap.get("type"), courseFee, goodsGroups, goodsList, otherGoodsList, studentRegistration.getMusicGroupId(), ApplyOrder);

        return succeed(payMap);
    }

    @GetMapping("/test")
    public Object test() throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserByMobile("15829292961");


//        Map query = payService.query("1017201910111756231647562047");
//
//        System.out.println(query);
//
        String orderNo = idGeneratorService.generatorId("payment") + "";
        BigDecimal amount = new BigDecimal("0.01");
        String payChannel = PayChannelEnum.ALIPAY_QR.getCode();
        Map map = payService.getPayMap(amount, orderNo, "http://47.99.212.176:8000/studentOrder/notify", "https://baodiu.com", "测试订单", "测试订单");
        return succeed(map);
    }

    @ApiOperation(value = "订单状态查询")
    @GetMapping("/getOrderStatus")
    @ApiImplicitParams({@ApiImplicitParam(name = "orderNo", value = "订单号", required = true, dataType = "String")})
    public HttpResponseResult getOrderStatus(String orderNo) {
        if (StringUtils.isBlank(orderNo)) {
            return failed("请指定订单");
        }
        StudentPaymentOrder orderByOrderNo = studentPaymentOrderService.findOrderByOrderNo(orderNo);
        if (Objects.isNull(orderByOrderNo)) {
            return failed("未找到指定订单");
        }
        MusicGroup musicGroup = musicGroupService.get(orderByOrderNo.getMusicGroupId());
        List<Goods> goodsList = studentPaymentOrderDetailService.findApplyOrderGoods(orderByOrderNo.getId());
        HashMap<String, Object> orderDetail = new HashMap<>();
        orderDetail.put("order",orderByOrderNo);
        orderDetail.put("goods",goodsList);
        orderDetail.put("course",musicGroup.getCourseForm());

        return succeed(orderDetail);
    }
}
