package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dto.ClassGroup4MixDto;
import com.ym.mec.biz.dal.dto.HighClassGroupDto;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.entity.ClassGroupTeacherMapper;
import com.ym.mec.biz.dal.enums.SalarySettlementTypeEnum;
import com.ym.mec.biz.dal.page.queryMusicGroupCourseScheduleQueryInfo;
import com.ym.mec.biz.service.ClassGroupService;
import com.ym.mec.biz.service.ClassGroupTeacherMapperService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.QueryInfo;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RequestMapping("classGroup")
@Api(tags = "班级服务")
@RestController
public class ClassGroupController extends BaseController {

    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private ClassGroupTeacherMapperService classGroupTeacherMapperService;

    @ApiOperation(value = "新增单技班班级")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('classGroup/add')")
    public Object add(@RequestBody ClassGroup classGroup) throws Exception {
        return succeed(classGroupService.addClassGroup(classGroup));
    }

    @ApiOperation(value = "新增合奏班")
    @PostMapping("/addMixClass")
    @PreAuthorize("@pcs.hasPermissions('classGroup/addMixClass')")
    public Object addMixClass(@ApiParam(value = "乐团编号", required = true) @RequestParam String musicGroupId,
                              @ApiParam(value = "班级名称", required = true) String name,
                              @ApiParam(value = "班级编号,号分割", required = true) String classGroupIds) throws Exception {
        return succeed(classGroupService.addMixClassGroup(musicGroupId, name, classGroupIds));
    }

    @ApiOperation(value = "新增提高班")
    @PostMapping("/addHighClass")
    @PreAuthorize("@pcs.hasPermissions('classGroup/addHighClass')")
    @ApiParam(value = "乐团提高班json", required = true)
    public Object addHighClass(@RequestBody List<HighClassGroupDto> highClassGroupDtoList) throws Exception {
        if (highClassGroupDtoList.size() <= 0) {
            return failed("参数不合法");
        }
        return succeed(classGroupService.addHighClassGroup(highClassGroupDtoList));
    }

    @ApiOperation(value = "删除单技班")
    @PostMapping("/delSingle")
    @PreAuthorize("@pcs.hasPermissions('classGroup/delSingle')")
    public Object delSingle(Integer classGroupId) {
        classGroupService.delSingle(classGroupId);
        return succeed();
    }

    @ApiOperation(value = "删除合奏班")
    @PostMapping("/delMix")
    @PreAuthorize("@pcs.hasPermissions('classGroup/delMix')")
    public Object delMix(Integer classGroupId) {
        classGroupService.delMix(classGroupId);
        return succeed();
    }

    @ApiOperation(value = "修改班级")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('classGroup/update')")
    public Object update(ClassGroup classGroup) {
        classGroup.setUpdateTime(new Date());
        classGroupService.update(classGroup);
        return succeed();
    }

    @ApiOperation(value = "分页查询班级列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('classGroup/queryPage')")
    public Object queryPage(QueryInfo queryInfo) {
        return succeed(classGroupService.queryPage(queryInfo));
    }

    @ApiOperation(value = "合奏班相关班级获取")
    @GetMapping("/findClassGroupAboutMix")
    @PreAuthorize("@pcs.hasPermissions('classGroup/findClassGroupAboutMix')")
    public HttpResponseResult findClassGroupAboutMix(@ApiParam(value = "乐团编号", required = true) @RequestParam String musicGroupId,
                                                     @ApiParam(value = "班级编号", required = false) Integer mixClassGroupId) {
        return succeed(classGroupService.findClassGroup(musicGroupId, mixClassGroupId));
    }

    @ApiOperation(value = "乐团单技班列表")
    @GetMapping("/findMusicGroupClass")
    @PreAuthorize("@pcs.hasPermissions('classGroup/findMusicGroupClass')")
    public HttpResponseResult findMusicGroupClass(@ApiParam(value = "乐团编号", required = true) @RequestParam String musicGroupId) {
        return succeed(classGroupService.findAllNormalClassGroupByMusicGroupId(musicGroupId));
    }

    @ApiOperation(value = "获取未分班的单技班列表")
    @GetMapping("/findNoClassSubjects")
    @PreAuthorize("@pcs.hasPermissions('classGroup/findNoClassSubjects')")
    public HttpResponseResult findNoClassSubjects(@ApiParam(value = "乐团编号", required = true) @RequestParam String musicGroupId) {
        return succeed(classGroupService.findNoClassSubjects(musicGroupId));
    }

    @ApiOperation(value = "乐团合奏班列表")
    @GetMapping("/findMixMusicGroupClass")
    @PreAuthorize("@pcs.hasPermissions('classGroup/findMixMusicGroupClass')")
    public HttpResponseResult findMixMusicGroupClass(@ApiParam(value = "乐团编号", required = true) @RequestParam String musicGroupId) {
        return succeed(classGroupService.findAllMixClassGroupByMusicGroupId(musicGroupId));
    }

    @ApiOperation(value = "乐团所有班级列表")
    @GetMapping("/findAllClassGroupByMusicGroup")
    @PreAuthorize("@pcs.hasPermissions('classGroup/findAllClassGroupByMusicGroup')")
    public HttpResponseResult findAllClassGroupByMusicGroup(@ApiParam(value = "乐团编号", required = true) @RequestParam String musicGroupId) {
        return succeed(classGroupService.findAllClassGroupByMusicGroup(musicGroupId));
    }

    @ApiOperation(value = "乐团详情--班级详情列表")
    @GetMapping("/queryMusicGroupClassGroup")
    @PreAuthorize("@pcs.hasPermissions('classGroup/queryMusicGroupClassGroup')")
    public HttpResponseResult queryMusicGroupClassGroup(QueryInfo queryInfo) {
        return succeed(classGroupService.queryMusicGroupClassGroup(queryInfo));
    }

    @ApiOperation(value = "乐团详情--课表详情列表（课酬调整共用）")
    @GetMapping("/queryMusicGroupCourseSchedule")
    @PreAuthorize("@pcs.hasPermissions('classGroup/queryMusicGroupCourseSchedule')")
    public HttpResponseResult queryMusicGroupCourseSchedule(queryMusicGroupCourseScheduleQueryInfo queryInfo) {
        return succeed(classGroupService.queryMusicGroupCourseSchedule(queryInfo));
    }

    @ApiOperation(value = "乐团班级老师设置")
    @PostMapping("/addClassGroupTeacher")
    @ApiParam(value = "乐团班级老师json", required = true)
    @PreAuthorize("@pcs.hasPermissions('classGroup/addClassGroupTeacher')")
    public HttpResponseResult addClassGroupTeacher(@RequestBody List<ClassGroupTeacherMapper> classGroupTeacherMapperList) {
        if (classGroupTeacherMapperList.size() <= 0) {
            return failed("参数不合法");
        }
        return succeed(classGroupTeacherMapperService.classGroupTeachersInsert(classGroupTeacherMapperList));
    }

    @ApiOperation(value = "获取乐团班级老师")
    @GetMapping("/findMusicGroupClassTeacher")
    @PreAuthorize("@pcs.hasPermissions('classGroup/findMusicGroupClassTeacher')")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "classGroupId", value = "班级编号", required = false, dataType = "Integer")})
    public HttpResponseResult findMusicGroupClassTeacher(String musicGroupId, Integer classGroupId) {
        return succeed(classGroupService.getClassGroupAndTeachers(musicGroupId, "NORMAL,MIX", classGroupId));
    }

    @ApiOperation(value = "获取乐团班级老师课酬")
    @GetMapping("/findMusicGroupClassTeacherSalary")
    @PreAuthorize("@pcs.hasPermissions('classGroup/findMusicGroupClassTeacherSalary')")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "结算类型（1-基准课酬，4-梯度课酬）", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "classGroupId", value = "班级编号", required = false, dataType = "Integer")})
    public HttpResponseResult findMusicGroupClassTeacherSalary(String musicGroupId, SalarySettlementTypeEnum type, Integer classGroupId) {
        try {
            return succeed(classGroupService.getClassGroupAndTeacherSalary(musicGroupId, type, classGroupId));
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @ApiOperation(value = "乐团班级老师课酬确认")
    @PostMapping("/setClassGroupTeacherSalary")
    @PreAuthorize("@pcs.hasPermissions('classGroup/setClassGroupTeacherSalary')")
    @ApiParam(value = "乐团班级老师<包含相应课酬>json", required = true)
    public Object setClassGroupTeacherSalary(@RequestBody List<ClassGroupTeacherMapper> classGroupTeacherMapperList) throws Exception {
        if (classGroupTeacherMapperList.size() <= 0) {
            return failed("参数不合法");
        }
        return succeed(classGroupTeacherMapperService.classGroupTeacherMapperUpdate(classGroupTeacherMapperList));
    }

    @ApiOperation(value = "乐团班级设置，成团确认")
    @PostMapping("/addMusicGroupTeam")
    @PreAuthorize("@pcs.hasPermissions('classGroup/addMusicGroupTeam')")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "teacherId", value = "老师编号", required = true, dataType = "Integer")})
    public Object addMusicGroupTeam(Integer teacherId, String musicGroupId, Integer improventClassesNum) throws Exception {
        if (teacherId == null || StringUtils.isEmpty(musicGroupId)) {
            return failed("参数校验错误");
        }
        classGroupService.addMusicGroupTeam(teacherId, musicGroupId, improventClassesNum);
        return succeed();
    }


    @ApiOperation(value = "调整班级(添加班级)")
    @PostMapping("/revisionAddClassGroup")
    @PreAuthorize("@pcs.hasPermissions('classGroup/revisionAddClassGroup')")
    @ApiImplicitParams({@ApiImplicitParam(name = "ClassGroup4MixDto", value = "添加班级结构", required = true, dataType = "String")})
    public HttpResponseResult revisionAddClassGroup(@RequestBody ClassGroup4MixDto classGroup4MixDto) throws Exception {
        return succeed(classGroupService.classGroupAdjust(classGroup4MixDto));
    }

    @ApiOperation(value = "调整班级(临时调整)")
    @PostMapping("/revisionClassGroup")
    @PreAuthorize("@pcs.hasPermissions('classGroup/revisionClassGroup')")
    @ApiImplicitParams({@ApiImplicitParam(name = "ClassGroup4MixDto", value = "添加班级结构", required = true, dataType = "String")})
    public HttpResponseResult revisionClassGroup(@RequestBody ClassGroup4MixDto classGroup4MixDto) throws Exception {
        return succeed(classGroupService.classGroupSnap(classGroup4MixDto));
    }

    @ApiOperation(value = "小班报名详情")
    @GetMapping("/highClassGroups")
    @PreAuthorize("@pcs.hasPermissions('classGroup/highClassGroups')")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String")})
    public HttpResponseResult highClassGroups(String musicGroupId) throws Exception {
        return succeed(classGroupService.highClassGroupList(musicGroupId));
    }

    @ApiOperation(value = "合奏子班列表（班级调整）")
    @GetMapping("/classGroupAndTeacher")
    @PreAuthorize("@pcs.hasPermissions('classGroup/classGroupAndTeacher')")
    @ApiImplicitParams({@ApiImplicitParam(name = "classGroupId", value = "班级id", required = true, dataType = "int")})
    public HttpResponseResult classGroupAndTeacher(Integer classGroupId) throws Exception {
        return succeed(classGroupService.classGroupAndTeacher(classGroupId));
    }

}