package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.School;
import com.ym.mec.biz.dal.page.SchoolQueryInfo;
import com.ym.mec.biz.service.SchoolService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

@RequestMapping("school")
@Api(tags = "学校(教学点)服务")
@RestController
public class SchoolController extends BaseController {

    @Autowired
    private SchoolService schoolService;

    @ApiOperation(value = "新增学校")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('school/add')")
    public Object add(School school){
        schoolService.insert(school);
        return succeed();
    }

    @ApiOperation(value = "删除学校")
    @PostMapping("/del/{id}")
    @PreAuthorize("@pcs.hasPermissions('school/del')")
    public Object del(@ApiParam(value = "学校编号", required = true) @PathVariable("id") Integer id){
        schoolService.delete(id);
        return succeed();
    }

    @ApiOperation(value = "修改学校")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('school/update')")
    public Object update(School school){
        school.setUpdateTime(new Date());
        schoolService.update(school);
        return succeed();
    }

    @ApiOperation(value = "根据学校编号查询学校")
    @GetMapping("/get/{id}")
    @PreAuthorize("@pcs.hasPermissions('school/get')")
    public Object get(@ApiParam(value = "学校编号", required = true) @PathVariable("id") Integer id){
        return succeed(schoolService.get(id));
    }

    @ApiOperation(value = "分页查询学校列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('school/queryPage')")
    public Object queryPage(SchoolQueryInfo queryInfo){
        return succeed(schoolService.queryPage(queryInfo));
    }

    @ApiOperation(value = "根据机构编号获取学校列表")
    @GetMapping("/queryByOrganId")
    @PreAuthorize("@pcs.hasPermissions('school/queryByOrganId')")
    public Object queryByOrganId(@RequestParam Integer organId){
        return succeed(schoolService.queryByOrganId(organId));
    }

    @ApiOperation(value = "获取教师vip课教学点")
    @GetMapping("/findVipSchoolByTeacher")
    public Object findVipSchoolByTeacher(@ApiParam(value = "教师编号", required = true) Integer userId){
        return succeed(schoolService.findVipSchoolByTeacher(userId,null));
    }
}
