package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.service.StudentManageService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.QueryInfo;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "学生管理")
@RequestMapping("studentManage")
@RestController
public class StudentManageController extends BaseController {

    @Autowired
    private StudentManageService studentManageService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private EmployeeDao employeeDao;

    @ApiOperation(value = "获取学生列表")
    @GetMapping("/queryStudentList")
    @PreAuthorize("@pcs.hasPermissions('studentManage/queryStudentList')")
    public Object queryStudentList(StudentManageQueryInfo queryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        if(!sysUser.getIsSuperAdmin()){
            if(StringUtils.isEmpty(queryInfo.getOrganId())){
                Employee employee = employeeDao.get(sysUser.getId());
                queryInfo.setOrganId(employee.getOrganIdList());
            }
        }
        return succeed(studentManageService.findStudentsByOrganId(queryInfo));
    }

    @ApiOperation(value = "乐团管理--乐团详情--获取学员列表")
    @GetMapping("/queryMusicGroupStudent")
    @PreAuthorize("@pcs.hasPermissions('studentManage/queryMusicGroupStudent')")
    public Object queryMusicGroupStudent(MusicGroupStudentQueryInfo queryInfo){
        return succeed(studentManageService.queryMusicGroupStudent(queryInfo));
    }

    @ApiOperation(value = "乐团管理--乐团详情--学员列表--查看班级")
    @GetMapping("/queryStudentClassGroup")
    @PreAuthorize("@pcs.hasPermissions('studentManage/queryStudentClassGroup')")
    @ApiImplicitParams({@ApiImplicitParam(name = "teacherId", value = "学生编号", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String")})
    public Object queryStudentClassGroup(String musicGroupId,Integer teacherId){
        if(StringUtils.isEmpty(musicGroupId) || teacherId == null){
            return failed("参数校验失败");
        }
        return succeed(studentManageService.queryStudentClassGroup(musicGroupId,teacherId));
    }

    @ApiOperation(value = "乐团管理--乐团详情--学员列表，汇总数据接口")
    @GetMapping("/musicGroupStudentsSum")
    @PreAuthorize("@pcs.hasPermissions('studentManage/musicGroupStudentsSum')")
    public Object musicGroupStudentsSum(String musicGroupId){
        return succeed(studentManageService.musicGroupStudentsSum(musicGroupId));
    }

    @ApiOperation(value = "乐团管理--乐团详情--点名总览")
    @GetMapping("/queryMusicGroupStudentsSign")
    @PreAuthorize("@pcs.hasPermissions('studentManage/queryMusicGroupStudentsSign')")
    public Object queryMusicGroupStudentsSign(QueryInfo queryInfo){
        return succeed(studentManageService.queryMusicGroupStudentsSign(queryInfo));
    }

    @ApiOperation(value = "根据学生ID获取学生基本报名信息")
    @GetMapping("/findStudentBaseInfo")
    @PreAuthorize("@pcs.hasPermissions('studentManage/findStudentBaseInfo')")
    public Object findStudentBaseInfo(@ApiParam(value = "学生编号", required = true) Integer userId){
        return succeed(studentManageService.findStudentManageBaseInfo(userId));
    }

    @ApiOperation(value = "根据学生ID获取其所在的乐团")
    @GetMapping("/findStudentMusicGroups")
    @PreAuthorize("@pcs.hasPermissions('studentManage/findStudentMusicGroups')")
    public Object findStudentMusicGroups(@ApiParam(value = "学生编号", required = true) Integer userId){
        return succeed(studentManageService.findStudentMusicGroupsByUserId(userId));
    }

    @ApiOperation(value = "根据乐团获取排课列表")
    @GetMapping("/findStudentCourses")
    @PreAuthorize("@pcs.hasPermissions('studentManage/findStudentCourses')")
    public Object findStudentCourses(StudentManageCourseQueryInfo queryInfo){
        return succeed(studentManageService.findStudentCourseList(queryInfo));
    }

    @ApiOperation(value = "获取学生签到列表")
    @GetMapping("/findStudentAttendances")
    @PreAuthorize("@pcs.hasPermissions('studentManage/findStudentAttendances')")
    public Object findStudentAttendances(StudentManageAttendanceQueryInfo queryInfo){
        return succeed(studentManageService.findStudentAttendances(queryInfo));
    }

    @ApiOperation(value = "获取学生vip课")
    @GetMapping("/findStudentVipGroups")
    @PreAuthorize("@pcs.hasPermissions('studentManage/findStudentVipGroups')")
    public Object findStudentVipGroups(Integer userId){
        return succeed(studentManageService.findStudentVipGroups(userId));
    }

    @ApiOperation(value = "获取用户默认账户基本信息")
    @GetMapping("/getUserCashAccountBaseInfo")
    @PreAuthorize("@pcs.hasPermissions('studentManage/getUserCashAccountBaseInfo')")
    public Object getUserCashAccountBaseInfo(Integer userId){
        return succeed(studentManageService.getStudentAccountBaseInfo(userId));
    }

    @ApiOperation(value = "获取学员扣费记录列表")
    @GetMapping("/queryStudentPayment")
    @PreAuthorize("@pcs.hasPermissions('studentManage/queryStudentPayment')")
    public Object queryStudentPayment(TeacherPaymentRecordInfo queryInfo){
        return succeed(studentManageService.queryStudentPayment(queryInfo));
    }
}
