package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.page.StudentPaymentOrderQueryInfo;
import com.ym.mec.biz.service.StudentPaymentOrderDetailService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RequestMapping("order")
@Api(tags = "订单服务")
@RestController
public class StudentPaymentOrderController extends BaseController {

    @Value("${payment.hiddenMode}")
    private Boolean hiddenMode;

    @Value("${payment.channel}")
    private String channel;

    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private EmployeeDao employeeDao;

    @ApiOperation(value = "获取订单列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('order/queryPage')")
    public Object queryOrderPage(StudentPaymentOrderQueryInfo queryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        if(!sysUser.getIsSuperAdmin()){
            if(StringUtils.isEmpty(queryInfo.getOrganId())){
                Employee employee = employeeDao.get(sysUser.getId());
                queryInfo.setOrganId(employee.getOrganIdList());
            }
        }
        if(hiddenMode){
            queryInfo.setPaymentChannel(channel);
        }
        return succeed(studentPaymentOrderService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取乐器采购清单")
    @GetMapping("/getMusicalList")
    @PreAuthorize("@pcs.hasPermissions('order/getMusicalList')")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String")})
    public Object getMusicalList(String musicGroupId){
        return succeed(studentPaymentOrderDetailService.getMusicalList(musicGroupId));
    }

    @ApiOperation(value = "乐器清单确认")
    @GetMapping("/verifyMusicalList")
    @PreAuthorize("@pcs.hasPermissions('order/verifyMusicalList')")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String")})
    public Object verifyMusicalListMusicalList(String musicGroupId){
        return succeed(studentPaymentOrderDetailService.verifyMusicalList(musicGroupId));
    }


}
