package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dto.ClassDateAdjustDto;
import com.ym.mec.biz.dal.dto.VipGroupApplyBaseInfoDto;
import com.ym.mec.biz.dal.dto.VipGroupApplyDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.StudentApplyRefunds.StudentApplyRefundsStatus;
import com.ym.mec.biz.dal.enums.AuditStatusEnum;
import com.ym.mec.biz.dal.page.VipGroupAttendanceQueryInfo;
import com.ym.mec.biz.dal.page.VipGroupQueryInfo;
import com.ym.mec.biz.dal.page.VipGroupSalaryQueryInfo;
import com.ym.mec.biz.dal.page.VipGroupTeachingRecordQueryInfo;
import com.ym.mec.biz.service.CourseScheduleService;
import com.ym.mec.biz.service.StudentApplyRefundsService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.biz.service.VipGroupService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Author Joburgess
 * @Date 2019/9/21
 */
@Api(tags = "vip课")
@RequestMapping("vipGroupManage")
@RestController
public class VipGroupManageController extends BaseController {

    @Autowired
    private VipGroupService vipGroupService;
    @Autowired
    private CourseScheduleService scheduleService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private EmployeeDao employeeDao;

    @Autowired
    private StudentApplyRefundsService studentApplyRefundsService;
    
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    
    @ApiOperation(value = "vip课申请")
    @PostMapping("/vipGroupApply")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/vipGroupApply')")
    public Object vipGroupApply(@RequestBody VipGroupApplyDto vipGroupApplyDto){
        SysUser sysUser = sysUserFeignService.queryUserById(vipGroupApplyDto.getVipGroupApplyBaseInfo().getUserId());
        if(Objects.isNull(sysUser)){
            return failed("请指定指导老师!");
        }
        vipGroupApplyDto.getVipGroupApplyBaseInfo().setOrganIdList(sysUser.getOrganId().toString());
        vipGroupService.createVipGroup(vipGroupApplyDto);
        return succeed();
    }

    @ApiOperation(value = "全查询")
    @GetMapping("/queryAll")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/queryAll')")
    public Object queryAll(VipGroupQueryInfo queryInfo) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        if(!sysUser.getIsSuperAdmin()){
            if(StringUtils.isEmpty(queryInfo.getOrganId())){
                Employee employee = employeeDao.get(sysUser.getId());
                queryInfo.setOrganId(employee.getOrganIdList());
            }
        }
        return succeed(vipGroupService.findVipGroups(queryInfo));
    }


    @ApiOperation(value = "获取小课学员")
    @GetMapping(value = "/findVipGroupStudents")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupStudents')")
    public Object findVipGroupStudents(VipGroupQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupStudents(queryInfo));
    }

    @ApiOperation(value = "vip课详情")
    @GetMapping("/getVipGroupDetail")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getVipGroupDetail')")
    public Object getVipGroupDetail(Long vipGroupId){
        return succeed(vipGroupService.getVipGroupDetail(vipGroupId));
    }

    @ApiOperation(value = "获取vip课考勤记录")
    @PostMapping("/getVipGroupAttendances")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getVipGroupAttendances')")
    public Object getVipGroupAttendances(@RequestBody VipGroupAttendanceQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupAttendances(queryInfo));
    }

    @ApiOperation(value = "退课申请")
    @PostMapping("/applyRefundForStudent")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/applyRefundForStudent')")
    public Object applyRefundForStudent(Long vipGroupId,Integer studentId){
        vipGroupService.applyRefund(vipGroupId,studentId);
        return succeed();
    }

    @ApiOperation(value = "退课申请审核")
    @PostMapping("/applyRefundAudit")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/applyRefundAudit')")
	public Object applyRefundAudit(Long id, StudentApplyRefundsStatus status, String remark, BigDecimal amount) {
		vipGroupService.applyRefundAudit(id, status, remark, amount);
		return succeed();
	}

    @ApiOperation(value = "查询退费详情")
    @GetMapping("/queryApplyRefundDetail")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/queryApplyRefundDetail')")
	public Object queryApplyRefundDetail(Long id) {
		StudentApplyRefunds studentApplyRefunds = studentApplyRefundsService.get(id);
		if (studentApplyRefunds == null) {
			return failed("参数输入有误");
		}

		Long orderId = studentApplyRefunds.getOrigPaymentOrderId();

		StudentPaymentOrder paymentOrder = studentPaymentOrderService.get(orderId);
		if (paymentOrder == null) {
			return failed("支付订单信息找不到");
		}
		String vipGroupId = paymentOrder.getMusicGroupId();
		VipGroup vipGroup = vipGroupService.get(Long.parseLong(vipGroupId));
		if (vipGroup == null) {
			return failed("vip课信息找不到");
		}

		SysUser student = sysUserFeignService.queryUserById(studentApplyRefunds.getUserId());

		SysUser teacher = sysUserFeignService.queryUserById(vipGroup.getUserId());

		ModelMap model = new ModelMap();
		model.put("student", student.getUsername());
		model.put("teacher", teacher.getUsername());
		model.put("className", vipGroup.getName());
		model.put("studentApplyRefunds", studentApplyRefunds);

		return succeed(model);
	}

    @ApiOperation(value = "获取VIP课教学记录")
    @GetMapping("/findVipGroupTeachingRecord")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupTeachingRecord')")
    public Object findVipGroupTeachingRecord(VipGroupTeachingRecordQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupTeachingRecord(queryInfo));
    }

    @ApiOperation(value = "获取vip课基本信息")
    @GetMapping("/findTeachingRecordBaseInfo")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findTeachingRecordBaseInfo')")
    public Object findTeachingRecordBaseInfo(Long vipGroupId){
        return succeed(vipGroupService.findTeachingRecordBaseInfo(vipGroupId));
    }

    @ApiOperation("课酬总费用")
    @PostMapping("/getVipGroupCostCount")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getVipGroupCostCount')")
    public Object getVipGroupCostCount(VipGroupApplyBaseInfoDto vipGroup){
        Map results = vipGroupService.countVipGroupPredictFee(vipGroup,
                vipGroup.getUserId(),
                vipGroup.getOnlineClassesUnitPrice(),
                vipGroup.getOfflineClassesUnitPrice(),
                true,
                false);
        return succeed(results);
    }

    @ApiOperation(value = "停止vip课")
    @PostMapping("/stopVipGroup")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/stopVipGroup')")
    public Object stopVipGroup(Long vipGroupId,String stopReason){
        if(Objects.isNull(vipGroupId)|| StringUtils.isBlank(stopReason)){
            return failed("请指定vip课并指定取消原因");
        }
        vipGroupService.stopGroup(vipGroupId,stopReason);
        return succeed();
    }

    @ApiOperation(value = "获取vip课财务信息")
    @GetMapping("/findVipGroupSalarys")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupSalarys')")
    public Object findVipGroupSalarys(VipGroupSalaryQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupSalarys(queryInfo));
    }

    @ApiOperation(value = "获取当前课程上课学员")
    @GetMapping("/findVipGroupAttendanceStudents")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupAttendanceStudents')")
    public Object findVipGroupStudents(Long courseScheduleId){
        return succeed(vipGroupService.findVipGroupAttendanceStudents(courseScheduleId));
    }

    @ApiOperation(value = "课时调整")
    @PostMapping(value = "/classStartDateAdjust")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/classStartDateAdjust')")
    public Object classStartDateAdjust(ClassDateAdjustDto classDateAdjustDto){
        if(Objects.isNull(classDateAdjustDto.getId())){
            return failed(HttpStatus.FORBIDDEN, "请指定课程!");
        }
        List<CourseSchedule> courseSchedules=new ArrayList<>();
        courseSchedules.add(classDateAdjustDto);
        scheduleService.courseAdjust(courseSchedules);
        return succeed();
    }

    @ApiOperation(value = "vip课审核")
    @PostMapping(value = "/vipApplyAudit")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/vipApplyAudit')")
    public Object vipApplyAudit(Long vipGroupId, AuditStatusEnum auditStatus, String reason){
        vipGroupService.vipApplyAudit(vipGroupId,auditStatus,reason);
        return succeed();
    }

    @ApiOperation(value = "更新vip课基本信息")
    @PostMapping(value = "/updateVipBaseInfo")
    public Object updateVipBaseInfo(@RequestBody VipGroupApplyBaseInfoDto vipGroupApplyBaseInfo){
        vipGroupService.updateVipGroupBaseInfo(vipGroupApplyBaseInfo);
        return succeed();
    }

    @ApiOperation(value = "新增vip课程计划")
    @PostMapping(value = "/appendVipGroupCourseSchedules")
    public Object appendVipGroupCourseSchedules(@RequestBody VipGroupApplyDto vipGroupApplyDto){
        vipGroupService.appendVipGroupCourseSchedules(vipGroupApplyDto);
        return succeed();
    }
}
